@@ -1,4 +1,4 @@
-/* $Id: about.cpp,v 1.74 2004/09/25 19:46:33 ydirson Exp $ */
+/* $Id: about.cpp,v 1.94 2005/01/25 00:33:04 isaaccp Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -11,6 +11,7 @@
    See the COPYING file for more details.
 */
 
+#include "global.hpp"
 #include "events.hpp"
 #include "font.hpp"
 #include "game_config.hpp"
@@ -28,7 +29,7 @@ namespace about
 {
 
 std::vector<std::string> get_text() {
-	static const std::string credits[] = {
+	static const char *credits[] = {
 		" ",
 		"- ",
 		" ",
@@ -42,34 +43,37 @@ std::vector<std::string> get_text() {
 		" ",
 		"- ",
 
-		_("+Core Developers"),
-		_("-   Main Developer"),
+		"_" N_("+Core Developers"),
+		"_" N_("-   Main Developer"),
 		"   David White (Sirp)",
 		"- ",
-		_("-   Artwork and graphics designer"),
+		"_" N_("-   Artwork and graphics designer"),
 		"   Francisco Muñoz (fmunoz)",
 		"+ ",
 
-		_("+Developers"),
+		"_" N_("+Developers"),
 		"-   Alfredo Beaumont (ziberpunk)",
+		"-   Cedric Duval",
 		"-   Cyril Bouthors (CyrilB)",
 		"-   Guillaume Melquiond (silene)",
 		"-   Isaac Clerencia",
+		"-   Jan Zvánovec (jaz)",
 		"-   John B. Messerly",
 		"-   Justin Zaun (jzaun)",
 		"-   J.R. Blain (Cowboy)",
 		"-   Kristoffer Erlandsson (erl)",
+		"-   Maksim Orlovich (SadEagle)",
 		"-   Philippe Plantier (Ayin)",
 		"-   Yann Dirson",
 		"-   Zas",
 		"+ ",
 
-		_("+General Purpose Administrators"),
-		"-   Cedric Duval",
+		"_" N_("+General Purpose Administrators"),
 		"-   Crossbow/Miyo",
+		"-   Isaac Clerencia",
 		"+ ",
 
-		_("+Artwork and graphics designers"),
+		"_" N_("+Artwork and graphics designers"),
 		"-   Andrew James Patterson (Kamahawk)",
 		"-   antwerp",
 		"-   Christophe Anjard",
@@ -86,21 +90,24 @@ std::vector<std::string> get_text() {
 		"-   Joseph Simmons (Turin)",
 		"-   J.W. Bjerk (Eleazar)",
 		"-   J.W.C. McNabb (Darth Fool)",
+		"-   Michael Gil de Muro (grp21)",
 		"-   Neorice",
+		"-   Peter Geinitz (Shadow)",
 		"-   Richard Kettering (Jetryl)",
 		"-   Slainte",
 		"-   Svetac",
 		"+ ",
 
-		_("+Music"),
+		"_" N_("+Music"),
 		"-   Aleksi",
 		"-   Fredrik Lindroth",
 		"-   Joseph Toscano (zhaymusic.com)",
 		"-   Pau Congost",
 		"+ ",
 
-		_("+Scenario Designers"),
+		"_" N_("+Scenario Designers"),
 		"-   Benjamin Drieu",
+		"-   Dacyn",
 		"-   David White (Sirp)",
 		"-   Francisco Muñoz (fmunoz)",
 		"-   James Spencer (Shade)",
@@ -108,34 +115,52 @@ std::vector<std::string> get_text() {
 		"-   Justin Zaun (jzaun)",
 		"+ ",
 
-		_("+Packagers"),
+		"_" N_("+Multiplayer Maps"),
+		"-   Peter Groen (pg)",
+		"-   Tom Chance (telex4)",
+		"+ ",
+
+		"_" N_("+Packagers"),
+		"-   Cyril Bouthors (CyrilB)",
 		"-   Darryl Dixon",
+		"-   edge",
+		"-   Isaac Clerencia",
 		"-   Jay Hopping",
 		"-   Marcin Konicki (ahwayakchih)",
 		"-   Marcus Phillips (Sithrandel)",
 		"-   Mark Michelsen (skovbaer)",
 		"+ ",
 
-		_("+Miscellaneous"),
+		"_" N_("+Miscellaneous"),
 		"-   Francesco Gigli (Jaramir)",
 		"-   Jordà Polo (ettin)",
-		"-   Tom Chance (telex4)",
 		"+ ",
 
-		_("+Internationalization Manager"),
+		"_" N_("+Internationalization Managers"),
+		"-   Cédric Duval",
+		"-   Isaac Clerencia",
 		"-   Mark Michelsen (skovbaer)",
+		"-   Susanna Björverud (sanna)",
 		"+ ",
 
-		_("+Basque Translation"),
+		"_" N_("+Basque Translation"),
 		"-   Alfredo Beaumont (ziberpunk)",
+		"-   Julen Landa (genars)",
 		"+ ",
 
-		_("+Catalan Translation"),
+		"_" N_("+Bulgarian Translation"),
+		"-   Anton Tsigularov (Atilla)",
+		"+ ",
+
+		"_" N_("+Catalan Translation"),
+		"-   Carles Company (brrr)",
 		"-   Dan Rosàs Garcia (focks)",
 		"-   Jordà Polo (ettin)",
+		"-   Mark Recasens",
+		"-   Pau Rul·lan Ferragut",
 		"+ ",
 
-		_("+Czech Translation"),
+		"_" N_("+Czech Translation"),
 		"-   David Nečas (Yeti)",
 		"-   Mintaka",
 		"-   Petr Kopač (Ferda)",
@@ -144,21 +169,21 @@ std::vector<std::string> get_text() {
 		"-   Vít Krčál",
 		"+ ",
 
-		_("+Danish Translation"),
+		"_" N_("+Danish Translation"),
 		"-   Mark Michelsen (skovbaer)",
 		"-   Mathias Bundgaard Svensson (freaken)",
 		"+ ",
 
-		_("+Dutch Translation"),
+		"_" N_("+Dutch Translation"),
 		"-   Lala",
 		"-   Pieter Vermeylen (Onne)",
 		"+ ",
 
-		_("+Finnish Translation"),
+		"_" N_("+Finnish Translation"),
 		"-   paxed",
 		"+ ",
 
-		_("+French Translation"),
+		"_" N_("+French Translation"),
 		"-   Benoit Astruc",
 		"-   Cédric Duval",
 		"-   Guillaume Duwelz-Rebert",
@@ -170,41 +195,59 @@ std::vector<std::string> get_text() {
 		"-   Zas",
 		"+ ",
 
-		_("+German Translation"),
-		"-   Andre Schmidt",
-		"-   Boris Stumm",
-		"-   Christoph Berg",
-		"-   Jan Heiner",
-		"-   Kai Ensenbach",
-		"-   Nils Kneuper",
-		"-   Stephan Grochtmann",
+		"_" N_("+German Translation"),
+		"-   Andre Schmidt (schmidta)",
+		"-   Boris Stumm (quijote_)",
+		"-   Christoph Berg (chrber)",
+		"-   Gerfried Fuchs (Rhonda)",
+		"-   Jan Heiner Laberenz (jan-heiner)",
+		"-   Kai Ensenbach (Pingu)",
+		"-   Nils Kneuper (Ivanovic)",
+		"-   Stephan Grochtmann (Schattenstephan)",
 		"+ ",
 
-		_("+Hungarian Translation"),
+		"_" N_("+Greek Translation"),
+		"-   Katerina Sykioti",
+		"-   Konstantinos Karasavvas",
+		"+ ",
+
+		"_" N_("+Hungarian Translation"),
+		"-   adson",
+		"-   dentro",
+		"-   Kékkői László (BlackEvil)",
 		"-   Khiraly",
+		"-   krix",
 		"-   Széll Tamás (TomJoad)",
 		"+ ",
 
-		_("+Italian Translation"),
+		"_" N_("+Italian Translation"),
+		"-   Alessio D'Ascanio (otaku)",
+		"-   Americo Iacovizzi (DarkAmex)",
 		"-   crys0000",
 		"-   Federico Tomassetti",
 		"-   isazi",
 		"-   RokStar",
 		"+ ",
+		
+		"_" N_("+Latin Translation"),
+		"-   Mark Polo (mpolo)",
+		"+ ",
 
-		_("+Norwegian Translation"),
+		"_" N_("+Norwegian Translation"),
 		"-   Hallvard Norheim Bø (Lysander)",
 		"-   Erik J. Mesoy (Circon)",
 		"-   Susanne Mesoy (Rarlgland)",
 		"+ ",
 
-		_("+Polish Translation"),
+		"_" N_("+Polish Translation"),
 		"-   Artur R. Czechowski",
 		"-   methinks",
 		"-   BOrsuk",
+		"-   Paweł Stradomski",
+		"-   Paweł Tomak",
 		"+ ",
 
-		_("+Portuguese (Brazil) Translation"),
+		"_" N_("+Portuguese (Brazil) Translation"),
 		"-   Ambra Viviani Loos",
 		"-   Celso Goya",
 		"-   Claus Aranha",
@@ -212,11 +255,17 @@ std::vector<std::string> get_text() {
 		"-   Renato Cunha",
 		"+ ",
 
-		_("+Slovak Translation"),
+		"_" N_("+Russian Translation"),
+		"-   Alexandr Menovchicov",
+		"-   Azamat Hackimov",
+		"-   Roman Tuchin (Sankt)",
+		"+ ",
+
+		"_" N_("+Slovak Translation"),
 		"-   Viliam Bur",
 		"+ ",
 
-		_("+Spanish Translation"),
+		"_" N_("+Spanish Translation"),
 		"-   David Martínez",
 		"-   Franciso Muñoz (fmunoz)",
 		"-   Gabriel Rodríguez (Chewie)",
@@ -224,27 +273,32 @@ std::vector<std::string> get_text() {
 		"-   Jose Manuel Gomez (joseg)",
 		"+ ",
 
-		_("+Swedish Translation"),
+		"_" N_("+Swedish Translation"),
 		"-   Alexander Kjäll (capitol)",
-		"-   Susanna Björverud (sanna)",
 		"-   Stefan Bergström (tephlon)",
+		"-   Susanna Björverud (sanna)",
 		"-   wint3r",
 		"+ ",
 		
-		_("+Contributors"),
-		"-   Cyril Bouthors (CyrilB)",
-		"-   edge",
+		"_" N_("+Contributors"),
 		"-   Frédéric Wagner",
-		"-   Jan Zvánovec (jaz)",
-		"-   Maksim Orlovich (SadEagle)",
 		"+ ",
 
-		_("+ Bots"),
+		"_" N_("+Bots"),
 		"-   wesbot",
 		"+ "
 	};
 
-	return std::vector<std::string>(credits, credits + sizeof(credits) / sizeof(*credits));
+	std::vector< std::string > res;
+	size_t len = sizeof(credits) / sizeof(*credits);
+	res.reserve(len);
+	for(size_t i = 0; i < len; ++i) {
+		const char *s = credits[i];
+		if (s[0] == '_')
+			s = gettext(s + 1);
+		res.push_back(s);
+	}
+	return res;
 }
 
 void show_about(display& disp)
