@@ -1,4 +1,4 @@
-/* $Id: actions.hpp,v 1.45 2004/08/25 02:59:50 Sirp Exp $ */
+/* $Id: actions.hpp,v 1.48 2004/12/05 18:37:36 silene Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -47,11 +47,6 @@ std::string recruit_unit(const gamemap& map, int team, unit_map& units,
 //battle that could take place.
 struct battle_stats
 {
-	std::string attack_name, defend_name;
-	std::string attack_type, defend_type;
-	std::string attack_special, defend_special;
-	std::string range;
-	std::string attack_icon, defend_icon;
 	int chance_to_hit_attacker, chance_to_hit_defender;
 	int damage_attacker_takes, damage_defender_takes;
 	int amount_attacker_drains, amount_defender_drains;
@@ -59,8 +54,18 @@ struct battle_stats
 	int attack_with, defend_with;
 	bool attacker_plague, defender_plague;
 	bool attacker_slows, defender_slows;
-	std::vector<std::string> attack_calculations, defend_calculations;
 	bool to_the_death, defender_strikes_first;
+	std::string attacker_special, defender_special;
+};
+
+struct battle_stats_strings
+{
+	std::string attack_name, defend_name;
+	std::string attack_type, defend_type;
+	std::string attack_special, defend_special;
+	std::string range;
+	std::string attack_icon, defend_icon;
+	std::vector<std::string> attack_calculations, defend_calculations;
 };
 
 //evaluate_battle_stats: a function which, if given an attacker
@@ -76,26 +81,24 @@ struct battle_stats
 //if include_strings is false, then none of the strings in
 //battle_stats will be populated, and the function will run
 //substantially faster.
-battle_stats evaluate_battle_stats(
-                   const gamemap& map,
-                   const gamemap::location& attacker,
-                   const gamemap::location& defender,
-				   int attack_with,
-				   std::map<gamemap::location,unit>& units,
-				   const gamestatus& state,
-				   const game_data& info,
-				   gamemap::TERRAIN attacker_terrain_override=0,
-				   bool include_strings=true);
+battle_stats evaluate_battle_stats(const gamemap& map,
+                                   const gamemap::location& attacker,
+                                   const gamemap::location& defender,
+                                   int attack_with,
+                                   std::map<gamemap::location,unit>& units,
+                                   const gamestatus& state,
+                                   gamemap::TERRAIN attacker_terrain_override = 0,
+                                   battle_stats_strings *strings = NULL);
 
 //attack: executes an attack.
 void attack(display& gui, const gamemap& map,
-				std::vector<team>& teams,
+            std::vector<team>& teams,
             gamemap::location attacker,
             gamemap::location defender,
             int attack_with,
             std::map<gamemap::location,unit>& units,
             const gamestatus& state,
-            const game_data& info, bool player_is_attacker);
+            const game_data& info);
 
 //given the location of a village, will return the 0-based index of the team
 //that currently owns it, and -1 if it is unowned.
@@ -187,7 +190,7 @@ size_t move_unit(display* disp, const game_data& gamedata,
 				std::vector<gamemap::location> steps,
 				replay* move_recorder, undo_list* undos,
 				gamemap::location *next_unit = NULL,
-				bool continue_move = false);
+				bool continue_move = false, bool should_clear_shroud=true);
 
 //function which recalculates the fog
 void recalculate_fog(const gamemap& map, const gamestatus& status,
