@@ -1,4 +1,4 @@
-/* $Id: ai.cpp,v 1.120 2004/10/04 22:45:37 Sirp Exp $ */
+/* $Id: ai.cpp,v 1.137 2005/02/06 10:40:12 isaaccp Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -10,8 +10,13 @@
 
    See the COPYING file for more details.
 */
+
+#include "global.hpp"
+
 #include "actions.hpp"
 #include "ai.hpp"
+#include "ai2.hpp"
+//#include "advanced_ai.hpp"
 #include "ai_attack.hpp"
 #include "ai_move.hpp"
 #include "dialogs.hpp"
@@ -28,6 +33,7 @@
 #include "statistics.hpp"
 #include "unit_display.hpp"
 #include "util.hpp"
+#include "wassert.hpp"
 
 #include <iostream>
 
@@ -51,7 +57,7 @@ public:
 		do_recruitment();
 	}
 
-private:
+protected:
 	void do_attacks() {
 		std::map<location,paths> possible_moves;
 		move_map srcdst, dstsrc;
@@ -99,14 +105,14 @@ private:
 
 	int choose_weapon(const location& attacker, const location& defender) {
 		const unit_map::const_iterator att = get_info().units.find(attacker);
-        assert(att != get_info().units.end());
+        wassert(att != get_info().units.end());
         
         const std::vector<attack_type>& attacks = att->second.attacks();
 
         int best_attack_rating = -1;
         int best_attack = -1;
         for(size_t n = 0; n != attacks.size(); ++n) {
-			const battle_stats stats = evaluate_battle_stats(get_info().map,attacker,defender,n,get_info().units,get_info().state,get_info().gameinfo,0,false);
+			const battle_stats stats = evaluate_battle_stats(get_info().map, attacker, defender, n, get_info().units, get_info().state);
 			const int attack_rating = stats.damage_defender_takes*stats.nattacks*stats.chance_to_hit_defender;
 			if(best_attack == -1 || attack_rating > best_attack_rating) {
                  best_attack = n;
@@ -133,35 +139,35 @@ private:
 
 	void do_moves() {
 		unit_map::const_iterator leader;
-        for(leader = get_info().units.begin(); leader != get_info().units.end(); ++leader) {
-            if(leader->second.can_recruit() && current_team().is_enemy(leader->second.side())) {
-                break;
-            }
-        }
+		for(leader = get_info().units.begin(); leader != get_info().units.end(); ++leader) {
+			if(leader->second.can_recruit() && current_team().is_enemy(leader->second.side())) {
+				break;
+			}
+		}
 
-        if(leader == get_info().units.end())
-            return;
+		if(leader == get_info().units.end())
+			return;
 
-        std::map<location,paths> possible_moves;
-        move_map srcdst, dstsrc;
-        calculate_possible_moves(possible_moves,srcdst,dstsrc,false);
+		std::map<location,paths> possible_moves;
+		move_map srcdst, dstsrc;
+		calculate_possible_moves(possible_moves,srcdst,dstsrc,false);
 
-        int closest_distance = -1;
-        std::pair<location,location> closest_move;
+		int closest_distance = -1;
+		std::pair<location,location> closest_move;
 
-        for(move_map::const_iterator i = dstsrc.begin(); i != dstsrc.end(); ++i) {
-            const int distance = distance_between(i->first,leader->first);
-            if(closest_distance == -1 || distance < closest_distance) {
-                closest_distance = distance;
-                closest_move = *i;
-            }
-        }
+		for(move_map::const_iterator i = dstsrc.begin(); i != dstsrc.end(); ++i) {
+			const int distance = distance_between(i->first,leader->first);
+			if(closest_distance == -1 || distance < closest_distance) {
+				closest_distance = distance;
+				closest_move = *i;
+			}
+		}
 
-        if(closest_distance != -1) {
-            move_unit(closest_move.second,closest_move.first,possible_moves);
-            do_moves();
-        }
-    }
+		if(closest_distance != -1) {
+			move_unit(closest_move.second,closest_move.first,possible_moves);
+			do_moves();
+		}
+	}
 
 	void do_recruitment() {
 		const std::set<std::string>& options = current_team().recruits();
@@ -187,6 +193,10 @@ ai_interface* create_ai(const std::string& name, ai_interface::info& info)
 		return new sample_ai(info);
 	else if(name == "idle_ai")
 		return new idle_ai(info);
+	//else if(name == "advanced_ai")
+	//	return new advanced_ai(info);
+	else if(name == "ai2")
+		return new ai2(info);
 	else if(name != "")
 		lg::err(lg::ai) << "AI not found: '" << name << "'\n";
 
@@ -241,13 +251,19 @@ bool ai::recruit_usage(const std::string& usage)
 bool ai_interface::recruit(const std::string& unit_name, location loc)
 {
 	const std::set<std::string>& recruits = current_team().recruits();
-	const std::set<std::string>::const_iterator i = std::find(recruits.begin(),recruits.end(),unit_name);
+
+	const std::set<std::string>::const_iterator i = recruits.find(unit_name);
 	if(i == recruits.end()) {
 		return false;
 	}
 
 	const int num = std::distance(recruits.begin(),i);
 
+	//we have to add the recruit command now, because when the unit is created it has
+	//to have the recruit command in the recorder to be able to put random numbers into
+	//to generate unit traits. However we're not sure the transaction is going through yet,
+	//so use a replay_undo object to cancel it if we don't get to the point where the transaction
+	//is confirmed
 	recorder.add_recruit(num,loc);
 	replay_undo replay_guard(recorder);
 
@@ -340,9 +356,9 @@ gamemap::location ai_interface::move_unit(location from, location to, std::map<l
 	LOG_AI << "ai_interface::move_unit " << (from.x + 1) << "," << (from.y + 1)
 		<< " -> " << (to.x + 1) << "," << (to.y + 1) << "\n";
 	//stop the user from issuing any commands while the unit is moving
-	const command_disabler disable_commands(&info_.disp);
+	const command_disabler disable_commands;
 
-	assert(info_.units.find(to) == info_.units.end() || from == to);
+	wassert(info_.units.find(to) == info_.units.end() || from == to);
 
 	info_.disp.select_hex(from);
 	info_.disp.update_display();
@@ -351,7 +367,7 @@ gamemap::location ai_interface::move_unit(location from, location to, std::map<l
 	unit_map::iterator u_it = info_.units.find(from);
 	if(u_it == info_.units.end()) {
 		lg::err(lg::ai) << "Could not find unit at " << from.x << ", " << from.y << "\n";
-		assert(false);
+		wassert(false);
 		return location();
 	}
 
@@ -368,11 +384,10 @@ gamemap::location ai_interface::move_unit(location from, location to, std::map<l
 	paths current_paths(info_.map,info_.state,info_.gameinfo,info_.units,from,info_.teams,ignore_zocs,teleport);
 	paths_wiper wiper(info_.disp);
 
-	unit current_unit = u_it->second;
-	info_.units.erase(u_it);
-
 	const std::map<location,paths>::iterator p_it = possible_moves.find(from);
 
+	unit current_unit = u_it->second;
+
 	if(p_it != possible_moves.end()) {
 		paths& p = p_it->second;
 		std::map<location,paths::route>::iterator rt = p.routes.begin();
@@ -413,18 +428,24 @@ gamemap::location ai_interface::move_unit(location from, location to, std::map<l
 
 			steps.push_back(to); //add the destination to the steps
 
-			if(show_move && unit_display::unit_visible_on_path(info_.disp,info_.map,steps,current_unit,info_.state.get_time_of_day(),info_.units,info_.teams)) {
+			if(show_move && unit_display::unit_visible_on_path(info_.disp,info_.map,steps,u_it->second,info_.state.get_time_of_day(),info_.units,info_.teams)) {
 
 				if(!info_.disp.fogged(from.x,from.y)) {
 					info_.disp.set_paths(&current_paths);
 				}
 
 				info_.disp.scroll_to_tiles(from.x,from.y,to.x,to.y);
+				info_.units.erase(u_it);
+				u_it = info_.units.end();
 				unit_display::move_unit(info_.disp,info_.map,steps,current_unit,info_.state.get_time_of_day(),info_.units,info_.teams);
 			}
 		}
 	}
 
+	if(u_it != info_.units.end()) {
+		info_.units.erase(u_it);
+	}
+
 	current_unit.set_movement(0);
 	info_.units.insert(std::pair<location,unit>(to,current_unit));
 	if(info_.map.is_village(to)) {
@@ -906,7 +927,7 @@ bool ai::do_combat(std::map<gamemap::location,paths>& possible_moves, const move
 void ai_interface::attack_enemy(const location& u, const location& target, int weapon)
 {
 	//stop the user from issuing any commands while the unit is attacking
-	const command_disabler disable_commands(&info_.disp);
+	const command_disabler disable_commands;
 
 	if(info_.units.count(u) && info_.units.count(target)) {
 		if(info_.units.find(target)->second.stone()) {
@@ -917,7 +938,7 @@ void ai_interface::attack_enemy(const location& u, const location& target, int w
 		recorder.add_attack(u,target,weapon);
 		game_events::fire("attack",u,target);
 
-		attack(info_.disp,info_.map,info_.teams,u,target,weapon,info_.units,info_.state,info_.gameinfo,false);
+		attack(info_.disp, info_.map, info_.teams, u, target, weapon, info_.units, info_.state, info_.gameinfo);
 		check_victory(info_.units,info_.teams);
 		dialogs::advance_unit(info_.gameinfo,info_.map,info_.units,u,info_.disp,true);
 
@@ -1268,7 +1289,7 @@ bool ai::move_to_targets(std::map<gamemap::location,paths>& possible_moves, move
 		LOG_AI << "choosing move...\n";
 		std::pair<location,location> move = choose_move(targets,srcdst,dstsrc,enemy_srcdst,enemy_dstsrc);
 		for(std::vector<target>::const_iterator ittg = targets.begin(); ittg != targets.end(); ++ittg) {
-			assert(map_.on_board(ittg->loc));
+			wassert(map_.on_board(ittg->loc));
 		}
 
 		if(move.first.valid() == false) {
@@ -1360,6 +1381,13 @@ int ai::average_resistance_against(const unit_type& a, const unit_type& b) const
 		weight_sum += weight;
 	}
 
+	// catch division by zero here if the attacking unit
+	// has zero attacks and/or zero damage
+	// if it has no attack at all the ai shouldn't prefer
+	// that unit anyway
+	if (weight_sum == 0) {
+		return sum;
+	}
 	return sum/weight_sum;
 }
 
@@ -1497,7 +1525,7 @@ void ai::analyze_potential_recruit_movements()
 		const shortest_path_calculator calc(temp_unit,current_team(),units,teams_,map_,state_);
 		for(std::vector<target>::const_iterator t = targets.begin(); t != targets.end(); ++t) {
 			LOG_AI << "analyzing '" << *i << "' getting to target...\n";
-			const paths::route& route = a_star_search(start,t->loc,100.0,calc);
+			const paths::route& route = a_star_search(start, t->loc, 100.0, &calc, get_info().map.x(), get_info().map.y());
 			if(route.steps.empty() == false) {
 				LOG_AI << "made it: " << route.move_left << "\n";
 				cost += route.move_left;
@@ -1614,7 +1642,7 @@ void ai::do_recruitment()
 	const std::vector<std::string>& options = current_team().recruitment_pattern();
 
 	if(options.empty()) {
-		assert(false);
+		wassert(false);
 		return;
 	}
 
@@ -1644,7 +1672,8 @@ void ai::move_leader_to_goals(const move_map& enemy_srcdst, const move_map& enem
 	
 	do_recruitment();
 
-	const paths::route route = a_star_search(leader->first,dst,1000.0,shortest_path_calculator(leader->second,current_team(),units_,teams_,map_,state_));
+	shortest_path_calculator calc(leader->second, current_team(), units_, teams_, map_, state_);
+	const paths::route route = a_star_search(leader->first, dst, 1000.0, &calc, get_info().map.x(), get_info().map.y());
 	if(route.steps.empty()) {
 		LOG_AI << "route empty";
 		return;
@@ -1736,7 +1765,7 @@ void ai::move_leader_after_recruit(const move_map& enemy_dstsrc)
 			//if this is a village of ours, that an enemy can capture on their turn, and
 			//which we might be able to reach in two turns.
 			if(map_.is_village(i->first) && current_team().owns_village(i->first) &&
-				distance_between(i->first,leader->first) <= leader->second.total_movement()*2) {
+				int(distance_between(i->first,leader->first)) <= leader->second.total_movement()*2) {
 				
 				int current_distance = distance_between(i->first,leader->first);
 				location current_loc;
