@@ -1,4 +1,4 @@
-/* $Id: builder.hpp,v 1.25 2004/09/11 21:22:57 gruikya Exp $ */
+/* $Id: builder.hpp,v 1.26 2004/12/02 07:59:49 silene Exp $ */
 /*
    Copyright (C) 2004 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -50,7 +50,7 @@ public:
 
 	//updates the animation at a given tile. returns true if something has
 	//changed, and must be redrawn.
-	bool terrain_builder::update_animation(const gamemap::location &loc);
+	bool update_animation(const gamemap::location &loc);
 
 	// regenerate the generated content at the given
 	// location. Currently: set the image at that location to the
