@@ -1,4 +1,5 @@
 #include "../config.hpp"
+#include "../filesystem.hpp"
 #include "../network.hpp"
 #include "../publish_campaign.hpp"
 #include "../util.hpp"
@@ -65,7 +66,7 @@ void campaign_server::run()
 			}
 
 			config data;
-			while((sock = network::receive_data(data)) != NULL) {
+			while((sock = network::receive_data(data)) != network::null_connection) {
 				if(data.child("request_campaign_list") != NULL) {
 					config campaign_list = campaigns();
 					config::child_list cmps = campaign_list.get_children("campaign");
@@ -103,6 +104,7 @@ void campaign_server::run()
 							campaign = &campaigns().add_child("campaign");
 						}
 
+						(*campaign)["title"] = (*upload)["title"];
 						(*campaign)["name"] = (*upload)["name"];
 						(*campaign)["filename"] = (*upload)["name"];
 						(*campaign)["passphrase"] = (*upload)["passphrase"];
@@ -118,7 +120,9 @@ void campaign_server::run()
 						const config* const data = upload->child("data");
 						if(data != NULL) {
 							compression_schema schema;
-							write_file((*campaign)["filename"],data->write_compressed(schema));
+							const std::string& filedata = data->write_compressed(schema);
+							write_file((*campaign)["filename"],filedata);
+							(*campaign)["size"] = lexical_cast<std::string>(filedata.size());
 						}
 
 						write_file(file_,cfg_.write());
