@@ -1,4 +1,4 @@
-/* $Id: config.cpp,v 1.109 2004/09/12 21:55:19 Sirp Exp $ */
+/* $Id: config.cpp,v 1.123 2005/01/28 20:17:46 silene Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -11,13 +11,9 @@
    See the COPYING file for more details.
 */
 
-//disable the very annoying VC++ warning 4786
-#ifdef WIN32
-#pragma warning(disable:4786)
-#endif
+#include "global.hpp"
 
 #include <algorithm>
-#include <cassert>
 #include <cctype>
 #include <fstream>
 #include <iostream>
@@ -31,10 +27,14 @@
 #include "game_events.hpp"
 #include "gettext.hpp"
 #include "log.hpp"
-#include "scoped_resource.hpp"
 #include "util.hpp"
+#include "wassert.hpp"
 #include "wesconfig.h"
 
+#define ERR_CF lg::err(lg::config)
+#define WRN_CF lg::warn(lg::config)
+#define LOG_CF lg::info(lg::config)
+
 bool operator<(const line_source& a, const line_source& b)
 {
 	return a.linenum < b.linenum;
@@ -74,108 +74,12 @@ line_source get_line_source(const std::vector<line_source>& line_src, int line)
 	return res;
 }
 
-void read_file_internal(const std::string& fname, std::string& res)
-{
-	const int size = file_size(fname);
-	if(size < 0) {
-		return;
-	}
-
-	std::vector<char> v;
-	v.reserve(size);
-
-	//const util::scoped_resource<FILE*,close_FILE> file(fopen(fname.c_str(),"rb"));
-	const util::scoped_FILE file(fopen(fname.c_str(),"rb"));
-
-	if(file == NULL) {
-		return;
-	}
-
-	const int block_size = 65536;
-
-	while(v.size() < size) {
-		const size_t expected = minimum<size_t>(block_size,size - v.size());
-
-		if(expected > 0) {
-			v.resize(v.size() + expected);
-			const size_t nbytes = fread(&v[v.size() - expected],1,expected,file);
-			if(nbytes < expected) {
-				v.resize(v.size() - (expected - nbytes));
-				break;
-			}
-		}
-	}
-
-	res.resize(v.size());
-	std::copy(v.begin(),v.end(),res.begin());
-}
-
 } //end anon namespace
 
-std::string read_stdin()
-{
-	std::vector<char> v;
-	const int block_size = 65536;
-
-	size_t nbytes = 1;
-	while(nbytes > 0) {
-		v.resize(v.size() + block_size);
-		nbytes = fread(&v[v.size() - block_size],1,block_size,stdin);
-		if(nbytes < block_size) {
-			v.resize(v.size() - (block_size - nbytes));
-			break;
-		}
-	}
-	
-	std::string res;
-	res.resize(v.size());
-	std::copy(v.begin(),v.end(),res.begin());
-	return res;
-}
-
 const char* io_exception::what() const throw() {
 	return message.c_str();
 }
 
-std::string read_file(const std::string& fname)
-{
-	//if we have a path to the data,
-	//convert any filepath which is relative
-	if(!fname.empty() && fname[0] != '/' && !game_config::path.empty()) {
-		std::string res;
-		read_file_internal(game_config::path + "/" + fname,res);
-		if(!res.empty()) {
-			return res;
-		}
-	}
-
-	std::string res;
-	read_file_internal(fname,res);
-	return res;
-}
-
-//throws io_exception if an error occurs
-void write_file(const std::string& fname, const std::string& data)
-{
-	//const util::scoped_resource<FILE*,close_FILE> file(fopen(fname.c_str(),"wb"));
-	const util::scoped_FILE file(fopen(fname.c_str(),"wb"));
-	if(file.get() == NULL) {
-		throw io_exception("Could not open file for writing: '" + fname + "'");
-	}
-
-	const size_t block_size = 4096;
-	char buf[block_size];
-
-	for(size_t i = 0; i < data.size(); i += block_size) {
-		const size_t bytes = minimum<size_t>(block_size,data.size() - i);
-		std::copy(data.begin() + i, data.begin() + i + bytes,buf);
-		const size_t res = fwrite(buf,1,bytes,file.get());
-		if(res != bytes) {
-			throw io_exception("Error writing to file: '" + fname + "'");
-		}
-	}
-}
-
 namespace {
 
 //this function takes a macro and parses it into the macro followed by its
@@ -268,7 +172,7 @@ void internal_preprocess_data(const std::string& data,
 
 				const preproc_define& val = defines_map[symbol];
 				if(val.arguments.size() != items.size()) {
-					std::cerr << "error: preprocessor symbol '" << symbol << "' has "
+					ERR_CF << "preprocessor symbol '" << symbol << "' has "
 					          << items.size() << " arguments, "
 							  << val.arguments.size() << " expected: '" << newfilename << "'\n";
 				}
@@ -288,7 +192,7 @@ void internal_preprocess_data(const std::string& data,
 						str.replace(pos,item.size(),replace_with);
 						const std::string::size_type new_pos = str.find(item);
 						if(new_pos < pos+replace_with.size()) {
-							std::cerr << "macro substitution in symbol '" << symbol
+							ERR_CF << "macro substitution in symbol '" << symbol
 								      << "' could lead to infinite recursion. Aborting.\n";
 							break;
 						}
@@ -312,7 +216,7 @@ void internal_preprocess_data(const std::string& data,
 					nfname.erase(nfname.begin(),nfname.begin()+1);
 					nfname = get_user_data_dir() + "/data/" + nfname;
 
-					std::cerr << "got relative name '" << newfilename << "' -> '" << nfname << "'\n";
+					LOG_CF << "got relative name '" << newfilename << "' -> '" << nfname << "'\n";
 
 					if(newfilename[0] == '@' && file_exists(nfname) == false && is_directory(nfname) == false) {
 						nfname = "data/" + newfilename.substr(1);
@@ -557,6 +461,7 @@ void config::read(const std::string& data,
 	std::stack<std::map<std::string,config*> > last_element; //allows [+element] syntax
 	std::stack<std::string> textdomains;
 	std::string current_textdomain = PACKAGE;
+	std::string current_textdomain_location = "";
 	elements.push(this);
 	element_names.push("");
 	element_locs.push(0);
@@ -565,9 +470,11 @@ void config::read(const std::string& data,
 	enum { ELEMENT_NAME, IN_ELEMENT, VARIABLE_NAME, VALUE }
 	state = IN_ELEMENT;
 	std::string var;
+
+	std::vector<std::pair<std::string,bool> > stored_values;
 	std::string value;
 
-	bool in_quotes = false, has_quotes = false, in_comment = false, escape_next = false, translatable = false;;
+	bool in_quotes = false, has_quotes = false, in_comment = false, escape_next = false, translatable = false, expecting_value = false;
 
 	int line = 0;
 
@@ -593,7 +500,7 @@ void config::read(const std::string& data,
 			case ELEMENT_NAME:
 				if(c == ']') {
 					if(value == "end" || value.empty() == false && value[0] == '/') {
-						assert(!elements.empty());
+						wassert(!elements.empty());
 
 						if(value[0] == '/' &&
 						   std::string("/" + element_names.top()) != value) {
@@ -643,6 +550,7 @@ void config::read(const std::string& data,
 							current_textdomain = textdomains.top();
 							textdomains.pop();
 						}
+						current_textdomain_location = "";
 
 						state = IN_ELEMENT;
 
@@ -708,18 +616,19 @@ void config::read(const std::string& data,
 				if(c == '[' && in_quotes) {
 					if(line_sources != NULL) {
 						const line_source src = get_line_source(*line_sources,line);
-						std::cerr << src.file << " " << src.fileline << ": ";
+						LOG_CF << src.file << " " << src.fileline << ": ";
 					} else {
-						std::cerr << "line " << line << ": ";
+						LOG_CF << "line " << line << ": ";
 					}
 
-					std::cerr << "WARNING: square bracket found in string. Is this a run-away string?\n";
+					WRN_CF << "square bracket found in string. Is this a run-away string?\n";
 				}
 				
 				if(in_quotes && c == '"' && (i+1) != data.end() && *(i+1) == '"') {
 					push_back(value, c);
 					++i; // skip the next double-quote
 				} else if(c == '"') {
+					expecting_value = false;
 					in_quotes = !in_quotes;
 					has_quotes = true;
 
@@ -732,8 +641,27 @@ void config::read(const std::string& data,
 							translatable = true;
 						}
 					}
+				} else if(c == '+' && has_quotes && !in_quotes) {
+					stored_values.push_back(std::make_pair(value,translatable));
+					value = "";
+					translatable = false;
+					expecting_value = true;
+				} else if(c == '\n' && !in_quotes && expecting_value) {
+					//do nothing...just ignore
 				} else if(c == '\n' && !in_quotes) {
 
+					stored_values.push_back(std::make_pair(value,translatable));
+					value = "";
+					for(std::vector<std::pair<std::string,bool> >::const_iterator i = stored_values.begin(); i != stored_values.end(); ++i) {
+						if(i->second) {
+							value += dsgettext(current_textdomain.c_str(),i->first.c_str());
+						} else {
+							value += i->first;
+						}
+					}
+
+					stored_values.clear();
+
 					//see if this is a CSV list=CSV list style assignment (e.g. x,y=5,8)
 					std::vector<std::string> vars, values;
 					if(std::count(var.begin(),var.end(),',') > 0) {
@@ -746,8 +674,13 @@ void config::read(const std::string& data,
 							textdomains.push(current_textdomain);
 							current_textdomain = value;
 							bindtextdomain(current_textdomain.c_str(),
-								       get_intl_dir().c_str());
+								       current_textdomain_location.empty() ?
+								       get_intl_dir().c_str() :
+								       current_textdomain_location.c_str());
 							bind_textdomain_codeset (current_textdomain.c_str(), "UTF-8");
+						} else if (var == "translations") {
+							const std::string& location = get_binary_file_location(value, ".");
+							current_textdomain_location = location;
 						}
 					}
 
@@ -768,11 +701,6 @@ void config::read(const std::string& data,
 								strip(value);
 							}
 
-							if(translatable) {
-								value = dsgettext(current_textdomain.c_str(),
-										  value.c_str());
-							}
-
 							if(n < vars.size()) {
 								elements.top()->values[vars[n]] = value;
 							} else {
@@ -788,6 +716,10 @@ void config::read(const std::string& data,
 					escape_next = false;
 					translatable = false;
 				} else if(in_quotes || !has_quotes) {
+					expecting_value = false;
+					push_back(value, c);
+				} else if(expecting_value) {
+					// after a +, emulate !has_quotes so we can see any _ when we encounter a " later
 					push_back(value, c);
 				}
 
@@ -900,9 +832,9 @@ std::string config::write() const
 	res.resize(write_size());
 
 	const std::string::iterator i = write_internal(res.begin());
-	assert(i == res.end());
+	wassert(i == res.end());
 	if(i != res.end()) {
-		std::cerr << "ERROR in size of config buffer: " << (i - res.begin()) << "/" << res.size() << "\n";
+		ERR_CF << "size of config buffer: " << (i - res.begin()) << "/" << res.size() << "\n";
 	}
 
 	return res;
@@ -1078,7 +1010,7 @@ std::string::const_iterator config::read_compressed_internal(std::string::const_
 
 				const compression_schema::char_word_map::const_iterator itor = schema.char_to_word.find(code);
 				if(itor == schema.char_to_word.end()) {
-					std::cerr << "illegal word code: " << code << "\n";
+					ERR_CF << "illegal word code: " << code << "\n";
 					throw error("Illegal character in compression input\n");
 				}
 
@@ -1184,6 +1116,36 @@ config& config::add_child(const std::string& key, const config& val)
 	return *v.back();
 }
 
+config& config::add_child_at(const std::string& key, const config& val, size_t index)
+{
+	child_list& v = children[key];
+	if(index > v.size()) {
+		throw error("illegal index to add child at");
+	}
+
+	v.insert(v.begin()+index,new config(val));
+
+	bool inserted = false;
+
+	const child_pos value(children.find(key),index);
+
+	std::vector<child_pos>::iterator ord = ordered_children.begin();
+	for(; ord != ordered_children.end(); ++ord) {
+		if(!inserted && ord->index == index && ord->pos->first == key) {
+			ord = ordered_children.insert(ord,value);
+			inserted = true;
+		} else if(ord->index >= index && ord->pos->first == key) {
+			ord->index++;
+		}
+	}
+
+	if(!inserted) {
+		ordered_children.push_back(value);
+	}
+
+	return *v[index];
+}
+
 struct remove_ordered {
 	remove_ordered(const std::string& key) : key_(key) {}
 
@@ -1214,7 +1176,7 @@ void config::remove_child(const std::string& key, size_t index)
 
 	//remove from the child map
 	child_list& v = children[key];
-	assert(index < v.size());
+	wassert(index < v.size());
 	config* const res = v[index];
 	v.erase(v.begin()+index);
 	delete res;
@@ -1452,7 +1414,7 @@ bool not_id(char c)
 
 void do_interpolation(std::string& res, size_t npos, const string_map* m)
 {
-	std::cerr << "doing interpolation into '" << res << "': " << npos << "\n";
+	LOG_CF << "doing interpolation into '" << res << "': " << npos << "\n";
 	const std::string::iterator i = std::find(res.begin()+npos,res.end(),'$');
 	if(i == res.end() || i+1 == res.end()) {
 		return;
@@ -1614,13 +1576,14 @@ config config::get_diff(const config& c) const
 			} else {
 				//we have to work out what the most appropriate operation --
 				//delete, insert, or change is the best to get b[bi] looking like a[ai]
+				std::stringstream buf;
+
 				//if b has more elements than a, then we assume this element is an
 				//element that needs deleting
 				if(b.size() - bi > a.size() - ai) {
 					config& new_delete = res.add_child("delete_child");
-					char buf[50];
-					sprintf(buf,"%d",bi-ndeletes);
-					new_delete.values["index"] = buf;
+					buf << bi - ndeletes;
+					new_delete.values["index"] = buf.str();
 					new_delete.add_child(*itor);
 
 					++ndeletes;
@@ -1631,9 +1594,8 @@ config config::get_diff(const config& c) const
 				//element that needs inserting
 				else if(b.size() - bi < a.size() - ai) {
 					config& new_insert = res.add_child("insert_child");
-					char buf[50];
-					sprintf(buf,"%d",ai);
-					new_insert.values["index"] = buf;
+					buf << ai;
+					new_insert.values["index"] = buf.str();
 					new_insert.add_child(*itor,*a[ai]);
 
 					++ai;
@@ -1643,9 +1605,8 @@ config config::get_diff(const config& c) const
 				//changing this element to match
 				else {
 					config& new_change = res.add_child("change_child");
-					char buf[50];
-					sprintf(buf,"%d",bi);
-					new_change.values["index"] = buf;
+					buf << bi;
+					new_change.values["index"] = buf.str();
 					new_change.add_child(*itor,a[ai]->get_diff(*b[bi]));
 
 					++ai;
@@ -1700,17 +1661,7 @@ void config::apply_diff(const config& diff)
 		const size_t index = atoi((**i)["index"].c_str());
 		for(all_children_iterator j = (*i)->ordered_begin(); j != (*i)->ordered_end(); ++j) {
 			const std::pair<const std::string*,const config*> item = *j;
-
-			if(item.first->empty()) {
-				continue;
-			}
-
-			child_list& v = children[*item.first];
-			if(index > v.size()) {
-				throw error("error in diff: could not find element '" + *item.first + "'");
-			}
-
-			v.insert(v.begin()+index,new config(*item.second));
+			add_child_at(*item.first,*item.second,index);
 		}
 	}
 
@@ -1720,17 +1671,7 @@ void config::apply_diff(const config& diff)
 		for(all_children_iterator j = (*i)->ordered_begin(); j != (*i)->ordered_end(); ++j) {
 			const std::pair<const std::string*,const config*> item = *j;
 
-			if(item.first->empty()) {
-				continue;
-			}
-
-			const child_map::iterator itor = children.find(*item.first);
-			if(itor == children.end() || index > itor->second.size()) {
-				throw error("error in diff: could not find element '" + *item.first + "'");
-			}
-
-			delete *(itor->second.begin()+index);
-			itor->second.erase(itor->second.begin()+index);
+			remove_child(*item.first,index);
 		}
 	}
 }
