@@ -1,4 +1,4 @@
-/* $Id: dialogs.hpp,v 1.24 2004/09/22 00:05:33 cedricd Exp $ */
+/* $Id: dialogs.hpp,v 1.29 2004/12/19 21:18:14 silene Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -58,8 +58,7 @@ void unit_speak(const config& message_info, display& disp, const unit_map& units
 /// index of the button pressed, or -1 if the dialog was canceled
 /// through keypress.
 int show_file_chooser_dialog(display &displ, std::string &filename,
-							 const std::string title="Choose a File",
-							 int xloc=-1, int yloc=-1);
+                             std::string const &title, int xloc = -1, int yloc = -1);
 
 class unit_preview_pane : public gui::preview_pane
 {
@@ -74,8 +73,8 @@ public:
 	void set_selection(int index);
 
 private:
-	void draw();
-	void process();
+	void draw_contents();
+	void process_event();
 
 	gui::button details_button_;
 	const gamemap* map_;
@@ -86,7 +85,7 @@ private:
 	bool weapons_;
 };
 
-void show_unit_description(display& disp, const gamemap& map, const unit& u);
+void show_unit_description(display& disp, const unit& u);
 
 
 class campaign_preview_pane : public gui::preview_pane
@@ -99,7 +98,7 @@ public:
 	void set_selection(int index);
 
 private:
-	void draw();
+	void draw_contents();
 
 	const std::vector<std::pair<std::string,std::string> >* descriptions_;
 	int index_;
