@@ -1,4 +1,4 @@
-/* $Id: display.cpp,v 1.263 2004/10/02 16:16:34 Sirp Exp $ */
+/* $Id: display.cpp,v 1.288 2005/02/06 14:05:02 silene Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -11,6 +11,8 @@
    See the COPYING file for more details.
 */
 
+#include "global.hpp"
+
 #include "actions.hpp"
 #include "display.hpp"
 #include "events.hpp"
@@ -30,22 +32,30 @@
 #include "tooltips.hpp"
 #include "unit_display.hpp"
 #include "util.hpp"
+#include "wassert.hpp"
 
 #include "SDL_image.h"
 
 #include <algorithm>
-#include <cassert>
 #include <cmath>
 #include <iostream>
 #include <sstream>
 
+#define ERR_DP lg::err(lg::display)
+
 std::map<gamemap::location,double> display::debugHighlights_;
 
 namespace {
-	const int MinZoom = 36;
 	const int DefaultZoom = 72;
 	const int MaxZoom = 200;
 
+	int MinZoom(const gamemap& map, const SDL_Rect& viewport)
+	{
+		const int min_zoom1 = viewport.w/((map.x()*3)/4);
+		const int min_zoom2 = viewport.h/map.y();
+		return maximum<int>(min_zoom1,min_zoom2);
+	}
+
 	const size_t SideBarGameStatus_x = 16;
 	const size_t SideBarGameStatus_y = 220;
 
@@ -54,7 +64,7 @@ namespace {
 
 display::display(unit_map& units, CVideo& video, const gamemap& map,
 		const gamestatus& status, const std::vector<team>& t,
-		const config& theme_cfg, const config& config, const config& level) :
+		const config& theme_cfg, const config& cfg, const config& level) :
 	screen_(video), xpos_(0), ypos_(0),
 	zoom_(DefaultZoom), map_(map), units_(units),
 	minimap_(NULL), redrawMinimap_(false),
@@ -65,10 +75,10 @@ display::display(unit_map& units, CVideo& video, const gamemap& map,
 	currentTeam_(0), activeTeam_(0), hideEnergy_(false),
 	deadAmount_(0.0), advancingAmount_(0.0), updatesLocked_(0),
 	turbo_(false), grid_(false), sidebarScaling_(1.0),
-	theme_(theme_cfg,screen_area()), builder_(config, level, map),
+	theme_(theme_cfg,screen_area()), builder_(cfg, level, map),
 	first_turn_(true), in_game_(false), map_labels_(*this,map),
 	tod_hex_mask1(NULL), tod_hex_mask2(NULL), diagnostic_label_(0),
-	help_string_(0)
+	help_string_(0), fps_handle_(0)
 {
 	if(non_interactive())
 		updatesLocked_++;
@@ -97,7 +107,7 @@ display::display(unit_map& units, CVideo& video, const gamemap& map,
 			flag = teams_[i].flag();
 		}
 
-		std::cerr << "Adding flag for team " << i << " from animation " << flag << "\n";
+		lg::info(lg::display) << "Adding flag for team " << i << " from animation " << flag << "\n";
 		flags_.push_back(animated<image::locator>(flag));
 		flags_.back().start_animation(0, animated<image::locator>::INFINITE_CYCLES);
 	}
@@ -227,7 +237,7 @@ void display::highlight_hex(gamemap::location hex)
 		invalidate_unit();
 }
 
-gamemap::location display::hex_clicked_on(int xclick, int yclick)
+gamemap::location display::hex_clicked_on(int xclick, int yclick, gamemap::location::DIRECTION* nearest_hex)
 {
 	const SDL_Rect& rect = map_area();
 	if(point_in_rect(xclick,yclick,rect) == false) {
@@ -237,12 +247,74 @@ gamemap::location display::hex_clicked_on(int xclick, int yclick)
 	xclick -= rect.x;
 	yclick -= rect.y;
 
-	const int tile_width = hex_width();
+	return pixel_position_to_hex(xpos_ + xclick, ypos_ + yclick, nearest_hex);
+}
 
-	const int xtile = (xpos_ + xclick)/tile_width;
-	const int ytile = (ypos_ + yclick - (is_odd(xtile) ? zoom_/2 : 0))/zoom_; //(yclick + is_odd(xtile) ? -yclick/2 : 0)/zoom_;
+gamemap::location display::pixel_position_to_hex(int x, int y, gamemap::location::DIRECTION* nearest_hex)
+{
+	const int s = hex_size();
+	const int tesselation_x_size = s * 3 / 2;
+	const int tesselation_y_size = s;
+	const int x_base = x / tesselation_x_size * 2;
+	const int x_mod = x % tesselation_x_size;
+	const int y_base = y / tesselation_y_size;
+	const int y_mod = y % tesselation_y_size;
+	
+	int x_modifier = 0;
+	int y_modifier = 0;
+	
+	if (y_mod < tesselation_y_size / 2) {
+		if ((x_mod * 2 + y_mod) < (s / 2)) {
+			x_modifier = -1;
+			y_modifier = -1;
+		} else if ((x_mod * 2 - y_mod) < (s * 3 / 2)) {
+			x_modifier = 0;
+			y_modifier = 0;
+		} else {
+			x_modifier = 1;
+			y_modifier = -1;
+		}
+		
+	} else {
+		if ((x_mod * 2 - (y_mod - s / 2)) < 0) {
+			x_modifier = -1;
+			y_modifier = 0;
+		} else if ((x_mod * 2 + (y_mod - s / 2)) < s * 2) {
+			x_modifier = 0;
+			y_modifier = 0;
+		} else {
+			x_modifier = 1;
+			y_modifier = 0;
+		}
+	}
 
-	return gamemap::location(xtile,ytile);
+	const gamemap::location res(x_base + x_modifier, y_base + y_modifier);
+	
+	if(nearest_hex != NULL) {
+		const int westx = (get_location_x(res) - map_area().x + xpos_) + hex_size()/3;
+		const int eastx = westx + hex_size()/3;
+		const int centery = (get_location_y(res) - map_area().y + ypos_) + hex_size()/2;
+
+		const bool west = x < westx;
+		const bool east = x > eastx;
+		const bool north = y < centery;
+
+		if(north && west) {
+			*nearest_hex = gamemap::location::NORTH_WEST;
+		} else if(north && east) {
+			*nearest_hex = gamemap::location::NORTH_EAST;
+		} else if(north) {
+			*nearest_hex = gamemap::location::NORTH;
+		} else if(west) {
+			*nearest_hex = gamemap::location::SOUTH_WEST;
+		} else if(east) {
+			*nearest_hex = gamemap::location::SOUTH_EAST;
+		} else {
+			*nearest_hex = gamemap::location::SOUTH;
+		}
+	}
+
+	return res;
 }
 
 int display::get_location_x(const gamemap::location& loc) const
@@ -319,50 +391,23 @@ int display::hex_width() const
 
 double display::zoom(int amount)
 {
-	if(amount == 0 || !team_valid()) {
-		return double(zoom_)/double(DefaultZoom);
-	}
+	int new_zoom = zoom_ + amount;
+	if (amount != 0 && team_valid() && new_zoom >= MinZoom(map_, map_area()) && new_zoom <= MaxZoom) {
+		SDL_Rect const &area = map_area();
+		xpos_ += (xpos_ + area.w / 2) * amount / zoom_;
+		ypos_ += (ypos_ + area.h / 2) * amount / zoom_;
+		zoom_ = new_zoom;
+		bounds_check_position();
 
-	const int orig_xpos = xpos_;
-	const int orig_ypos = ypos_;
-
-	xpos_ /= zoom_;
-	ypos_ /= zoom_;
-
-	const int orig_zoom = zoom_;
-
-	zoom_ += amount;
-	if(zoom_ < MinZoom || zoom_ > MaxZoom) {
-		zoom_ = orig_zoom;
-		xpos_ = orig_xpos;
-		ypos_ = orig_ypos;
-		return double(zoom_)/double(DefaultZoom);
-	}
-
-	xpos_ *= zoom_;
-	ypos_ *= zoom_;
-
-	xpos_ += amount*2;
-	ypos_ += amount*2;
-
-	const int prev_zoom = zoom_;
-
-	bounds_check_position();
-
-	if(zoom_ != prev_zoom) {
-		xpos_ = orig_xpos;
-		ypos_ = orig_ypos;
-		zoom_ = orig_zoom;
+		energy_bar_rects_.clear();
 		image::set_zoom(zoom_);
-		return double(zoom_)/double(DefaultZoom);
+		map_labels_.recalculate_labels();
+		invalidate_all();
+
+		// Forces a redraw after zooming. This prevents some graphic glitches from occuring.
+		draw();
 	}
 
-	energy_bar_rects_.clear();
-
-	image::set_zoom(zoom_);
-	map_labels_.recalculate_labels();
-	invalidate_all();
-
 	return double(zoom_)/double(DefaultZoom);
 }
 
@@ -436,9 +481,7 @@ void display::scroll_to_tiles(int x1, int y1, int x2, int y2,
 
 void display::bounds_check_position()
 {
-	const int min_zoom1 = map_area().w/((map_.x()*3)/4);
-	const int min_zoom2 = map_area().h/map_.y();
-	const int min_zoom = maximum<int>(min_zoom1,min_zoom2);
+	const int min_zoom = MinZoom(map_,map_area());
 
 	const int orig_zoom = zoom_;
 
@@ -631,8 +674,9 @@ void display::draw(bool update,bool force)
 		lastDraw_ = SDL_GetTicks();
 
 		if(wait_time >= 0 || drawSkips_ >= max_skips || force) {
-			if(changed)
+			if(changed) {
 				update_display();
+			}
 		} else {
 			drawSkips_++;
 		}
@@ -644,6 +688,31 @@ void display::update_display()
 	if(updatesLocked_ > 0)
 		return;
 
+	if(preferences::show_fps()) {
+		static int last_sample = SDL_GetTicks();
+		static int frames = 0;
+		++frames;
+
+		if(frames == 10) {
+			const int this_sample = SDL_GetTicks();
+
+			const int fps = (frames*1000)/(this_sample - last_sample);
+			last_sample = this_sample;
+			frames = 0;
+
+			if(fps_handle_ != 0) {
+				font::remove_floating_label(fps_handle_);
+				fps_handle_ = 0;
+			}
+			std::ostringstream stream;
+			stream << fps << "fps";
+			fps_handle_ = font::add_floating_label(stream.str(),12,font::NORMAL_COLOUR,10,100,0,0,-1,screen_area(),font::LEFT_ALIGN);
+		}
+	} else if(fps_handle_ != 0) {
+		font::remove_floating_label(fps_handle_);
+		fps_handle_ = 0;
+	}
+
 	screen_.flip();
 }
 
@@ -676,12 +745,12 @@ void display::draw_sidebar()
 	}
 
 	if(invalidateGameStatus_) {
-		draw_game_status(mapx()+SideBarGameStatus_x,SideBarGameStatus_y);
+		draw_game_status();
 		invalidateGameStatus_ = false;
 	}
 }
 
-void display::draw_game_status(int x, int y)
+void display::draw_game_status()
 {
 	if(teams_.empty())
 		return;
@@ -691,7 +760,7 @@ void display::draw_game_status(int x, int y)
 	}	
 }
 
-void display::draw_image_for_report(surface& img, surface& surf, SDL_Rect& rect)
+void display::draw_image_for_report(surface& img, SDL_Rect& rect)
 {
 	SDL_Rect visible_area = get_non_transperant_portion(img);
 	SDL_Rect target = rect;
@@ -766,7 +835,7 @@ void display::draw_report(reports::TYPE report_num)
 			if(rect.w > 0 && rect.h > 0) {
 				surf.assign(get_surface_portion(screen_.getSurface(),rect));
 				if(reportSurfaces_[report_num] == NULL) {
-					std::cerr << "Could not backup background for report!\n";
+					ERR_DP << "Could not backup background for report!\n";
 				}
 			}
 
@@ -809,7 +878,7 @@ void display::draw_report(reports::TYPE report_num)
 					surface img(image::get_image(i->image,image::UNSCALED));
 
 					if(img == NULL) {
-						std::cerr << "could not find image for report: '" << i->image << "'\n";
+						ERR_DP << "could not find image for report: '" << i->image << "'\n";
 						continue;
 					}
 
@@ -817,7 +886,7 @@ void display::draw_report(reports::TYPE report_num)
 					area.y = y;
 					area.w = minimum<int>(rect.w + rect.x - x, img->w);
 					area.h = minimum<int>(rect.h + rect.y - y, img->h);
-					draw_image_for_report(img,surf,area);
+					draw_image_for_report(img, area);
 
 					if(area.h > tallest) tallest = area.h;
 					x += area.w;
@@ -935,11 +1004,7 @@ void display::draw_unit_details(int x, int y, const gamemap::location& loc,
 		        << (at_it->range() == attack_type::SHORT_RANGE ?
 		            _("melee") :
 					_("ranged"));
-	
-		if(at_it->hexes() > 1) {
-			details << " (" << at_it->hexes() << ")";
-		}
-					
+						
 		details << "\n\n";
 	}
 
@@ -987,6 +1052,8 @@ void display::draw_minimap(int x, int y, int w, int h)
 	SDL_Rect loc = minimap_location;
 	SDL_BlitSurface(surf,NULL,video().getSurface(),&loc);
 
+	int map_w = map_.x(), map_h = map_.y();
+
 	for(unit_map::const_iterator u = units_.begin(); u != units_.end(); ++u) {
 		if(fogged(u->first.x,u->first.y) || 
 				(teams_[currentTeam_].is_enemy(u->second.side()) &&
@@ -998,14 +1065,14 @@ void display::draw_minimap(int x, int y, int w, int h)
 		const int side = u->second.side();
 		const SDL_Color& col = team::get_side_colour(side);
 		const Uint32 mapped_col = SDL_MapRGB(video().getSurface()->format,col.r,col.g,col.b);
-		SDL_Rect rect = {x + (u->first.x*w)/map_.x(),
-		                 y + (u->first.y*h)/map_.y(),
-						 w/map_.x(), h/map_.y() };
+		SDL_Rect rect = { x + (u->first.x * w) / map_w,
+		                  y + (u->first.y * h + (is_odd(u->first.x) ? h / 2 : 0)) / map_h,
+		                  w / map_w, h / map_h };
 		SDL_FillRect(video().getSurface(),&rect,mapped_col);
 	}
 
-	const double xscaling = double(surf->w)/double(map_.x());
-	const double yscaling = double(surf->h)/double(map_.y());
+	const double xscaling = double(surf->w) / map_w;
+	const double yscaling = double(surf->h) / map_h;
 
 	const int xbox = static_cast<int>(xscaling*xpos_/(zoom_*0.75));
 	const int ybox = static_cast<int>(yscaling*ypos_/zoom_);
@@ -1021,23 +1088,6 @@ void display::draw_minimap(int x, int y, int w, int h)
 	update_rect(minimap_location);
 }
 
-gamemap::TERRAIN display::get_terrain_on(int palx, int paly, int x, int y)
-{
-	const int height = 37;
-	if(y < paly || x < palx)
-		return 0;
-
-	const std::vector<gamemap::TERRAIN>& terrains=map_.get_terrain_precedence();
-	if(static_cast<size_t>(y) > paly+terrains.size()*height)
-		return 0;
-
-	const size_t index = (y - paly)/height;
-	if(index >= terrains.size())
-		return 0;
-
-	return terrains[index];
-}
-
 void display::draw_halo_on_tile(int x, int y)
 {
 	const gamemap::location loc(x,y);
@@ -1127,9 +1177,9 @@ void display::draw_unit_on_tile(int x, int y, surface unit_image_override,
 			}
 		}
 
-		assert(energy_file != NULL);
+		wassert(energy_file != NULL);
 		if(energy_file == NULL) {
-			std::cerr << "energy file is NULL\n";
+			ERR_DP << "energy file is NULL\n";
 			return;
 		}
 
@@ -1235,13 +1285,21 @@ void display::draw_unit_on_tile(int x, int y, surface unit_image_override,
 	if(u.experience() > 0 && u.can_advance()) {
 		const double filled = double(u.experience())/double(u.max_experience());
 		const int level = maximum<int>(u.type().level(),1);
-		const SDL_Color normal_colour = {02,153,255,0}, near_advance_colour = {255,255,255,0};
+		const SDL_Color normal_colour = {2,153,255,0}, near_advance_colour = {255,255,255,0};
 		const bool near_advance = u.max_experience() - u.experience() <= game_config::kill_experience*level;
 		const SDL_Color colour = near_advance ? near_advance_colour : normal_colour;
 
 		draw_bar("misc/bar-energy-enemy.png",xpos+5,ypos,u.max_experience()/(level*2),filled,colour,bar_alpha);
 	}
 
+	if (u.can_recruit()) {
+		surface crown(image::get_image("misc/leader-crown.png",image::SCALED,image::NO_ADJUST_COLOUR));
+		if(!crown.null()) {
+			SDL_Rect r = {0, 0, crown->w, crown->h};
+			blit_surface(xpos,ypos,crown,&r);
+		}
+	}
+
 	const std::vector<std::string>& overlays = it->second.overlays();
 	for(std::vector<std::string>::const_iterator ov = overlays.begin(); ov != overlays.end(); ++ov) {
 		const surface img(image::get_image(*ov));
@@ -1404,7 +1462,7 @@ void display::draw_tile(int x, int y, surface unit_image, double alpha, Uint32 b
 		surface surface(image::get_image("terrain/void.png"));
 
 		if(surface == NULL) {
-			std::cerr << "Could not get void surface!\n";
+			ERR_DP << "Could not get void surface!\n";
 			return;
 		}
 
@@ -1521,7 +1579,7 @@ void display::draw_footstep(const gamemap::location& loc, int xloc, int yloc)
 
 	surface image(image::get_image(*image_str));
 	if(image == NULL) {
-		std::cerr << "Could not find image: " << *image_str << "\n";
+		ERR_DP << "Could not find image: " << *image_str << "\n";
 		return;
 	}
 
@@ -1618,7 +1676,7 @@ std::vector<std::string> display::get_fog_shroud_graphics(const gamemap::locatio
 				//	stream << "void";
 				//else
 				//	stream << "fog";
-				stream << "terrain/" << map_.get_terrain_info(*terrain).default_image();
+				stream << "terrain/" << map_.get_terrain_info(*terrain).symbol_image();
 
 				for(int n = 0; *terrain == tiles[i] && n != 6; i = (i+1)%6, ++n) {
 					stream << get_direction(i);
@@ -1659,8 +1717,8 @@ std::vector<surface> display::get_terrain_images(int x, int y, image::TYPE image
 				image::locator image(*it);
 				// image.filename = "terrain/" + *it;
 
-				const surface surface(get_terrain(image,image_type,x,y));
-				if(surface != NULL) {
+				const surface surface(image::get_image(image, image_type));
+				if (!surface.null()) {
 					res.push_back(surface);
 				}
 			}
@@ -1685,8 +1743,8 @@ std::vector<surface> display::get_terrain_images(int x, int y, image::TYPE image
 			image::locator image = it->get_current_frame();
 			// image.filename = "terrain/" + image.filename;
 
-			const surface surface(get_terrain(image,image_type,x,y));
-			if(surface != NULL) {
+			const surface surface(image::get_image(image, image_type));
+			if (!surface.null()) {
 				res.push_back(surface);
 			}
 		}
@@ -1695,50 +1753,6 @@ std::vector<surface> display::get_terrain_images(int x, int y, image::TYPE image
 	return res;
 }
 
-surface display::get_terrain(const image::locator& image, image::TYPE image_type,
-                                 int x, int y)
-{
-
-#if 0
-	//see if there is a time-of-day specific version of this image
-	if(search_tod) {
-		// image::locator tod_image = image;
-		// tod_image.filename = image.filename + "-" + tod.id + ".png";
-		const image::locator& tod_image = image::get_alternative(image, "-" + tod.id);
-		im = image::get_image(tod_image,image_type);
-
-		if(im != NULL) {
-			return im;
-		}
-	}
-#endif
-
-	// image::locator tmp = image;
-	// tmp.filename += ".png";
-
-	const surface im(image::get_image(image, image_type));
-	if(im == NULL) {
-		return im;
-	}
-
-#if 0
-	//see if this tile is illuminated to a different colour than it'd
-	//normally be displayed as
-	const int radj = tod_at.red - tod.red;
-	const int gadj = tod_at.green - tod.green;
-	const int badj = tod_at.blue - tod.blue;
-	
-	if((radj|gadj|badj) != 0 && im != NULL) {
-		const surface backup(im);
-		im = surface(adjust_surface_colour(im,radj,gadj,badj));
-		if(im == NULL)
-			std::cerr << "could not adjust surface..\n";
-	}
-#endif
-	
-	return im;	
-}
-
 surface display::get_flag(gamemap::TERRAIN terrain, int x, int y)
 {
 	const bool village = map_.is_village(terrain);
@@ -1776,9 +1790,8 @@ surface display::get_minimap(int w, int h)
 	}
 
 	if(minimap_ == NULL) {
-		minimap_ = image::getMinimap(w,h,map_,
-				status_.get_time_of_day().lawful_bonus,
-				team_valid() ? &teams_[currentTeam_] : NULL);
+		minimap_ = image::getMinimap(w, h, map_,
+		                             team_valid() ? &teams_[currentTeam_] : NULL);
 	}
 
 	return minimap_;
@@ -1858,7 +1871,7 @@ void display::draw_unit(int x, int y, surface image,
 	}
 
 	if(surf == NULL) {
-		std::cerr << "surface lost...\n";
+		ERR_DP << "surface lost...\n";
 		return;
 	}
 
@@ -2016,7 +2029,7 @@ void display::write_overlays(config& cfg) const
 
 void display::set_team(size_t team)
 {
-	assert(team < teams_.size());
+	wassert(team < teams_.size());
 	currentTeam_ = team;
 
 	labels().recalculate_shroud();
@@ -2024,7 +2037,7 @@ void display::set_team(size_t team)
 
 void display::set_playing_team(size_t team)
 {
-	assert(team < teams_.size());
+	wassert(team < teams_.size());
 	activeTeam_ = team;
 	invalidate_game_status();
 }
@@ -2070,7 +2083,7 @@ void display::set_grid(bool grid)
 
 void display::debug_highlight(const gamemap::location& loc, double amount)
 {
-	assert(game_config::debug);
+	wassert(game_config::debug);
 	debugHighlights_[loc] += amount;
 }
 
@@ -2115,13 +2128,13 @@ const theme& display::get_theme() const
 	return theme_;
 }
 
-const theme::menu* display::menu_pressed(int mousex, int mousey, bool button_pressed)
+const theme::menu* display::menu_pressed()
 {
 
 	for(std::vector<gui::button>::iterator i = buttons_.begin(); i != buttons_.end(); ++i) {
 		if(i->pressed()) {
 			const size_t index = i - buttons_.begin();
-			assert(index < theme_.menus().size());
+			wassert(index < theme_.menus().size());
 			return &theme_.menus()[index];
 		}
 	}
@@ -2140,7 +2153,18 @@ int display::set_help_string(const std::string& str)
 	font::remove_floating_label(help_string_);
 
 	const SDL_Color colour = {0x0,0x00,0x00,0x77};
-	help_string_ = font::add_floating_label(str,font::SIZE_LARGE,font::NORMAL_COLOUR,x()/2,y(),0.0,0.0,-1,screen_area(),font::CENTER_ALIGN,&colour,5);
+
+	int size = font::SIZE_LARGE;
+
+	while(size > 0) {
+		if(font::line_width(str, size) > x()) {
+			size--;
+		} else {
+			break;
+		}
+	}
+
+	help_string_ = font::add_floating_label(str,size,font::NORMAL_COLOUR,x()/2,y(),0.0,0.0,-1,screen_area(),font::CENTER_ALIGN,&colour,5);
 	const SDL_Rect& rect = font::get_floating_label_rect(help_string_);
 	font::move_floating_label(help_string_,0.0,-double(rect.h));
 	return help_string_;
@@ -2192,13 +2216,21 @@ namespace {
 	const int chat_message_border = 5;
 	const int chat_message_x = 10;
 	const int chat_message_y = 10;
-	const SDL_Color chat_message_colour = {200,200,200,200};
-	const SDL_Color chat_message_bg     = {0,0,0,100};
+	const SDL_Color chat_message_colour = {255,255,255,255};
+	const SDL_Color chat_message_bg     = {0,0,0,140};
 }
 
 void display::add_chat_message(const std::string& speaker, int side, const std::string& message, display::MESSAGE_TYPE type)
 {
-	std::string msg = message;
+	bool action;
+	std::string msg;
+	if(message.find("/me ") == 0) {
+		msg.assign(message,4,message.size());
+		action = true;
+	} else {
+		msg = message;
+		action = false;
+	}
 	gui::text_to_lines(msg,80);
 
 	int ypos = chat_message_x;
@@ -2207,14 +2239,23 @@ void display::add_chat_message(const std::string& speaker, int side, const std::
 	}
 
 	std::stringstream str;
-	if(type == MESSAGE_PUBLIC) {
-		str << "<" << speaker << ">";
-	} else {
-		str << "*" << speaker << "*";
-	}
-
 	std::stringstream message_str;
 	message_str << msg;
+	if(type == MESSAGE_PUBLIC) {
+		if(action) {
+			str << "<" << speaker;
+			message_str << ">";
+		} else {
+			str << "<" << speaker << ">";
+		}
+	} else {
+		if(action) {
+			str << "*" << speaker;
+			message_str << "*";
+		} else {
+			str << "*" << speaker << "*";
+		}
+	}
 
 	SDL_Color speaker_colour = {255,255,255,255};
 	if(side >= 1) {
