@@ -1,4 +1,4 @@
-/* $Id: display.hpp,v 1.83 2004/09/11 21:22:57 gruikya Exp $ */
+/* $Id: display.hpp,v 1.92 2005/02/06 10:40:12 isaaccp Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -55,7 +55,7 @@ public:
 	display(unit_map& units, CVideo& video,
 			const gamemap& map, const gamestatus& status,
 			const std::vector<team>& t, const config& theme_cfg,
-			const config& config, const config& level);
+			const config& cfg, const config& level);
 	~display();
 
 	Uint32 rgb(Uint8 red, Uint8 green, Uint8 blue);
@@ -135,11 +135,16 @@ public:
 	//over
 	void highlight_hex(gamemap::location hex);
 
-	//given x,y co-ordinates of a pixel, will return the location of the
-	//hex that this pixel corresponds to. Returns an invalid location is
-	//the mouse isn't over any valid location.
-	gamemap::location hex_clicked_on(int x, int y);
-
+	//given x,y co-ordinates of an onscreen pixel, will return the
+	//location of the hex that this pixel corresponds to. Returns an
+	//invalid location is the mouse isn't over any valid location.
+	gamemap::location hex_clicked_on(int x, int y, gamemap::location::DIRECTION* nearest_hex=NULL);
+	
+	//given x,y co-ordinates of a pixel on the map, will return the
+	//location of the hex that this pixel corresponds to. Returns an
+	//invalid location is the mouse isn't over any valid location.
+	gamemap::location pixel_position_to_hex(int x, int y, gamemap::location::DIRECTION* nearest_hex=NULL);
+	
 	//given x,y co-ordinates of the mouse, will return the location of the
 	//hex in the minimap that the mouse is currently over, or an invalid
 	//location if the mouse isn't over the minimap.
@@ -297,7 +302,7 @@ public:
 
 	const theme& get_theme() const;
 
-	const theme::menu* menu_pressed(int mousex, int mousey, bool button_pressed);
+	const theme::menu* menu_pressed();
 
 	void add_observer(const std::string& name);
 	void remove_observer(const std::string& name);
@@ -360,14 +365,13 @@ private:
 
 	void draw_sidebar();
 	void draw_minimap(int x, int y, int w, int h);
-	void draw_game_status(int x, int y);
+	void draw_game_status();
 
 	SDL_Rect gameStatusRect_;
 	SDL_Rect unitDescriptionRect_;
 	SDL_Rect unitProfileRect_;
 
-	void draw_image_for_report(surface& img, 
-			surface& surf, SDL_Rect& rect);
+	void draw_image_for_report(surface& img, SDL_Rect& rect);
 	void draw_report(reports::TYPE report_num);
 	SDL_Rect reportRects_[reports::NUM_REPORTS];
 	surface reportSurfaces_[reports::NUM_REPORTS];
@@ -379,14 +383,6 @@ private:
 	std::vector<surface> get_terrain_images(int x, int y, image::TYPE type, ADJACENT_TERRAIN_TYPE terrain_type);
 	std::vector<std::string> get_fog_shroud_graphics(const gamemap::location& loc);
 
-
-	//this surface must be freed by the caller
-	//surface get_terrain(gamemap::TERRAIN, image::TYPE type,
-	//                        int x, int y, const std::string& dir="");
-	//this surface must be freed by the caller
-	surface get_terrain(const image::locator &image, image::TYPE type,
-	                        int x, int y);
-
 	//this surface must be freed by the caller
 	surface get_flag(gamemap::TERRAIN, int x, int y);
 
@@ -512,6 +508,9 @@ private:
 
 	//animated flags for each team
 	std::vector<animated<image::locator> > flags_;
+
+	//the handle for the label which displays fps
+	int fps_handle_;
 };
 
 //an object which will lock the display for the duration of its lifetime.
