@@ -103,22 +103,18 @@ map_editor::map_editor(display &gui, gamemap &map, config &theme, config &game_c
 			std::cerr << "Error when reading " << prefs_filename << ": "
 					  << e.message << std::endl;
 		}
-		// Clear the current hotkeys. Alot of hotkeys are already set
-		// through other configuration files (e.g. english.cfg and
-		// preferences) and we need to clear these or they will overlap.
-		hotkey::get_hotkeys().clear();
-		hotkey::add_hotkeys(theme_, true);
-		hotkey::add_hotkeys(prefs_, true);
+		hotkey::load_hotkeys(theme_);
+		hotkey::load_hotkeys(prefs_);
 		left_button_func_changed(DRAW);
 		first_time_created_ = false;
 	}
 	else {
-		hotkey::get_hotkeys().clear();
-		hotkey::add_hotkeys(hotkeys_, true);
+		hotkey::load_hotkeys(hotkeys_);
 		palette_.select_fg_terrain(old_fg_terrain_);
 		palette_.select_bg_terrain(old_bg_terrain_);
 		brush_.select_brush_size(old_brush_size_);
 	}
+	hotkey::load_descriptions();
 	recalculate_starting_pos_labels();
 	gui_.invalidate_game_status();
 	gui_.begin_game();
@@ -159,7 +155,7 @@ void map_editor::handle_event(const SDL_Event &event) {
 }
 
 void map_editor::handle_keyboard_event(const SDL_KeyboardEvent &event,
-									   const int /*mousex*/, const int /*mousey*/) {
+                                       const int /*mousex*/, const int /*mousey*/) {
 	if (event.type == SDL_KEYDOWN) {
 		const SDLKey sym = event.keysym.sym;
 		// We must intercept escape-presses here because we don't want the
@@ -184,7 +180,7 @@ void map_editor::handle_keyboard_event(const SDL_KeyboardEvent &event,
 }
 
 void map_editor::handle_mouse_button_event(const SDL_MouseButtonEvent &event,
-										   const int mousex, const int mousey) {
+                                           const int mousex, const int mousey) {
 	if (event.type == SDL_MOUSEBUTTONDOWN) {
 		const Uint8 button = event.button;
 		if (button == SDL_BUTTON_RIGHT) {
@@ -302,17 +298,16 @@ void map_editor::right_click(const gamemap::location hex_clicked ) {
 void map_editor::edit_save_as() {
 	const std::string default_dir =
 		get_dir(get_dir(get_user_data_dir() + "/editor") + "/maps/");
-	std::string input_name = filename_ == "" ? default_dir : filename_;
+	std::string input_name = filename_.empty() ? default_dir : filename_;
 	int res = 0;
 	int overwrite = 0;
 	do {
-		res = dialogs::show_file_chooser_dialog(gui_, input_name,
-												_("Choose a Map to Load"));
+		res = dialogs::show_file_chooser_dialog(gui_, input_name, _("Save the Map As"));
 		if (res == 0) {
 			if (file_exists(input_name)) {
-				overwrite = gui::show_dialog(gui_, NULL,
-											 "", _("The map already exists. Do you want to overwrite it?"),
-											 gui::YES_NO);
+				overwrite = gui::show_dialog(gui_, NULL, "",
+					_("The map already exists. Do you want to overwrite it?"),
+					gui::YES_NO);
 			}
 			else
 				overwrite = 0;
@@ -333,8 +328,8 @@ void map_editor::perform_set_starting_pos() {
 		players.push_back(str.str());
 	}
 	const int res = gui::show_dialog(gui_, NULL, _("Which Player?"),
-									 _("Which player should start here?"),
-									 gui::OK_CANCEL, &players);
+	                                 _("Which player should start here?"),
+	                                 gui::OK_CANCEL, &players);
 	if (res >= 0) {
 		set_starting_position(res, selected_hex_);
 	}
@@ -372,7 +367,7 @@ void map_editor::edit_quit() {
 
 void map_editor::edit_new_map() {
 	const std::string map = new_map_dialog(gui_, palette_.selected_bg_terrain(),
-										   changed_since_save(), game_config_);
+	                                       changed_since_save(), game_config_);
  	if (map != "") {
 		num_operations_since_save_ = 0;
 		clear_undo_actions();
@@ -381,10 +376,9 @@ void map_editor::edit_new_map() {
 }
 
 void map_editor::edit_load_map() {
-	std::string fn = filename_ == "" ?
+	std::string fn = filename_.empty() ?
 		get_dir(get_dir(get_user_data_dir() + "/editor") + "/maps/") : filename_;
-	int res = dialogs::show_file_chooser_dialog(gui_, fn,
-												_("Choose a Map to Load"));
+	int res = dialogs::show_file_chooser_dialog(gui_, fn, _("Choose a Map to Load"));
 	if (res == 0) {
 		std::string new_map;
 		try {
@@ -448,7 +442,7 @@ void map_editor::perform_paste() {
 			const int start_side = (*it).starting_side;
 			if (start_side != -1) {
 				undo_action.add_starting_location(start_side, start_side,
-												  map_.starting_position(start_side), target);
+					map_.starting_position(start_side), target);
 				map_.set_starting_position(start_side, target);
 			}
 			filled.insert(target);
@@ -582,7 +576,7 @@ void map_editor::insert_selection_in_clipboard() {
 		const int y_offset = (*it).y - offset_hex.y;
 		gamemap::TERRAIN terrain = map_.get_terrain(*it);
 		clipboard_.push_back(clipboard_item(x_offset, y_offset, terrain,
-											starting_side_at(map_, *it)));
+		                     starting_side_at(map_, *it)));
 	}
 }
 
@@ -745,7 +739,7 @@ void map_editor::set_starting_position(const int player, const gamemap::location
 	}
 	else {
 		gui::show_dialog(gui_, NULL, "",
-						 _("You must have a hex selected on the board."), gui::OK_ONLY);
+		                 _("You must have a hex selected on the board."), gui::OK_ONLY);
 	}
 }
 
@@ -758,7 +752,7 @@ void map_editor::set_file_to_save_as(const std::string filename) {
 }
 
 gamemap::location map_editor::get_hex_with_offset(const gamemap::location loc,
-												  const int x_offset, const int y_offset) {
+                                                  const int x_offset, const int y_offset) {
 	gamemap::location new_loc(loc.x + x_offset, loc.y + y_offset);
 	if (new_loc.x % 2 != loc.x % 2 && is_odd(new_loc.x)) {
 		new_loc.y--;
@@ -900,8 +894,7 @@ void map_editor::perform_selection_move() {
 	save_undo_action(undo_action);
 }
 
-void map_editor::draw_terrain(const gamemap::TERRAIN terrain,
-							  const gamemap::location hex) {
+void map_editor::draw_terrain(const gamemap::TERRAIN terrain, const gamemap::location hex) {
 	const gamemap::TERRAIN current_terrain = map_.get_terrain(hex);
 	map_undo_action undo_action;
 	undo_action.add_terrain(current_terrain, terrain, hex);
@@ -911,8 +904,7 @@ void map_editor::draw_terrain(const gamemap::TERRAIN terrain,
 	save_undo_action(undo_action);
 }
 
-void map_editor::terrain_changed(const gamemap::location &hex,
-								 map_undo_action &undo_action) {
+void map_editor::terrain_changed(const gamemap::location &hex, map_undo_action &undo_action) {
 	std::vector<gamemap::location> v;
 	v.push_back(hex);
 	terrain_changed(v, undo_action);
@@ -927,8 +919,7 @@ void map_editor::terrain_changed(const std::vector<gamemap::location> &hexes,
 			// A terrain which had a starting position has changed, save
 			// this position in the undo_action and unset it.
 			map_.set_starting_position(start_side, gamemap::location());
-			undo_action.add_starting_location(start_side, start_side,
-											  *it, gamemap::location());
+			undo_action.add_starting_location(start_side, start_side, *it, gamemap::location());
 		}
 	}
 	invalidate_all_and_adjacent(hexes);
@@ -997,12 +988,12 @@ void map_editor::middle_button_down(const int mousex, const int mousey) {
 
 bool map_editor::confirm_exit_and_save() {
 	if (gui::show_dialog(gui_, NULL, "",
-						 _("Quit Editor"), gui::YES_NO) != 0) {
+	                     _("Quit Editor"), gui::YES_NO) != 0) {
 		return false;
 	}
 	if (changed_since_save() &&
-		gui::show_dialog(gui_, NULL, "",
-						 _("Do you want to save the map before quitting?"), gui::YES_NO) == 0) {
+	    gui::show_dialog(gui_, NULL, "",
+		             _("Do you want to save the map before quitting?"), gui::YES_NO) == 0) {
 		if (!save_map("", false)) {
 			return false;
 		}
@@ -1042,36 +1033,25 @@ void map_editor::show_menu(const std::vector<std::string>& items_arg, const int
 	// menu is what to display in the menu.
 	std::vector<std::string> menu;
 	if(items.size() == 1) {
-		execute_command(hotkey::string_to_command(items.front()));
+		execute_command(hotkey::get_hotkey(items.front()).get_id());
 		return;
 	}
 	for(std::vector<std::string>::const_iterator i = items.begin();
 		i != items.end(); ++i) {
+
+		const hotkey::hotkey_item hk = hotkey::get_hotkey(*i);
 		std::stringstream str;
 		// Try to translate it to nicer format.
-		str << translate_string("action_" + *i);
-		// See if this menu item has an associated hotkey.
-		const hotkey::HOTKEY_COMMAND cmd = hotkey::string_to_command(*i);
-		const std::vector<hotkey::hotkey_item>& hotkeys = hotkey::get_hotkeys();
-		std::vector<hotkey::hotkey_item>::const_iterator hk;
-		for(hk = hotkeys.begin(); hk != hotkeys.end(); ++hk) {
-			if(hk->action == cmd) {
-				break;
-			}
-		}
-		if(hk != hotkeys.end()) {
-			// Hotkey was found for this item, add the hotkey description to
-			// the menu item.
-			str << "," << hotkey::get_hotkey_name(*hk);
-		}
+		str << hk.get_description() << COLUMN_SEPARATOR << hk.get_name();
+
 		menu.push_back(str.str());
 	}
 	static const std::string style = "menu2";
 	const int res = gui::show_dialog(gui_, NULL, "", "", gui::MESSAGE, &menu, NULL, "",
-									 NULL, NULL, NULL, xloc, yloc, &style);
+	                                 NULL, 256, NULL, NULL, xloc, yloc, &style);
 	if(res < 0 || (unsigned)res >= items.size())
 		return;
-	const hotkey::HOTKEY_COMMAND cmd = hotkey::string_to_command(items[res]);
+	const hotkey::HOTKEY_COMMAND cmd = hotkey::get_hotkey(items[res]).get_id();
 	execute_command(cmd);
 }
 
@@ -1130,9 +1110,8 @@ void map_editor::update_mouse_over_hexes(const gamemap::location mouse_over_hex)
 void map_editor::left_button_func_changed(const LEFT_BUTTON_FUNC func) {
 	if (func != l_button_func_) {
 		l_button_func_ = func;
-		const std::string string_to_translate = std::string("action_") + get_action_name(func);
 		reports::set_report_content(reports::EDIT_LEFT_BUTTON_FUNCTION,
-					    translate_string(string_to_translate));
+				hotkey::get_hotkey(get_action_name(func)).get_description());
 		gui_.invalidate_game_status();
 		l_button_palette_dirty_ = true;
 	}
@@ -1217,8 +1196,7 @@ void map_editor::main_loop() {
 			update_mouse_over_hexes(cur_hex);
 			last_brush_size = brush_.selected_brush_size();
 		}
-		const theme::menu* const m = gui_.menu_pressed(mousex, mousey,
-													   mouse_flags & SDL_BUTTON_LMASK);
+		const theme::menu* const m = gui_.menu_pressed();
 		if (m != NULL) {
 			const SDL_Rect& menu_loc = m->location(gui_.screen_area());
 			const int x = menu_loc.x + 1;
