@@ -1,4 +1,4 @@
-/* $Id: filesystem.cpp,v 1.45 2004/09/18 21:42:13 Sirp Exp $ */
+/* $Id: filesystem.cpp,v 1.59 2005/01/28 23:38:52 ydirson Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -11,10 +11,7 @@
    See the COPYING file for more details.
 */
 
-//disable the very annoying VC++ warning 4786
-#ifdef WIN32
-#pragma warning(disable:4786)
-#endif
+#include "global.hpp"
 
 //include files for opendir(3), readdir(3), etc. These files may vary
 //from platform to platform, since these functions are NOT ANSI-conforming
@@ -62,8 +59,68 @@ BPath be_path;
 #include "config.hpp"
 #include "filesystem.hpp"
 #include "game_config.hpp"
+#include "log.hpp"
+#include "scoped_resource.hpp"
 #include "util.hpp"
 
+#define LOG_G lg::info(lg::general)
+
+#ifdef USE_ZIPIOS
+#include <sstream>
+#include <zipios++/collcoll.h>
+#include <zipios++/dircoll.h>
+#include <zipios++/zipfile.h>
+#include <zipios++/simplesmartptr.h>
+#include <zipios++/basicentry.h>
+#include "zipios++/xcoll.hpp"
+
+namespace {
+	xzipios::XCColl the_collection;
+
+	void register_zipdir(const std::string directory) {
+		// look for zip files
+		zipios::DirectoryCollection dir(directory,false);
+		zipios::ConstEntries entries = dir.entries();
+		for (zipios::ConstEntries::iterator i = entries.begin(); i != entries.end(); ++i) {
+			if ((**i).isValid()) {
+				const std::string fname = (**i).getName();
+				const std::string suffix = ".zip";
+				if ((fname.size() > suffix.size()) &&
+				    (0 == fname.compare(fname.size() - suffix.size(), suffix.size(), suffix))) {
+					zipios::ZipFile zip(game_config::path + "/" + fname);
+					the_collection.addCollection(zip);
+					LOG_G << "zip collection " << fname << 
+						" has " << zip.size() << " elements\n";
+				}
+			}
+			else LOG_G << "skipping invalid entry\n";
+		}
+		
+	}
+}
+#endif
+
+bool filesystem_init()
+{
+#ifdef USE_ZIPIOS
+	if (!get_user_data_dir().empty()) {
+		LOG_G << "looking at user dir " << get_user_data_dir() << "\n";
+		zipios::DirectoryCollection dir(get_user_data_dir());
+		LOG_G << "user collection has " << dir.size() << " elements\n";
+		the_collection.addCollection(dir);
+		register_zipdir(get_user_data_dir());
+	}
+	if (!game_config::path.empty()) {
+		LOG_G << "looking at system dir " << game_config::path << "\n";
+		zipios::DirectoryCollection dir(game_config::path);
+		LOG_G << "system collection has " << dir.size() << " elements\n";
+		the_collection.addCollection(dir);
+		register_zipdir(game_config::path);
+	}
+#endif
+	return true;
+}
+
 namespace {
 	const mode_t AccessMode = 00770;
 }
@@ -79,80 +136,89 @@ void get_files_in_dir(const std::string& directory,
                       std::vector<std::string>* dirs,
                       FILE_NAME_MODE mode)
 {
-	//if we have a path to find directories in, then convert relative
-	//pathnames to be rooted on the wesnoth path
-	if(!directory.empty() && directory[0] != '/' && !game_config::path.empty()){
-		const std::string& dir = game_config::path + "/" + directory;
-		if(is_directory(dir)) {
-			get_files_in_dir(dir,files,dirs,mode);
+#ifdef USE_ZIPIOS
+	if (the_collection.hasSubdir(directory)) {
+		the_collection.childrenOf(directory, files, dirs);
+	}
+	else
+#endif
+	{
+		
+		//if we have a path to find directories in, then convert relative
+		//pathnames to be rooted on the wesnoth path
+		if(!directory.empty() && directory[0] != '/' && !game_config::path.empty()){
+			const std::string& dir = game_config::path + "/" + directory;
+			if(is_directory(dir)) {
+				get_files_in_dir(dir,files,dirs,mode);
+				return;
+			}
+		}
+
+#ifdef _WIN32
+		_finddata_t fileinfo;
+		long dir = _findfirst((directory + "/*.*").c_str(),&fileinfo);
+#else
+
+		DIR* dir = opendir(directory.c_str());
+#endif
+
+		if(DIR_INVALID(dir)) {
 			return;
 		}
-	}
-
-#ifdef _WIN32
-	_finddata_t fileinfo;
-	long dir = _findfirst((directory + "/*.*").c_str(),&fileinfo);
-#else
-
-	DIR* dir = opendir(directory.c_str());
-#endif
-
-	if(DIR_INVALID(dir)) {
-		return;
-	}
 
 #ifdef _WIN32
 
-	int res = dir;
-	do {
-		if(fileinfo.name[0] != '.') {
-			const std::string path = (mode == ENTIRE_FILE_PATH ?
-				directory + "/" : std::string("")) + fileinfo.name;
+		int res = dir;
+		do {
+			if(fileinfo.name[0] != '.') {
+				const std::string path = (mode == ENTIRE_FILE_PATH ?
+							  directory + "/" : std::string("")) + fileinfo.name;
 
-			if(fileinfo.attrib&_A_SUBDIR) {
-				if(dirs != NULL)
-					dirs->push_back(path);
-			} else {
-				if(files != NULL)
-					files->push_back(path);
+				if(fileinfo.attrib&_A_SUBDIR) {
+					if(dirs != NULL)
+						dirs->push_back(path);
+				} else {
+					if(files != NULL)
+						files->push_back(path);
+				}
 			}
-		}
 
-		res = _findnext(dir,&fileinfo);
-	} while(!DIR_INVALID(res));
+			res = _findnext(dir,&fileinfo);
+		} while(!DIR_INVALID(res));
 
-	_findclose(dir);
+		_findclose(dir);
 
 #else
-	struct dirent* entry;
-	while((entry = readdir(dir)) != NULL) {
-		if(entry->d_name[0] == '.')
-			continue;
+		struct dirent* entry;
+		while((entry = readdir(dir)) != NULL) {
+			if(entry->d_name[0] == '.')
+				continue;
 
-		const std::string name((directory + "/") + entry->d_name);
+			const std::string name((directory + "/") + entry->d_name);
 
-		struct stat st;
-		if (::stat(name.c_str(), &st) != -1) {
-			if (S_ISREG(st.st_mode)) {
-				if (files != NULL) {
-					if (mode == ENTIRE_FILE_PATH)
-						files->push_back(name);
-					else
-						files->push_back(entry->d_name);
-				}
-			} else if (S_ISDIR(st.st_mode)) {
-				if (dirs != NULL) {
-					if (mode == ENTIRE_FILE_PATH)
-						dirs->push_back(name);
-					else
-						dirs->push_back(entry->d_name);
+			struct stat st;
+			if (::stat(name.c_str(), &st) != -1) {
+				if (S_ISREG(st.st_mode)) {
+					if (files != NULL) {
+						if (mode == ENTIRE_FILE_PATH)
+							files->push_back(name);
+						else
+							files->push_back(entry->d_name);
+					}
+				} else if (S_ISDIR(st.st_mode)) {
+					if (dirs != NULL) {
+						if (mode == ENTIRE_FILE_PATH)
+							dirs->push_back(name);
+						else
+							dirs->push_back(entry->d_name);
+					}
 				}
 			}
 		}
-	}
 
-	closedir(dir);
+		closedir(dir);
 #endif
+	}
 
 	if(files != NULL)
 		std::sort(files->begin(),files->end());
@@ -185,6 +251,9 @@ std::string get_cache_dir()
 
 std::string get_intl_dir()
 {
+#ifdef _WIN32
+	return get_cwd() + "/po";
+#endif
 
 #ifdef __APPLE__
 	return get_cwd() + "/messages";
@@ -327,6 +396,115 @@ std::string get_user_data_dir()
 #endif
 }
 
+namespace {
+void read_file_internal(const std::string& fname, std::string& res)
+{
+	const int size = file_size(fname);
+	if(size < 0) {
+		return;
+	}
+
+	std::vector<char> v;
+	v.reserve(size);
+
+	//const util::scoped_resource<FILE*,close_FILE> file(fopen(fname.c_str(),"rb"));
+	const util::scoped_FILE file(fopen(fname.c_str(),"rb"));
+
+	if(file == NULL) {
+		return;
+	}
+
+	const int block_size = 65536;
+
+	while(v.size() < size) {
+		const size_t expected = minimum<size_t>(block_size,size - v.size());
+
+		if(expected > 0) {
+			v.resize(v.size() + expected);
+			const size_t nbytes = fread(&v[v.size() - expected],1,expected,file);
+			if(nbytes < expected) {
+				v.resize(v.size() - (expected - nbytes));
+				break;
+			}
+		}
+	}
+
+	res.resize(v.size());
+	std::copy(v.begin(),v.end(),res.begin());
+}
+} //end anon namespace
+
+std::string read_stream(std::istream& s)
+{
+	std::stringstream ss;
+	ss << s.rdbuf();
+	return ss.str();
+}
+
+std::string read_stdin()
+{
+	return read_stream(std::cin);
+}
+
+std::string read_file(const std::string& fname)
+{
+	//if we have a path to the data,
+	//convert any filepath which is relative
+	LOG_G << "Reading " << fname << "\n";
+#ifdef USE_ZIPIOS
+	if(!fname.empty() && fname[0] != '/') {
+		zipios::ConstEntryPointer p = the_collection.getEntry(fname);
+		if (p != 0) {
+			std::istream* s = the_collection.getInputStream(p);
+			if (s != NULL) {
+				std::string contents = read_stream(*s);
+				delete s;
+				return contents;
+			}
+		}
+	}
+#else
+	if(!fname.empty() && fname[0] != '/' && !game_config::path.empty()) {
+		std::string res;
+		read_file_internal(game_config::path + "/" + fname,res);
+		if(!res.empty()) {
+			return res;
+		}
+	}
+#endif
+
+	// FIXME: why do we rely on this even with relative paths ?
+	{
+		// still useful with zipios, for things like cache and prefs
+		std::string res;
+		read_file_internal(fname,res);
+		return res;
+	}
+}
+
+//throws io_exception if an error occurs
+void write_file(const std::string& fname, const std::string& data)
+{
+	//const util::scoped_resource<FILE*,close_FILE> file(fopen(fname.c_str(),"wb"));
+	const util::scoped_FILE file(fopen(fname.c_str(),"wb"));
+	if(file.get() == NULL) {
+		throw io_exception("Could not open file for writing: '" + fname + "'");
+	}
+
+	const size_t block_size = 4096;
+	char buf[block_size];
+
+	for(size_t i = 0; i < data.size(); i += block_size) {
+		const size_t bytes = minimum<size_t>(block_size,data.size() - i);
+		std::copy(data.begin() + i, data.begin() + i + bytes,buf);
+		const size_t res = fwrite(buf,1,bytes,file.get());
+		if(res != bytes) {
+			throw io_exception("Error writing to file: '" + fname + "'");
+		}
+	}
+}
+
+
 std::string read_map(const std::string& name)
 {
 	std::string res = read_file("data/maps/" + name);
@@ -369,16 +547,27 @@ bool is_directory_internal(const std::string& fname)
 
 bool is_directory(const std::string& fname)
 {
+#ifdef USE_ZIPIOS
+	if(!fname.empty() && fname[0] != '/') {
+		return the_collection.hasSubdir(fname);
+	}
+
+#else
 	if(!fname.empty() && fname[0] != '/' && !game_config::path.empty()) {
 		if(is_directory_internal(game_config::path + "/" + fname))
 			return true;
 	}
+#endif
 
 	return is_directory_internal(fname);
 }
 
 bool file_exists(const std::string& name)
 {
+#ifdef USE_ZIPIOS
+	if (the_collection.getEntry(name))
+		return true;
+#endif
 	std::ifstream file(name.c_str());
 	if (file.rdstate() != 0)
 	        return false;
@@ -464,9 +653,9 @@ const file_tree_checksum& data_tree_checksum()
 	if(checksum.nfiles == 0) {
 		get_file_tree_checksum_internal("data/",checksum);
 		get_file_tree_checksum_internal(get_user_data_dir() + "/data/",checksum);
-		std::cerr << "calculated data tree checksum: "
-		          << checksum.nfiles << " files; "
-		          << checksum.sum_size << " bytes\n";
+		LOG_G << "calculated data tree checksum: "
+		      << checksum.nfiles << " files; "
+		      << checksum.sum_size << " bytes\n";
 	}
 
 	return checksum;
