@@ -1,4 +1,4 @@
-/* $Id: font.cpp,v 1.86 2004/09/18 21:24:59 silene Exp $ */
+/* $Id: font.cpp,v 1.97 2005/01/08 21:49:29 silene Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -11,8 +11,11 @@
    See the COPYING file for more details.
 */
 
-#include "config.hpp"
+#include "wesconfig.h"
+#include "global.hpp"
+
 #include "font.hpp"
+#include "filesystem.hpp"
 #include "game_config.hpp"
 #include "language.hpp"
 #include "log.hpp"
@@ -46,14 +49,15 @@ TTF_Font* open_font(const std::string& fname, int size)
 	std::string name;
 
 	LOG_FT << "Opening font '" << fname << "' ...\n";
+#ifndef USE_ZIPIOS
 	if(game_config::path.empty() == false) {
 		name = game_config::path + "/fonts/" + fname;
 		LOG_FT << "Trying file '" << name << "' ...\n";
 
-		if(read_file(name).empty()) {
+		if (!file_exists(name)) {
 			name = "fonts/" + fname;
 			WRN_FT << "Failed opening '" << name << "'; now trying '" << name << "' ...\n";
-			if(read_file(name).empty()) {
+			if (!file_exists(name)) {
 				ERR_FT << "Failed opening font: " << fname << "\n";
 				return NULL;
 			}
@@ -62,7 +66,7 @@ TTF_Font* open_font(const std::string& fname, int size)
 		name = "fonts/" + fname;
 		LOG_FT << "Trying file '" << name << "' ...\n";
 
-		if(read_file(name).empty()) {
+		if(!file_exists(name)) {
 			ERR_FT << "Failed opening font '" << fname << "'\n";
 			return NULL;
 		}
@@ -71,6 +75,17 @@ TTF_Font* open_font(const std::string& fname, int size)
 	LOG_FT << "Opening font file '" << name << "', font size is " << size << "\n";
 
 	TTF_Font* font = TTF_OpenFont(name.c_str(),size);
+#else
+	std::string tmp = read_file("fonts/" + fname);
+	if (tmp.empty()) {
+		ERR_FT << "Failed opening font file '" << fname << "'\n";
+		return NULL;
+	}
+	// the following statement would leak memory if fonts were closed
+	std::string *s = new std::string(tmp);
+	SDL_RWops* ops = SDL_RWFromMem((void*)s->c_str(), s->size());
+	TTF_Font* font = TTF_OpenFontRW(ops, 0, size);
+#endif
 	if(font == NULL) {
 		ERR_FT << "Failed opening font file '" << name << "'\n";
 		return NULL;
@@ -122,11 +137,13 @@ struct font_style_setter
 		//SDL_ttf to segfault. We work around this here by disallowing combinations
 		//of styles
 		if((style&TTF_STYLE_UNDERLINE) != 0) {
-			style = TTF_STYLE_NORMAL; //TTF_STYLE_UNDERLINE;
+			//style = TTF_STYLE_NORMAL; //TTF_STYLE_UNDERLINE;
+			style = TTF_STYLE_UNDERLINE;
 		} else if((style&TTF_STYLE_BOLD) != 0) {
 			style = TTF_STYLE_BOLD;
 		} else if((style&TTF_STYLE_ITALIC) != 0) {
-			style = TTF_STYLE_NORMAL; //TTF_STYLE_ITALIC;
+			//style = TTF_STYLE_NORMAL; //TTF_STYLE_ITALIC;
+			style = TTF_STYLE_ITALIC;
 		} 
 
 		TTF_SetFontStyle(font_,style);
@@ -273,6 +290,16 @@ surface const &text_surface::get_surface() const
 
 	// Validate the UTF-8 string: workaround a SDL_TTF bug that makes it
 	// crash when used with an invalid UTF-8 string
+	wide_string ws = string_to_wstring(str_);
+
+	for(wide_string::const_iterator itor = ws.begin(); itor != ws.end(); ++itor) {
+		int minx, miny, maxx, maxy, advance;	
+
+		if(TTF_GlyphMetrics(font_, *itor, &minx, &maxx, &miny, &maxy, &advance) != 0 ) {
+			std::cerr << "glyph with strange size: " << *itor << "(" << char(*itor) << ") " << minx << ", " << maxx  << ", " << miny << ", " << maxy << ", " << advance << "\n";
+		}
+	}
+
 	std::string fixed_str = wstring_to_string(string_to_wstring(str_));
 
 	font_style_setter const style_setter(font_, style_);
@@ -350,7 +377,7 @@ surface render_text(TTF_Font* font,const std::string& text, const SDL_Color& col
 		for(std::vector< surface >::const_iterator i = surfaces.begin(),
 		    i_end = surfaces.end(); i != i_end; ++i) {
 			SDL_SetAlpha(*i, 0, 0); // direct blit without alpha blending
-			SDL_Rect dstrect = {0, ypos};
+			SDL_Rect dstrect = {0, ypos, 0, 0};
 			SDL_BlitSurface(*i, NULL, res, &dstrect);
 			ypos += (*i)->h;
 		}
@@ -428,7 +455,7 @@ surface get_rendered_text(const std::string& str, int size, const SDL_Color& col
 
 SDL_Rect draw_text_line(surface gui_surface, const SDL_Rect& area, int size,
 		   const SDL_Color& colour, const std::string& text,
-		   int x, int y, surface bg, bool use_tooltips, int style)
+		   int x, int y, bool use_tooltips, int style)
 {
 	
 	TTF_Font* const font = get_font(size);
@@ -492,7 +519,7 @@ SDL_Rect draw_text_line(surface gui_surface, const SDL_Rect& area, int size,
 
 SDL_Rect draw_text_line(display* gui, const SDL_Rect& area, int size,
                         const SDL_Color& colour, const std::string& text,
-                        int x, int y, surface bg, bool use_tooltips, int style)
+                        int x, int y, bool use_tooltips, int style)
 {
 	surface surface;
 	
@@ -502,19 +529,18 @@ SDL_Rect draw_text_line(display* gui, const SDL_Rect& area, int size,
 		surface = gui->video().getSurface();
 	}
 	
-	return draw_text_line(surface, area, size, colour, text, x, y, bg, use_tooltips, style);
+	return draw_text_line(surface, area, size, colour, text, x, y, use_tooltips, style);
 }
 
 SDL_Rect text_area(const std::string& text, int size, int style)
 {
 	const SDL_Rect area = {0,0,10000,10000};
-	return draw_text(NULL,area,size,font::NORMAL_COLOUR,text,0,0,NULL,false,USE_MARKUP,style);
+	return draw_text(NULL, area, size, font::NORMAL_COLOUR, text, 0, 0, false, style);
 }
 
 SDL_Rect draw_text(display* gui, const SDL_Rect& area, int size,
                    const SDL_Color& colour, const std::string& txt,
-                   int x, int y, surface bg, bool use_tooltips,
-                   MARKUP use_markup, int style)
+                   int x, int y, bool use_tooltips, int style)
 {
 	//make sure there's always at least a space, so we can ensure
 	//that we can return a rectangle for height
@@ -541,7 +567,7 @@ SDL_Rect draw_text(display* gui, const SDL_Rect& area, int size,
 
 			config::unescape(new_string);
 
-			const SDL_Rect rect = draw_text_line(gui,area,sz,col,new_string,x,y,bg,use_tooltips,text_style);
+			const SDL_Rect rect = draw_text_line(gui, area, sz, col, new_string, x, y, use_tooltips, text_style);
 			if(rect.w > res.w) {
 				res.w = rect.w;
 			}
@@ -706,10 +732,10 @@ namespace font {
 	
 	SDL_Rect draw_wrapped_text(display* gui, const SDL_Rect& area, int font_size,
 			     const SDL_Color& colour, const std::string& text,
-			     int x, int y, int max_width, surface bg)
+			     int x, int y, int max_width)
 	{
 		std::string wrapped_text = word_wrap_text(text, font_size, max_width);
-		return font::draw_text(gui, area, font_size, colour, wrapped_text, x, y, bg, false, NO_MARKUP);
+		return font::draw_text(gui, area, font_size, colour, wrapped_text, x, y, false);
 	}
 
 }
@@ -721,11 +747,12 @@ class floating_label
 {
 public:
 	floating_label(const std::string& text, int font_size, const SDL_Color& colour, const SDL_Color& bgcolour,
-		double xpos, double ypos, double xmove, double ymove, int lifetime, const SDL_Rect& clip_rect, font::ALIGN align,
-		int border_size, bool scroll_with_map)
-		  : surf_(NULL), buf_(NULL), foreground_(NULL), text_(text), font_size_(font_size), colour_(colour), bgcolour_(bgcolour), bgalpha_(bgcolour.unused), xpos_(xpos), ypos_(ypos),
-		    xmove_(xmove), ymove_(ymove), lifetime_(lifetime), clip_rect_(clip_rect),
-			alpha_change_(-255/lifetime), visible_(true), align_(align), border_(border_size), scroll_(scroll_with_map)
+			double xpos, double ypos, double xmove, double ymove, int lifetime, const SDL_Rect& clip_rect,
+			font::ALIGN align, int border_size, bool scroll_with_map)
+		: surf_(NULL), buf_(NULL), foreground_(NULL), text_(text), font_size_(font_size), colour_(colour),
+		bgcolour_(bgcolour), bgalpha_(bgcolour.unused), xpos_(xpos), ypos_(ypos),
+		xmove_(xmove), ymove_(ymove), lifetime_(lifetime), clip_rect_(clip_rect),
+		alpha_change_(-255/lifetime), visible_(true), align_(align), border_(border_size), scroll_(scroll_with_map)
 	{}
 
 	void move(double xmove, double ymove);
@@ -789,15 +816,15 @@ int floating_label::xpos(size_t width) const
 surface floating_label::create_surface()
 {
 	if (surf_.null()) {
-		surf_ = font::render_text(get_font(font_size_), text_, colour_, 0);
+		foreground_ = font::render_text(get_font(font_size_), text_, colour_, 0);
 
-		if(surf_ == NULL) {
+		if(foreground_ == NULL) {
 			return NULL;
 		}
 
 		//if the surface has to be created onto some kind of background, then do that here
 		if(bgalpha_ != 0) {
-			surface tmp(create_compatible_surface(surf_,surf_->w+border_*2,surf_->h+border_*2));
+			surface tmp(create_compatible_surface(foreground_,foreground_->w+border_*2,foreground_->h+border_*2));
 			if(tmp == NULL) {
 				return NULL;
 			}
@@ -810,9 +837,15 @@ surface floating_label::create_surface()
 				}
 			}
 
-			foreground_.assign(surf_);
 			surf_.assign(tmp);
+		} else {
+			surface background = font::render_text(get_font(font_size_), text_, font::BLACK_COLOUR, 0);
+			background = blur_surface(background,4);
+			background = adjust_surface_alpha(background, 4);
+
+			surf_.assign(background);
 		}
+
 	}
 
 	return surf_;
@@ -869,8 +902,13 @@ void floating_label::undraw(surface screen)
 	move(xmove_,ymove_);
 	if(lifetime_ > 0) {
 		--lifetime_;
-		if(surf_ != NULL && alpha_change_ != 0 && (xmove_ != 0.0 || ymove_ != 0.0)) {
-			surf_.assign(adjust_surface_alpha_add(surf_,alpha_change_));
+		if(alpha_change_ != 0 && (xmove_ != 0.0 || ymove_ != 0.0)) {
+			if (!surf_.null()) {
+				surf_.assign(adjust_surface_alpha_add(surf_,alpha_change_));
+			}
+			if (!foreground_.null()) {
+				foreground_.assign(adjust_surface_alpha_add(foreground_,alpha_change_));
+			}
 		}
 	}
 }
@@ -885,8 +923,8 @@ void floating_label::show(bool value) { visible_ = value; }
 
 namespace font {
 int add_floating_label(const std::string& text, int font_size, const SDL_Color& colour,
-					   double xpos, double ypos, double xmove, double ymove, int lifetime, const SDL_Rect& clip_rect, ALIGN align,
-					   const SDL_Color* bg_colour, int border_size, LABEL_SCROLL_MODE scroll_mode)
+		double xpos, double ypos, double xmove, double ymove, int lifetime, const SDL_Rect& clip_rect, ALIGN align,
+		const SDL_Color* bg_colour, int border_size, LABEL_SCROLL_MODE scroll_mode)
 {
 	if(label_contexts.empty()) {
 		return 0;
