@@ -1,4 +1,4 @@
-/* $Id: game.cpp,v 1.141 2004/10/04 00:19:16 Sirp Exp $ */
+/* $Id: game.cpp,v 1.188 2005/02/06 10:40:12 isaaccp Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -11,10 +11,7 @@
    See the COPYING file for more details.
 */
 
-//disable the very annoying VC++ warning 4786
-#ifdef WIN32
-#pragma warning(disable:4786)
-#endif
+#include "global.hpp"
 
 #include "SDL.h"
 
@@ -54,6 +51,7 @@
 #include "unit_types.hpp"
 #include "unit.hpp"
 #include "video.hpp"
+#include "wassert.hpp"
 #include "widgets/button.hpp"
 #include "widgets/menu.hpp"
 
@@ -142,9 +140,9 @@ LEVEL_RESULT play_game(display& disp, game_state& state, config& game_config,
 						try {
 							config snapshot;
 
-							recorder.save_game(units_data,label,snapshot,state.starting_pos);
-						} catch(gamestatus::save_game_failed& e) {
-							gui::show_dialog(disp,NULL,"",_("The game could not be saved"),gui::MESSAGE);
+							recorder.save_game(label, snapshot, state.starting_pos);
+						} catch(gamestatus::save_game_failed&) {
+							gui::show_error_message(disp, _("The game could not be saved"));
 							retry = true;
 						};
 					}
@@ -166,17 +164,13 @@ LEVEL_RESULT play_game(display& disp, game_state& state, config& game_config,
 				return res;
 			}
 		} catch(gamestatus::load_game_failed& e) {
-			gui::show_dialog(disp,NULL,"","The game could not be loaded: " + e.message,gui::OK_ONLY);
-			std::cerr << "The game could not be loaded: " << e.message << "\n";
+			gui::show_error_message(disp, _("The game could not be loaded: ") + e.message);
 			return QUIT;
 		} catch(gamestatus::game_error& e) {
-			gui::show_dialog(disp,NULL,"","An error occurred while playing the game: " + e.message,gui::OK_ONLY);
-			std::cerr << "An error occurred while playing the game: "
-			          << e.message << "\n";
+			gui::show_error_message(disp, _("Error while playing the game: ") + e.message);
 			return QUIT;
 		} catch(gamemap::incorrect_format_exception& e) {
-			gui::show_dialog(disp,NULL,"",e.msg_,gui::OK_ONLY);
-			std::cerr << "The game map could not be loaded: " << e.msg_ << "\n";
+			gui::show_error_message(disp, std::string(_("The game map could not be loaded: ")) + e.msg_);
 			return QUIT;
 		}
 
@@ -204,8 +198,8 @@ LEVEL_RESULT play_game(display& disp, game_state& state, config& game_config,
 				if(should_save == 0) {
 					try {
 						save_game(state);
-					} catch(gamestatus::save_game_failed& e) {
-						gui::show_dialog(disp,NULL,"",_("The game could not be saved"),gui::MESSAGE);
+					} catch(gamestatus::save_game_failed&) {
+						gui::show_error_message(disp, _("The game could not be saved"));
 						retry = true;
 					}
 				}
@@ -217,9 +211,8 @@ LEVEL_RESULT play_game(display& disp, game_state& state, config& game_config,
 		recorder.set_save_info(state);
 	}
 
-	if(state.scenario != "" && state.scenario != "null") {
-		gui::show_dialog(disp,NULL,"",
-		                 "Error - Unknown scenario: '" + state.scenario + "'");
+	if (!state.scenario.empty() && state.scenario != "null") {
+		gui::show_error_message(disp, _("Unknown scenario: '") + state.scenario + '\'');
 		return QUIT;
 	}
 
@@ -245,6 +238,8 @@ void read_game_cfg(preproc_map& defines, std::vector<line_source>& line_src, con
 
 			str << "-" << i->first;
 		}
+		std::string localename = get_locale().localename;
+		str << "-lang_" << (localename.empty() ? "default" : localename);
 
 		if(is_valid) {
 			const std::string& cache = get_cache_dir();
@@ -294,7 +289,7 @@ void read_game_cfg(preproc_map& defines, std::vector<line_source>& line_src, con
 					data_tree_checksum().write(checksum_cfg);
 					std::cerr << "wrote checksum: '" << checksum_cfg.write() << "'\n";
 					write_file(fname_checksum,checksum_cfg.write());
-				} catch(io_exception& e) {
+				} catch(io_exception&) {
 					std::cerr << "could not write to cache '" << fname << "'\n";
 				}
 
@@ -333,12 +328,15 @@ public:
 	bool play_multiplayer();
 	bool change_language();
 
-	void play_game();
+	enum RELOAD_GAME_DATA { RELOAD_DATA, NO_RELOAD_DATA };
+	void play_game(RELOAD_GAME_DATA reload=RELOAD_DATA);
 
 private:
 	game_controller(const game_controller&);
 	void operator=(const game_controller&);
 
+	void refresh_game_cfg();
+
 	void download_campaigns();
 	void upload_campaign(const std::string& campaign, network::connection sock);
 	void delete_campaign(const std::string& campaign, network::connection sock);
@@ -354,6 +352,7 @@ private:
 	const preferences::manager prefs_manager_;
 	const image::manager image_manager_;
 	const events::event_context main_event_context_;
+	const hotkey::manager hotkey_manager_;
 	binary_paths_manager paths_manager_;
 
 	bool test_mode_, multiplayer_mode_, no_gui_;
@@ -370,7 +369,7 @@ private:
 	std::string loaded_game_;
 	bool loaded_game_show_replay_;
 
-	preproc_map defines_map_;
+	preproc_map defines_map_, old_defines_map_;
 };
 
 game_controller::game_controller(int argc, char** argv, bool use_sound)
@@ -385,7 +384,9 @@ game_controller::game_controller(int argc, char** argv, bool use_sound)
 			continue;
 		}
 
-		if(val == "--nocache") {
+		if(val == "--fps") {
+			preferences::set_show_fps(true);
+		} else if(val == "--nocache") {
 			use_caching_ = false;
 		} else if(val == "--resolution" || val == "-r") {
 			if(arg_+1 != argc_) {
@@ -420,11 +421,14 @@ game_controller::game_controller(int argc, char** argv, bool use_sound)
 		} else if(val == "--debug" || val == "-d") {
 			game_config::debug = true;
 		} else if (val.substr(0, 6) == "--log-") {
+		} else if(val == "--nosound") {
+			//handled elsewhere
 		} else if(val[0] == '-') {
 			std::cerr << "unknown option: " << val << "\n";
 			throw config::error("unknown option");
 		} else {
 
+		  std::cerr << "Setting path using " << val;
 			if(val[0] == '/') {
 				game_config::path = val;
 			} else {
@@ -443,6 +447,11 @@ game_controller::game_controller(int argc, char** argv, bool use_sound)
 display& game_controller::disp()
 {
 	if(disp_.get() == NULL) {
+
+		if(get_video_surface() == NULL) {
+			throw CVideo::error();
+		}
+
 		static display::unit_map dummy_umap;
 		static config dummy_cfg("");
 		static gamemap dummy_map(dummy_cfg, "1");
@@ -466,22 +475,23 @@ bool game_controller::init_video()
 		return true;
 	}
 
-    image::set_wm_icon();
+	image::set_wm_icon();
 
 	int video_flags = preferences::fullscreen() ? FULL_SCREEN : 0;
 
 	std::pair<int,int> resolution = preferences::resolution();
 
-	std::cerr << "checking mode possible...\n";
+	std::cerr << "Checking video mode: " << resolution.first
+		  << "x" << resolution.second << "x16...\n";
 	int bpp = video_.modePossible(resolution.first,resolution.second,16,video_flags);
 
 	std::cerr << bpp << "\n";
 
 	if(bpp == 0) {
  		//Video mode not supported, maybe from bad prefs.
- 		std::cerr << "The video mode, " << resolution.first
+ 		std::cerr << "Video mode " << resolution.first
  		          << "x" << resolution.second << "x16 "
- 		          << "is not supported\nAttempting 1024x768x16...\n";
+ 		          << "is not supported - attempting 1024x768x16...\n";
  		
  		//Attempt 1024x768.
  		resolution.first = 1024;
@@ -490,10 +500,7 @@ bool game_controller::init_video()
  		bpp = video_.modePossible(resolution.first,resolution.second,16,video_flags);
 
 		if(bpp == 0) {
-			 //Attempt 1024x768.
- 			resolution.first = 1024;
- 			resolution.second = 768;
-			std::cerr << "1024x768x16 is not possible.\nAttempting 800x600x16...\n";
+			std::cerr << "1024x768x16 not available - attempting 800x600x16...\n";
 
 			resolution.first = 800;
 			resolution.second = 600;
@@ -501,6 +508,26 @@ bool game_controller::init_video()
 			bpp = video_.modePossible(resolution.first,resolution.second,16,video_flags);
 		}
 
+#ifdef USE_TINY_GUI
+		if(bpp == 0) {
+			std::cerr << "800x600x16 not available - attempting 640x480x16...\n";
+
+			resolution.first = 640;
+			resolution.second = 480;
+
+			bpp = video_.modePossible(resolution.first,resolution.second,16,video_flags);
+		}
+
+		if(bpp == 0) {
+			std::cerr << "640x480x16 not available - attempting 320x240x16...\n";
+
+			resolution.first = 320;
+			resolution.second = 240;
+
+			bpp = video_.modePossible(resolution.first,resolution.second,16,video_flags);
+		}
+#endif
+
  		if(bpp == 0) {
  			//couldn't do 1024x768 or 800x600
 
@@ -541,37 +568,29 @@ bool game_controller::init_video()
 
 bool game_controller::init_config()
 {
+	units_data_.clear();
 	defines_map_.clear();
 
 	//load in the game's configuration files
+#if defined(__APPLE__)
+	defines_map_["APPLE"] = preproc_define();
+#endif
+
 	defines_map_["NORMAL"] = preproc_define();
 	defines_map_["MEDIUM"] = preproc_define();
 
 	if(multiplayer_mode_) {
 		defines_map_["MULTIPLAYER"] = preproc_define();
 	}
-	std::vector<line_source> line_src;
 
-	try {
-		log_scope("loading config");
-		read_game_cfg(defines_map_,line_src,game_config_,use_caching_);
-	} catch(config::error& e) {
-		gui::show_dialog(disp(),NULL,"","Error loading game configuration files: '" + e.message + "' (The game will now exit)",
-		                 gui::MESSAGE);
-		throw e;
-	}
+	refresh_game_cfg();
 
 	game_config::load_config(game_config_.child("game_config"));
 
-	hotkey::add_hotkeys(game_config_,false);
+	hotkey::load_hotkeys(game_config_);
 
 	paths_manager_.set_paths(game_config_);
 
-	const config* const units = game_config_.child("units");
-	if(units != NULL) {
-		units_data_.set_config(*units);
-	}
-
 	return true;
 }
 
@@ -591,6 +610,9 @@ bool game_controller::init_language()
 	if(!no_gui_) {
 		SDL_WM_SetCaption(_("The Battle for Wesnoth"), NULL);
 	}
+
+	hotkey::load_descriptions();
+
 	return true;
 }
 
@@ -707,23 +729,26 @@ bool game_controller::play_multiplayer_mode()
    		                                          controller = side_controllers.find(side_num),
    		                                          algorithm = side_algorithms.find(side_num);
 
-   		const config* side = type == side_types.end() ? era_cfg->child("multiplayer_side") :
-   		                                                era_cfg->find_child("multiplayer_side","type",type->second);
+   		const config* side = type == side_types.end() ?
+   			era_cfg->find_child("multiplayer_side", "random_faction", "yes") :
+   			era_cfg->find_child("multiplayer_side", "id", type->second);
 
-#if 0
-   		size_t tries = 0;
-   		while(side != NULL && (*side)["type"] == "random" && ++tries < 100) {
-   			const config::child_list& v = era_cfg->get_children("multiplayer_side");
-   			side = v[rand()%v.size()];
-   		}
-#endif
-
-   		if(side == NULL || (*side)["random_faction"] == "yes" || (*side)["type"] == "random") {
+		if (side == NULL) {
+			unknown_side_id:
    			std::string side_name = (type == side_types.end() ? "default" : type->second);
-   			std::cerr << "Could not find side '" << side_name << "' for side " << side_num << "\n";
+  			std::cerr << "Could not find side '" << side_name << "' for side " << side_num << "\n";
    			return false;
    		}
 
+		if ((*side)["random_faction"] == "yes") {
+			const config::child_list& eras = era_cfg->get_children("multiplayer_side");
+			for(int i = 0; i != 100 && (*side)["random_faction"] == "yes"; ++i) {
+				side = eras[rand()%eras.size()];
+			}
+	   		if ((*side)["random_faction"] == "yes")
+	   			goto unknown_side_id;
+   		}
+
    		char buf[20];
    		sprintf(buf,"%d",side_num);
    		(*itors.first)->values["side"] = buf;
@@ -787,7 +812,28 @@ bool game_controller::load_game()
 	}
 
 	try {
-		::load_game(units_data_,game,state_);
+		//to load a save file, we first load the file in, then we re-parse game
+		//data with the save's #defines, and then we finally parse the save file,
+		//with the game data ready to go.
+		config cfg;
+		read_save_file(game,cfg);
+
+		defines_map_.clear();
+		defines_map_[cfg["difficulty"]] = preproc_define();
+
+		if(defines_map_.count("NORMAL")) {
+			defines_map_["MEDIUM"] = preproc_define();
+		}
+
+		const std::string& campaign_define = cfg["campaign_define"];
+		if(campaign_define.empty() == false) {
+			defines_map_[campaign_define] = preproc_define();
+		}
+
+		refresh_game_cfg();
+
+		state_ = read_game(units_data_,&cfg);
+		
 		if(state_.version != game_config::version) {
 			const int res = gui::show_dialog(disp(),NULL,"",
 			                      _("This save is from a different version of the game. Do you want to try to load it?"),
@@ -797,21 +843,14 @@ bool game_controller::load_game()
 			}
 		}
 
-		defines_map_.clear();
-		defines_map_[state_.difficulty] = preproc_define();
 	} catch(gamestatus::error& e) {
-		std::cerr << "caught load_game_failed\n";
-		gui::show_dialog(disp(),NULL,"",
-		           _("The file you have tried to load is corrupt") + std::string(": '") + e.message + "'",gui::OK_ONLY);
+		gui::show_error_message(disp(), _("The file you have tried to load is corrupt: '") + e.message + '\'');
 		return false;
 	} catch(config::error& e) {
-		std::cerr << "caught config::error\n";
-		gui::show_dialog(disp(),NULL,"",
-		    _("The file you have tried to load is corrupt") + std::string(": '") + e.message + "'",
-		    gui::OK_ONLY);
+		gui::show_error_message(disp(), _("The file you have tried to load is corrupt: '") + e.message + '\'');
 		return false;
-	} catch(io_exception& e) {
-		gui::show_dialog(disp(),NULL,"",_("File I/O Error while reading the game"),gui::OK_ONLY);
+	} catch(io_exception&) {
+		gui::show_error_message(disp(), _("File I/O Error while reading the game"));
 		return false;
 	}
 
@@ -840,7 +879,9 @@ bool game_controller::load_game()
 		}
 	}
 
-	if(state_.campaign_type == "multiplayer") {
+	if(state_.campaign_type == "tutorial") {
+		defines_map_["TUTORIAL"] = preproc_define();
+	} else if(state_.campaign_type == "multiplayer") {
 		//make all network players local
 		for(config::child_itors sides = state_.snapshot.child_range("side");
 		    sides.first != sides.second; ++sides.first) {
@@ -863,13 +904,9 @@ bool game_controller::load_game()
 			play_level(units_data_,game_config_,&starting_pos,video_,state_,story);
 			recorder.clear();
 		} catch(gamestatus::load_game_failed& e) {
-			gui::show_dialog(disp(),NULL,"","error loading the game: " + e.message,gui::OK_ONLY);
-			std::cerr << "error loading the game: " << e.message
-			          << "\n";
+			gui::show_error_message(disp(), _("The game could not be loaded: ") + e.message);
 		} catch(gamestatus::game_error& e) {
-			gui::show_dialog(disp(),NULL,"","error while playing the game: " + e.message,gui::OK_ONLY);
-			std::cerr << "error while playing the game: "
-			          << e.message << "\n";
+			gui::show_error_message(disp(), _("Error while playing the game: ") + e.message);
 		} catch(gamestatus::load_game_exception& e) {
 			//this will make it so next time through the title screen loop, this game is loaded
 			loaded_game_ = e.game;
@@ -884,6 +921,7 @@ bool game_controller::load_game()
 
 void game_controller::set_tutorial()
 {
+	state_ = game_state();
 	state_.campaign_type = "tutorial";
 	state_.scenario = "tutorial";
 	defines_map_["TUTORIAL"] = preproc_define();
@@ -892,7 +930,6 @@ void game_controller::set_tutorial()
 bool game_controller::new_campaign()
 {
 	state_ = game_state();
-
 	state_.campaign_type = "scenario";
 
 	config::child_list campaigns = game_config_.get_children("campaign");
@@ -906,10 +943,10 @@ bool game_controller::new_campaign()
 		const std::string& icon = (**i)["icon"];
 		const std::string desc = (**i)["description"];
 		const std::string image = (**i)["image"];
-		if(icon == "") {
-			str << " ,";
+		if(icon.empty()) {
+			str << COLUMN_SEPARATOR;
 		} else {
-			str << "&" << icon << ",";
+			str << IMAGE_PREFIX << icon << COLUMN_SEPARATOR;
 		}
 
 		str << (**i)["name"];
@@ -918,7 +955,7 @@ bool game_controller::new_campaign()
 		campaign_desc.push_back(std::pair<std::string,std::string>(desc,image));
 	}
 
-	campaign_names.push_back(_(" ,Get More Campaigns..."));
+	campaign_names.push_back(std::string(1, COLUMN_SEPARATOR) + _("Get More Campaigns..."));
 	campaign_desc.push_back(std::pair<std::string,std::string>(_("Download more campaigns from a server on Internet."),game_config::download_campaign_image));
 
 	int res = 0;
@@ -955,7 +992,7 @@ bool game_controller::new_campaign()
 	if(difficulties.empty() == false) {
 		if(difficulty_options.size() != difficulties.size()) {
 			difficulty_options.resize(difficulties.size());
-			std::transform(difficulties.begin(),difficulties.end(),difficulty_options.begin(),translate_string);
+			std::copy(difficulties.begin(),difficulties.end(),difficulty_options.begin());
 		}
 
 		const int res = gui::show_dialog(disp(),NULL,_("Difficulty"),
@@ -975,6 +1012,42 @@ bool game_controller::new_campaign()
 	return true;
 }
 
+namespace
+{
+
+std::string format_file_size(const std::string& size_str)
+{
+	double size = lexical_cast_default<double>(size_str,0.0);
+
+	const double k = 1024;
+	if(size > 0.0) {
+		std::string size_postfix = _("B");
+		if(size > k) {
+			size /= k;
+			size_postfix = _("KB");
+			if(size > k) {
+				size /= k;
+				size_postfix = _("MB");
+			}
+		}
+
+		std::ostringstream stream;
+		//Visual C++ makes 'precision' set the number of decimal places. Other platforms
+		//make it set the number of significant figures
+#ifdef _MSC_VER
+		stream.precision(1);
+#else
+		stream.precision(3);
+#endif
+		stream << std::fixed << size << size_postfix;
+		return stream.str();
+	} else {
+		return "";
+	}
+}
+
+}
+
 void game_controller::download_campaigns()
 {
 	std::string host = "campaigns.wesnoth.org";
@@ -992,7 +1065,7 @@ void game_controller::download_campaigns()
 		const network::manager net_manager;
 		const network::connection sock = network::connect(items.front(),lexical_cast_default<int>(items.back(),15002));
 		if(!sock) {
-			gui::show_dialog(disp(),NULL,_("Error"),_("Could not connect to host."),gui::OK_ONLY);
+			gui::show_error_message(disp(), _("Could not connect to host."));
 			return;
 		}
 
@@ -1007,46 +1080,60 @@ void game_controller::download_campaigns()
 
 		const config* const error = cfg.child("error");
 		if(error != NULL) {
-			gui::show_dialog(disp(),NULL,_("Error"),(*error)["message"],gui::OK_ONLY);
+			gui::show_error_message(disp(), (*error)["message"]);
 			return;
 		}
 
 		const config* const campaigns_cfg = cfg.child("campaigns");
 		if(campaigns_cfg == NULL) {
-			gui::show_dialog(disp(),NULL,_("Error"),_("Error communicating with the server."),gui::OK_ONLY);
+			gui::show_error_message(disp(), _("Error communicating with the server."));
 			return;
 		}
 
 		std::vector<std::string> campaigns, options;
-		options.push_back(_(",Name,Version,Author,Downloads"));
+		std::string sep(1, COLUMN_SEPARATOR);
+		options.push_back(sep + _("Name") +
+		                  sep + _("Version") +
+		                  sep + _("Author") +
+		                  sep + _("Downloads") +
+		                  sep + _("Size"));
 		const config::child_list& cmps = campaigns_cfg->get_children("campaign");
 		const std::vector<std::string>& publish_options = available_campaigns();
 
 		std::vector<std::string> delete_options;
 
 		for(config::child_list::const_iterator i = cmps.begin(); i != cmps.end(); ++i) {
-			campaigns.push_back((**i)["name"]);
+			const std::string& name = (**i)["name"];
+			campaigns.push_back(name);
 			
-			std::string name = (**i)["name"];
-
 			if(std::count(publish_options.begin(),publish_options.end(),name) != 0) {
 				delete_options.push_back(name);
 			}
 
-			std::replace(name.begin(),name.end(),'_',' ');
-			options.push_back("&" + (**i)["icon"] + "," + name + "," + (**i)["version"] + "," + (**i)["author"] + "," + (**i)["downloads"]);
+			std::string title = (**i)["title"];
+			if(title == "") {
+				title = name;
+				std::replace(title.begin(),title.end(),'_',' ');
+			}
+			
+			options.push_back(IMAGE_PREFIX + (**i)["icon"] + COLUMN_SEPARATOR +
+			                  title + COLUMN_SEPARATOR +
+			                  (**i)["version"] + COLUMN_SEPARATOR +
+			                  (**i)["author"] + COLUMN_SEPARATOR +
+			                  (**i)["downloads"] + COLUMN_SEPARATOR +
+			                  format_file_size((**i)["size"]));
 		}
 
 		for(std::vector<std::string>::const_iterator j = publish_options.begin(); j != publish_options.end(); ++j) {
-			options.push_back(std::string(",") + _("Publish campaign: ") + *j);
+			options.push_back(sep + _("Publish campaign: ") + *j);
 		}
 
 		for(std::vector<std::string>::const_iterator d = delete_options.begin(); d != delete_options.end(); ++d) {
-			options.push_back(std::string(",") + _("Delete campaign: ") + *d);
+			options.push_back(sep + _("Delete campaign: ") + *d);
 		}
 
 		if(campaigns.empty() && publish_options.empty()) {
-			gui::show_dialog(disp(),NULL,_("Error"),_("There are no campaigns available for download from this server."),gui::OK_ONLY);
+			gui::show_error_message(disp(), _("There are no campaigns available for download from this server."));
 			return;
 		}
 
@@ -1075,7 +1162,7 @@ void game_controller::download_campaigns()
 		}
 
 		if(cfg.child("error") != NULL) {
-			gui::show_dialog(disp(),NULL,_("Error"),(*cfg.child("error"))["message"],gui::OK_ONLY);
+			gui::show_error_message(disp(), (*cfg.child("error"))["message"]);
 			return;
 		}
 
@@ -1084,16 +1171,17 @@ void game_controller::download_campaigns()
 		//force a reload of configuration information
 		const bool old_cache = use_caching_;
 		use_caching_ = false;
-		init_config();
+		old_defines_map_.clear();
+		refresh_game_cfg();
 		use_caching_ = old_cache;
 
 		gui::show_dialog(disp(),NULL,_("Campaign Installed"),_("The campaign has been installed."),gui::OK_ONLY);
-	} catch(config::error& e) {
-		gui::show_dialog(disp(),NULL,_("Error"),_("Network communication error."),gui::OK_ONLY);
-	} catch(network::error& e) {
-		gui::show_dialog(disp(),NULL,_("Error"),_("Remote host disconnected."),gui::OK_ONLY);
-	} catch(io_exception& e) {
-		gui::show_dialog(disp(),NULL,_("Error"),_("There was a problem creating the files necessary to install this campaign."),gui::OK_ONLY);
+	} catch(config::error&) {
+		gui::show_error_message(disp(), _("Network communication error."));
+	} catch(network::error&) {
+		gui::show_error_message(disp(), _("Remote host disconnected."));
+	} catch(io_exception&) {
+		gui::show_error_message(disp(), _("There was a problem creating the files necessary to install this campaign."));
 	}
 }
 
@@ -1105,10 +1193,11 @@ void game_controller::upload_campaign(const std::string& campaign, network::conn
 	config data;
 	sock = network::receive_data(data,sock,60000);
 	if(!sock) {
-		gui::show_dialog(disp(),NULL,_("Error"),_("Connection timed out"),gui::OK_ONLY);
+		gui::show_error_message(disp(), _("Connection timed out"));
 		return;
 	} else if(data.child("error")) {
-		gui::show_dialog(disp(),NULL,_("Error"),_("The server responded with an error: \"") + (*data.child("error"))["message"] + "\"",gui::OK_ONLY);
+		gui::show_error_message(disp(), _("The server responded with an error: \"") +
+		                        (*data.child("error"))["message"] + '"');
 		return;
 	} else if(data.child("message")) {
 		const int res = gui::show_dialog(disp(),NULL,_("Terms"),(*data.child("message"))["message"],gui::OK_CANCEL);
@@ -1143,9 +1232,10 @@ void game_controller::upload_campaign(const std::string& campaign, network::conn
 	
 	sock = network::receive_data(data,sock,60000);
 	if(!sock) {
-		gui::show_dialog(disp(),NULL,_("Error"),_("Connection timed out"),gui::OK_ONLY);
+		gui::show_error_message(disp(), _("Connection timed out"));
 	} else if(data.child("error")) {
-		gui::show_dialog(disp(),NULL,_("Error"),_("The server responded with an error: \"") + (*data.child("error"))["message"] + "\"",gui::OK_ONLY);
+		gui::show_error_message(disp(), _("The server responded with an error: \"") +
+		                        (*data.child("error"))["message"] + '"');
 	} else if(data.child("message")) {
 		gui::show_dialog(disp(),NULL,_("Response"),(*data.child("message"))["message"],gui::OK_ONLY);
 	}
@@ -1167,9 +1257,10 @@ void game_controller::delete_campaign(const std::string& campaign, network::conn
 
 	sock = network::receive_data(data,sock,60000);
 	if(!sock) {
-		gui::show_dialog(disp(),NULL,_("Error"),_("Connection timed out"),gui::OK_ONLY);
+		gui::show_error_message(disp(), _("Connection timed out"));
 	} else if(data.child("error")) {
-		gui::show_dialog(disp(),NULL,_("Error"),_("The server responded with an error: \"") + (*data.child("error"))["message"] + "\"",gui::OK_ONLY);
+		gui::show_error_message(disp(), _("The server responded with an error: \"") +
+		                        (*data.child("error"))["message"] + '"');
 	} else if(data.child("message")) {
 		gui::show_dialog(disp(),NULL,_("Response"),(*data.child("message"))["message"],gui::OK_ONLY);
 	}
@@ -1177,15 +1268,18 @@ void game_controller::delete_campaign(const std::string& campaign, network::conn
 
 bool game_controller::play_multiplayer()
 {
+	state_ = game_state();
 	state_.campaign_type = "multiplayer";
-	state_.scenario = "";
 	state_.campaign_define = "MULTIPLAYER";
 
 	std::vector<std::string> host_or_join;
-	const std::string sep(1,gui::menu::HELP_STRING_SEPERATOR);
-	host_or_join.push_back(std::string("&icons/icon-server.png,") + _("Join Official Server") + sep + _("Log on to the official Wesnoth multiplayer server"));
-	host_or_join.push_back(std::string("&icons/icon-serverother.png,") + _("Join Game") + sep + _("Join a server or hosted game"));
-	host_or_join.push_back(std::string("&icons/icon-hostgame.png,") + _("Host Multiplayer Game") + sep + _("Host a game without using a server"));
+	std::string const pre = IMAGE_PREFIX + std::string("icons/icon-");
+	char const sep1 = COLUMN_SEPARATOR, sep2 = HELP_STRING_SEPARATOR;
+	host_or_join.push_back(pre + "server.png" + sep1 + _("Join Official Server") + sep2 + _("Log on to the official Wesnoth multiplayer server"));
+	host_or_join.push_back(pre + "serverother.png" + sep1 + _("Join Game") + sep2 + _("Join a server or hosted game"));
+	host_or_join.push_back(pre + "hostgame.png" + sep1 + _("Host Networked Game") + sep2 + _("Host a game without using a server"));
+	host_or_join.push_back(pre + "hotseat.png" + sep1 + _("Hotseat Game") + sep2 + _("Play a multiplayer game sharing the same machine"));
+	host_or_join.push_back(pre + "ai.png" + sep1 + _("Human vs AI") + sep2 + _("Play a game against AI opponents"));
 
 	std::string login = preferences::login();
 	const int res = gui::show_dialog(disp(),NULL,_("Multiplayer"),"",gui::OK_CANCEL,&host_or_join,NULL,_("Login") + std::string(": "),&login);
@@ -1196,17 +1290,19 @@ bool game_controller::play_multiplayer()
 	
 	try {
 		defines_map_[state_.campaign_define] = preproc_define();
-		std::vector<line_source> line_src;
-		config game_config;
-		read_game_cfg(defines_map_,line_src,game_config,use_caching_);
+		refresh_game_cfg();
 		
-		if(res == 2) {
+		if(res >= 2) {
 			std::vector<std::string> chat;
 			config game_data;
-			multiplayer_game_setup_dialog mp_dialog(disp(),units_data_,game_config,state_,true);
+
+			const std::string controller = (res == 2 ? "network" : (res == 3 ? "human" : "ai"));
+			const bool is_server = res == 2;
+
+			multiplayer_game_setup_dialog mp_dialog(disp(),units_data_,game_config_,state_,is_server,controller);
 			lobby::RESULT res = lobby::CONTINUE;
 			while(res == lobby::CONTINUE) {
-				res = lobby::enter(disp(),game_data,game_config,&mp_dialog,chat);
+				res = lobby::enter(disp(),game_data,game_config_,&mp_dialog,chat);
 			}
 
 			if(res == lobby::CREATE) {
@@ -1218,15 +1314,12 @@ bool game_controller::play_multiplayer()
 				host = preferences::official_network_host();
 			}
 
-			play_multiplayer_client(disp(),units_data_,game_config,state_,host);
+			play_multiplayer_client(disp(),units_data_,game_config_,state_,host);
 		}
 	} catch(gamestatus::load_game_failed& e) {
-		gui::show_dialog(disp(),NULL,"","error loading the game: " + e.message,gui::OK_ONLY);
-		std::cerr << "error loading the game: " << e.message << "\n";
+		gui::show_error_message(disp(), _("The game could not be loaded: ") + e.message);
 	} catch(gamestatus::game_error& e) {
-		gui::show_dialog(disp(),NULL,"","error while playing the game: " + e.message,gui::OK_ONLY);
-		std::cerr << "error while playing the game: "
-		          << e.message << "\n";
+		gui::show_error_message(disp(), _("Error while playing the game: ") + e.message);
 	} catch(network::error& e) {
 		std::cerr << "caught network error...\n";
 		if(e.message != "") {
@@ -1238,8 +1331,7 @@ bool game_controller::play_multiplayer()
 			gui::show_dialog(disp(),NULL,"",e.message,gui::OK_ONLY);
 		}
 	} catch(gamemap::incorrect_format_exception& e) {
-		gui::show_dialog(disp(),NULL,"",std::string("The game map could not be loaded: ") + e.msg_,gui::OK_ONLY);
-		std::cerr << "The game map could not be loaded: " << e.msg_ << "\n";
+		gui::show_error_message(disp(), std::string(_("The game map could not be loaded: ")) + e.msg_);
 	} catch(gamestatus::load_game_exception& e) {
 		//this will make it so next time through the title screen loop, this game is loaded
 		loaded_game_ = e.game;
@@ -1270,46 +1362,64 @@ bool game_controller::change_language()
 	                         gui::OK_CANCEL,&langs);
 	if(size_t(res) < langs.size()) {
 		set_language(known_languages[res]);
-		preferences::set_locale(known_languages[res].localename);
+		preferences::set_language(known_languages[res].localename);
+
+		//force a reload of configuration information
+		const bool old_cache = use_caching_;
+		use_caching_ = false;
+		init_config();
+		use_caching_ = old_cache;
 	}
 
+	hotkey::load_descriptions();
+
 	return false;
 }
 
-void game_controller::play_game()
+void game_controller::refresh_game_cfg()
 {
-	if(state_.campaign_define.empty() == false) {
-		defines_map_[state_.campaign_define] = preproc_define();
-	}
-
-	if(defines_map_.count("NORMAL")) {
-		defines_map_["MEDIUM"] = preproc_define();
-	}
-
-	//make a new game config item based on the difficulty level
-	std::vector<line_source> line_src;
-	config game_config;
-	
 	try {
-		read_game_cfg(defines_map_,line_src,game_config,use_caching_);
+		if(old_defines_map_.empty() || defines_map_ != old_defines_map_) {
+
+			units_data_.clear();
+			game_config_.clear();
+
+			std::vector<line_source> line_src;
+			read_game_cfg(defines_map_,line_src,game_config_,use_caching_);
+
+			const config* const units = game_config_.child("units");
+			if(units != NULL) {
+				units_data_.set_config(*units);
+			}
+
+			old_defines_map_ = defines_map_;
+		}
 	} catch(config::error& e) {
-		gui::show_dialog(disp(),NULL,"","Error loading game configuration files: '" + e.message + "' (The game will now exit)", gui::MESSAGE);
+		std::cerr << "Error loading game configuration files\n";
+		gui::show_error_message(disp(), _("Error loading game configuration files: '") +
+		                        e.message + _("' (The game will now exit)"));
 		throw e;
 	}
+}
 
-	const binary_paths_manager bin_paths_manager(game_config);
+void game_controller::play_game(RELOAD_GAME_DATA reload)
+{
+	if(reload == RELOAD_DATA) {
+		if(state_.campaign_define.empty() == false) {
+			defines_map_[state_.campaign_define] = preproc_define();
+		}
 
-	const config* const units = game_config.child("units");
-	if(units == NULL) {
-		std::cerr << "ERROR: Could not find game configuration files\n";
-		std::cerr << game_config.write();
-		return;
+		if(defines_map_.count("NORMAL")) {
+			defines_map_["MEDIUM"] = preproc_define();
+		}
+	
+		refresh_game_cfg();
 	}
 
-	game_data units_data(*units);
+	const binary_paths_manager bin_paths_manager(game_config_);
 
 	try {
-		const LEVEL_RESULT result = ::play_game(disp(),state_,game_config,units_data,video_);
+		const LEVEL_RESULT result = ::play_game(disp(),state_,game_config_,units_data_,video_);
 		if(result == VICTORY) {
 			the_end(disp());
 			about::show_about(disp());
@@ -1393,21 +1503,61 @@ int play_game(int argc, char** argv)
 	srand(time(NULL));
 
 	game_controller game(argc,argv,use_sound);
-	bool res = game.init_video();
+
+	if (!filesystem_init()) {
+		std::cerr << "cannot init filesystem code\n";
+		return 1;
+	}
+
+	// I would prefer to setup locale first so that early error
+	// messages can get localized, but we need the game_controller
+	// initialized to have get_intl_dir() to work.  Note: this
+	// setlocale() but this does not take GUI language setting
+	// into account.
+	setlocale (LC_ALL, "");
+	const std::string& intl_dir = get_intl_dir();
+	bindtextdomain (PACKAGE, intl_dir.c_str());
+	bind_textdomain_codeset (PACKAGE, "UTF-8");
+	bindtextdomain (PACKAGE "-lib", intl_dir.c_str());
+	bind_textdomain_codeset (PACKAGE "-lib", "UTF-8");
+	textdomain (PACKAGE);
+
+	bool res;
+#ifdef WIN32
+	res = game.init_config();
 	if(res == false) {
+		std::cerr << "could not initialize game config\n";
 		return 0;
 	}
+#endif
+
+	res = game.init_video();
+	if(res == false) {
+		std::cerr << "could not initialize display\n";
+		return 0;
+	}
+
+	res = game.init_language();
+	if(res == false) {
+		std::cerr << "could not initialize the language\n";
+		return 0;
+	}
+
+#ifndef WIN32
+	// it is better for gettext-native platforms to read the config
+	// files after having pre-initialized the language, maybe...
+	res = game.init_config();
+	if(res == false) {
+		std::cerr << "could not initialize game config\n";
+		return 0;
+	}
+#endif
 
 	const cursor::manager cursor_manager;
 #if defined(_X11) && !defined(__APPLE__)
 	SDL_EventState(SDL_SYSWMEVENT, SDL_ENABLE);
 #endif
 
-	res = game.init_language() && game.init_config();
-	if(res == false) {
-		return 0;
-	}
-
 	for(;;) {
 		statistics::fresh_stats();
 
@@ -1431,10 +1581,13 @@ int play_game(int argc, char** argv)
 		gui::TITLE_RESULT res = game.is_loading() ? gui::LOAD_GAME : gui::TITLE_CONTINUE;
 
 		int ntip = -1;
+		config tips_of_day;
 		while(res == gui::TITLE_CONTINUE) {
-			res = gui::show_title(game.disp(),&ntip);
+			res = gui::show_title(game.disp(),tips_of_day,&ntip);
 		}
+		tips_of_day.clear();
 
+		game_controller::RELOAD_GAME_DATA should_reload = game_controller::RELOAD_DATA;
 		std::cerr << "title screen returned result\n";
 		if(res == gui::QUIT_GAME) {
 			std::cerr << "quitting game...\n";
@@ -1443,6 +1596,8 @@ int play_game(int argc, char** argv)
 			if(game.load_game() == false) {
 				continue;
 			}
+
+			should_reload = game_controller::NO_RELOAD_DATA;
 		} else if(res == gui::TUTORIAL) {
 			game.set_tutorial();
 		} else if(res == gui::NEW_CAMPAIGN) {
@@ -1468,7 +1623,7 @@ int play_game(int argc, char** argv)
 			continue;
 		}
 		
-		game.play_game();
+		game.play_game(should_reload);
 	}
 
 	return 0;
@@ -1476,27 +1631,15 @@ int play_game(int argc, char** argv)
 
 int main(int argc, char** argv)
 {
-	// setup locale first so that early error messages can get localized
-	// but this does not take GUI language setting into account
-	setlocale (LC_ALL, "");
-
-	const std::string& intl_dir = get_intl_dir();
-	bindtextdomain (PACKAGE, intl_dir.c_str());
-	bind_textdomain_codeset (PACKAGE, "UTF-8");
-	bindtextdomain (PACKAGE "-lib", intl_dir.c_str());
-	bind_textdomain_codeset (PACKAGE "-lib", "UTF-8");
-
-	textdomain (PACKAGE);
-
 	try {
 		std::cerr << "started game: " << SDL_GetTicks() << "\n";
 		const int res = play_game(argc,argv);
 		std::cerr << "exiting with code " << res << "\n";
 		return res;
 	} catch(CVideo::error&) {
-		std::cerr << _("Could not initialize video. Exiting.\n");
+		std::cerr << "Could not initialize video. Exiting.\n";
 	} catch(font::manager::error&) {
-		std::cerr << _("Could not initialize fonts. Exiting.\n");
+		std::cerr << "Could not initialize fonts. Exiting.\n";
 	} catch(config::error& e) {
 		std::cerr << e.message << "\n";
 	} catch(gui::button::error&) {
