@@ -1,12 +1,17 @@
+#include "global.hpp"
+
 #include "config.hpp"
 #include "filesystem.hpp"
 #include "game_config.hpp"
 #include "image.hpp"
 #include "display.hpp"
 #include "font.hpp"
+#include "log.hpp"
 #include "sdl_utils.hpp"
 #include "team.hpp"
 #include "util.hpp"
+#include "wassert.hpp"
+#include "wesconfig.h"
 
 #include "SDL_image.h"
 
@@ -14,6 +19,9 @@
 #include <map>
 #include <string>
 
+#define LOG_DP lg::info(lg::display)
+#define ERR_DP lg::err(lg::display)
+
 namespace {
 
 typedef std::map<gamemap::TERRAIN, surface> mini_terrain_cache_map;
@@ -41,7 +49,11 @@ std::string image_mask;
 
 SDL_PixelFormat* pixel_format = NULL;
 
+#ifdef USE_TINY_GUI
+const int tile_size = 36;
+#else
 const int tile_size = 72;
+#endif
 int zoom = tile_size;
 
 //The "pointer to surfaces" vector is not cleared anymore (the surface are
@@ -190,18 +202,26 @@ bool locator::value::operator<(const value& a) const
 
 surface locator::load_image_file() const
 {
-	const std::string& location = get_binary_file_location("images", val_.filename_);
-
-	if(location.empty()) {
+#ifdef USE_ZIPIOS
+	std::string s = read_file("images/" + val_.filename_);
+	if (s.empty())
 		return surface(NULL);
-	} else {
-		const surface res(IMG_Load(location.c_str()));
-		if(res == NULL) {
-			std::cerr << "Error: could not open image '" << val_.filename_ << "'\n";
-		}
 
-		return res;
+	SDL_RWops* ops = SDL_RWFromMem((void*)s.c_str(), s.size());
+	const surface res(IMG_Load_RW(ops, 0));
+	SDL_FreeRW(ops);
+#else
+	const std::string& location = get_binary_file_location("images", val_.filename_);
+	if(location.empty())
+		return surface(NULL);
+
+	const surface res(IMG_Load(location.c_str()));
+#endif
+	if(res == NULL) {
+		ERR_DP << "could not open image '" << val_.filename_ << "'\n";
 	}
+
+	return res;
 }
 
 surface locator::load_image_sub_file() const
@@ -236,7 +256,7 @@ surface locator::load_from_disk() const
 		default:
 			return surface(NULL);
 	}
-	assert(false);
+	wassert(false);
 }
 
 #if 0
@@ -394,7 +414,7 @@ void set_zoom(int amount)
 	}
 }
 
-surface get_unmasked(const locator i_locator, COLOUR_ADJUSTMENT adj)
+surface get_unmasked(const locator i_locator)
 {
 	surface image(get_image(i_locator, UNSCALED));
 
@@ -505,7 +525,7 @@ surface get_image(const image::locator& i_locator, TYPE type, COLOUR_ADJUSTMENT
 			res = get_scaled(i_locator, adj);
 			break;
 		case UNMASKED:
-			res = get_unmasked(i_locator, adj);
+			res = get_unmasked(i_locator);
 			break;
 		case GREYED:
 			res = get_greyed(i_locator, adj);
@@ -585,7 +605,7 @@ locator get_alternative(const image::locator &i_locator, const std::string &alt)
 		res = locator(alternative, i_locator.get_loc());
 		break;
 	default:
-		assert(false);
+		wassert(false);
 	}
 	
 	i_locator.add_to_cache(alternative_images_, res);
@@ -611,22 +631,24 @@ bool exists(const image::locator& i_locator)
 	if(image_existance_map.find(i_locator) != image_existance_map.end())
 		return image_existance_map[i_locator];
 
+#ifdef USE_ZIPIOS
+	if(file_exists("images/" + i_locator.get_filename()))
+		ret = true;
+#else
 	if(get_binary_file_location("images",i_locator.get_filename()).empty() == false) {
 		ret = true;
 	}
-
+#endif
 	image_existance_map[i_locator] = ret;
 
 	return ret;
 }
 
-surface getMinimap(int w, int h, const gamemap& map, 
-		int lawful_bonus, const team* tm)
+surface getMinimap(int w, int h, const gamemap& map, const team* tm)
 {
-	std::cerr << "generating minimap\n";
 	const int scale = 8;
 
-	std::cerr << "creating minimap " << int(map.x()*scale*0.75) << "," << int(map.y()*scale) << "\n";
+	LOG_DP << "creating minimap " << int(map.x()*scale*0.75) << "," << int(map.y()*scale) << "\n";
 
 	const size_t map_width = map.x()*scale*3/4;
 	const size_t map_height = map.y()*scale;
@@ -644,8 +666,6 @@ surface getMinimap(int w, int h, const gamemap& map,
 	if(minimap == NULL)
 		return surface(NULL);
 
-	// std::cerr << "created minimap: " << int(minimap) << "," << int(minimap->pixels) << "\n";
-
 	typedef mini_terrain_cache_map cache_map;
 	cache_map& cache = mini_terrain_cache;
 
@@ -662,10 +682,10 @@ surface getMinimap(int w, int h, const gamemap& map,
 				cache_map::iterator i = cache.find(terrain);
 
 				if(i == cache.end()) {
-					surface tile(get_image("terrain/" + map.get_terrain_info(terrain).default_image() + ".png",image::UNSCALED));
+					surface tile(get_image("terrain/" + map.get_terrain_info(terrain).symbol_image() + ".png",image::UNSCALED));
 
 					if(tile == NULL) {
-						std::cerr << "Could not get image for terrrain '"
+						ERR_DP << "could not get image for terrrain '"
 						          << terrain << "'\n";
 						continue;
 					}
@@ -685,7 +705,7 @@ surface getMinimap(int w, int h, const gamemap& map,
 					surf = surface(adjust_surface_colour(surf,-50,-50,-50));
 				}
 
-				assert(surf != NULL);
+				wassert(surf != NULL);
 				
 				SDL_Rect maprect = {x*scale*3/4,y*scale + (is_odd(x) ? scale/2 : 0),0,0};
 				SDL_BlitSurface(surf, NULL, minimap, &maprect);
@@ -693,14 +713,12 @@ surface getMinimap(int w, int h, const gamemap& map,
 		}
 	}
 
-	// std::cerr << "scaling minimap..." << int(minimap) << "." << int(minimap->pixels) << "\n";
-
 	if((minimap->w != w || minimap->h != h) && w != 0) {
 		const surface surf(minimap);
 		minimap = surface(scale_surface(surf,w,h));
 	}
 
-	std::cerr << "done generating minimap\n";
+	LOG_DP << "done generating minimap\n";
 
 	return minimap;
 }
