@@ -1,4 +1,4 @@
-/* $Id: language.cpp,v 1.52 2004/10/03 13:04:07 cedricd Exp $ */
+/* $Id: language.cpp,v 1.63 2005/01/16 10:44:29 ydirson Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -10,6 +10,9 @@
 
    See the COPYING file for more details.
 */
+
+#include "global.hpp"
+
 #include "config.hpp"
 #include "font.hpp"
 #include "language.hpp"
@@ -29,6 +32,7 @@ namespace {
 
 language_def known_languages[] = {
  	language_def("", N_("System default language")),
+	language_def("bg_BG", "Български"),
 	language_def("es_ES", "Castellano"),
 	language_def("ca_ES", "Català"),
 	language_def("cs_CZ", "Čeština"),
@@ -40,12 +44,14 @@ language_def known_languages[] = {
  	language_def("el_GR", "Greek"),
 	language_def("hu_HU", "Hungarian"),
 	language_def("it_IT", "Italiano"),
+	language_def("la_IT", "Latina"),
 	language_def("nl_NL", "Nederlands"),
 	language_def("no_NO", "Norsk"),
 	language_def("pl_PL", "Polski"),
 	language_def("pt_BR", "Português do Brasil"),
 	language_def("ru_RU", "Русский (Russian)"),
 	language_def("sk_SK", "Slovenčina"),
+	language_def("sl_SL", "Slovenščina"),
 	language_def("fi_FI", "Suomi"),
 	language_def("sv_SE", "Svenska"),
 
@@ -55,17 +61,17 @@ language_def known_languages[] = {
 
 std::string languagedef_name (const language_def& def)
 {
-  return def.language;
+	return def.language;
 }
 
 bool languagedef_lessthan_p (const language_def& def1, const language_def& def2)
 {
-  return (def1.language < def2.language);
+	return (def1.language < def2.language);
 }
 
 bool language_def::operator== (const language_def& a)
 {
-  return ((language == a.language) /* && (localename == a.localename) */ );
+	return ((language == a.language) /* && (localename == a.localename) */ );
 }
 
 symbol_table string_table;
@@ -77,6 +83,11 @@ const std::string& symbol_table::operator[](const std::string& key) const
 		return i->second;
 	} else {
 		static std::string empty_string;
+		// Let's do it the painful way (untlb means untranslatABLE).
+		// It will cause problem if somebody stores more than one reference at once
+		// but I don't really care since this path is an error path and it should
+		// not have been taken in the first place. -- silene
+		empty_string = "UNTLB " + key;
 		return empty_string;
 	}
 }
@@ -86,16 +97,6 @@ const std::string& symbol_table::operator[](const char* key) const
 	return (*this)[std::string(key)];
 }
 
-const std::string& translate_string(const std::string& str)
-{
-	return str;
-}
-
-const std::string& translate_string_default(const std::string& str, const std::string& default_val)
-{
-	return str;
-}
-
 std::vector<language_def> get_languages()
 {
 	std::vector<language_def> res;
@@ -122,6 +123,11 @@ char* wesnoth_setlocale(int category, const char *locale)
 		std::cerr << "setenv LC_ALL failed: " << strerror(errno);
 #endif 
 
+#ifdef _WIN32
+	const std::string env = std::string("LANG=") + locale;
+	putenv(env.c_str());
+#endif
+
 	char* res = setlocale (category, locale);
 	if (res == NULL)
 		std::cerr << "WARNING: setlocale() failed for "
@@ -142,6 +148,7 @@ bool set_language(const language_def& locale)
 	current_language = locale;
 	wesnoth_setlocale (LC_MESSAGES, locale.localename.c_str());
 	font::set_font();
+	known_languages[0].language = gettext("System default language");
 
 	// fill string_table (should be moved somwhere else some day)
 	try {
@@ -171,7 +178,7 @@ const language_def& get_locale()
 {
 	//TODO: Add in support for querying the locale on Windows
 
-	const std::string& prefs_locale = preferences::locale();
+	const std::string& prefs_locale = preferences::language();
 	if(prefs_locale.empty() == false) {
 		char* setlocaleres = wesnoth_setlocale (LC_MESSAGES, prefs_locale.c_str());
 		if(setlocaleres == NULL)
@@ -209,7 +216,7 @@ namespace
 {
 std::string wstring_to_utf8(const wide_string &src)
 {
-    wchar_t ch;
+	wchar_t ch;
 	wide_string::const_iterator i;
 	int j;
 	Uint32 bitmask;
