@@ -1,7 +1,8 @@
-/* $Id: log.hpp,v 1.16 2004/09/19 11:13:56 silene Exp $ */
+/* $Id: log.hpp,v 1.17 2004/11/14 00:39:10 silene Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
-   Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
+                 2004 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License.
