@@ -1,4 +1,4 @@
-/* $Id: map.cpp,v 1.40 2004/08/05 21:53:31 gruikya Exp $ */
+/* $Id: map.cpp,v 1.47 2005/01/20 21:59:52 silene Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -10,52 +10,27 @@
 
    See the COPYING file for more details.
 */
+
+#include "global.hpp"
+
 #include "game.hpp"
+#include "log.hpp"
 #include "map.hpp"
 #include "pathfind.hpp"
 #include "util.hpp"
+#include "wassert.hpp"
 
 #include <algorithm>
-#include <cassert>
 #include <cctype>
 #include <cstdlib>
 #include <iostream>
 #include <sstream>
 
+#define ERR_CF lg::err(lg::config)
+#define LOG_G lg::info(lg::general)
+
 gamemap::location gamemap::location::null_location;
 
-const std::string& gamemap::terrain_name(gamemap::TERRAIN terrain) const
-{
-	static const std::string default_val;
-	const std::map<TERRAIN,terrain_type>::const_iterator i =
-	                                      letterToTerrain_.find(terrain);
-	if(i == letterToTerrain_.end())
-		return default_val;
-	else
-		return i->second.name();
-}
-
-std::vector<std::string> gamemap::underlying_terrain_name(gamemap::TERRAIN terrain) const
-{
-	const std::map<TERRAIN,terrain_type>::const_iterator i =
-	                                       letterToTerrain_.find(terrain);
-	if(i == letterToTerrain_.end()) {
-		return std::vector<std::string>();
-	} else {
-		if(i->second.is_alias()) {
-			std::vector<std::string> res;
-
-			const std::string& type = i->second.type();
-			for(std::string::const_iterator j = type.begin(); j != type.end(); ++j) {
-				res.push_back(terrain_name(*j));
-			}
-			return res;
-		} else {
-			return std::vector<std::string>(1,i->second.name());
-		}
-	}
-}
-
 const std::string& gamemap::underlying_terrain(TERRAIN terrain) const
 {
 	const std::map<TERRAIN,terrain_type>::const_iterator i = letterToTerrain_.find(terrain);
@@ -96,7 +71,7 @@ bool gamemap::is_village(const gamemap::location& loc) const
 
 bool gamemap::gives_healing(const gamemap::location& loc) const
 {
-	return is_village(loc);
+	return on_board(loc) && gives_healing(get_terrain(loc));
 }
 
 bool gamemap::is_castle(const gamemap::location& loc) const
@@ -184,16 +159,16 @@ gamemap::location gamemap::location::get_direction(
 		case SOUTH_WEST: return gamemap::location(x-1,y+is_odd(x));
 		case NORTH_WEST: return gamemap::location(x-1,y-is_even(x));
 		default:
-			assert(false);
+			wassert(false);
 			return gamemap::location();
 	}
 }
 
 gamemap::gamemap(const config& cfg, const std::string& data) : tiles_(1)
 {
-	std::cerr << "loading map: '" << data << "'\n";
+	LOG_G << "loading map: '" << data << "'\n";
 	const config::child_list& terrains = cfg.get_children("terrain");
-	create_terrain_maps(terrains,terrainPrecedence_,letterToTerrain_,terrain_);
+	create_terrain_maps(terrains,terrainList_,letterToTerrain_,terrain_);
 
 	read(data);
 }
@@ -218,8 +193,8 @@ void gamemap::read(const std::string& data)
 					startingPositions_[c - '0'] = location(x,y);
 					c = KEEP;
 				} else {
-					std::cerr << "Illegal character in map: (" << int(c) << ") '" << c << "'\n";
-					throw incorrect_format_exception("Illegal character");
+					ERR_CF << "Illegal character in map: (" << int(c) << ") '" << c << "'\n";
+					throw incorrect_format_exception("Illegal character found in map. The scenario cannot be loaded.");
 				}
 			}
 
@@ -237,15 +212,16 @@ void gamemap::read(const std::string& data)
 		}
 	}
 
-	for(size_t n = 0; n != tiles_.size(); ++n) {
-		if(tiles_[n].size() != size_t(this->y())) {
-			std::cerr << "Map is not rectangular!\n";
+	unsigned ysize = this->y();
+	for(size_t n = 0; n != tiles_.size(); ++n) { // tiles_.size() is not constant
+		if (tiles_[n].size() != ysize) {
+			ERR_CF << "Map is not rectangular!\n";
 			tiles_.erase(tiles_.begin()+n);
 			--n;
 		}
 	}
 
-	std::cerr << "loaded map: " << this->x() << "," << this->y() << "\n";
+	LOG_G << "loaded map: " << this->x() << ',' << ysize << '\n';
 }
 
 std::string gamemap::write() const
@@ -376,14 +352,9 @@ const terrain_type& gamemap::get_terrain_info(const gamemap::location &loc) cons
 	return get_terrain_info(get_terrain(loc));
 }
 
-const std::vector<gamemap::TERRAIN>& gamemap::get_terrain_precedence() const
+const std::vector<gamemap::TERRAIN>& gamemap::get_terrain_list() const
 {
-	return terrainPrecedence_;
-}
-
-bool gamemap::is_built(const location &loc) const
-{
-	return is_castle(loc);
+	return terrainList_;
 }
 
 void gamemap::set_terrain(const gamemap::location& loc, gamemap::TERRAIN ter)
