@@ -1,4 +1,4 @@
-/* $Id: map.hpp,v 1.27 2004/08/07 17:21:58 Sirp Exp $ */
+/* $Id: map.hpp,v 1.30 2005/01/21 22:21:54 Sirp Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -28,6 +28,7 @@ class gamemap
 public:
 
 	typedef char TERRAIN;
+	enum { NB_TERRAIN = 256, ADD_INDEX_TERRAIN = 128 };
 
 	//some types of terrain which must be known, and can't just be loaded
 	//in dynamically because they're special. It's asserted that there will
@@ -39,8 +40,6 @@ public:
 	//is the name of the terrain for game-logic purposes. I.e. if the terrain
 	//is simply an alias, the underlying terrain name is the name of the
 	//terrain that it's aliased to
-	const std::string& terrain_name(TERRAIN terrain) const;
-	std::vector<std::string> underlying_terrain_name(TERRAIN terrain) const;
 	const std::string& underlying_terrain(TERRAIN terrain) const;
 
 	//exception thrown if the map file is not in the correct format.
@@ -61,7 +60,12 @@ public:
 
 		void write(config& cfg) const;
 
-		bool valid() const { return x >= 0 && y >= 0; }
+		inline bool valid() const { return x >= 0 && y >= 0; }
+		
+		inline bool valid(const int parWidth, const int parHeight) const
+		{ 
+			return ((x >= 0) && (y >= 0) && (x < parWidth) && (y < parHeight));
+		}
 
 		int x, y;
 
@@ -146,11 +150,8 @@ public:
 	//shortcut to get_terrain_info(get_terrain(loc))
 	const terrain_type& get_terrain_info(const location &loc) const;
 	
-	//gets the list of which terrain types should display on top of
-	//other terrain types. Has no effect on gameplay, only display.
-	const std::vector<TERRAIN>& get_terrain_precedence() const;
-
-	bool is_built(const location& loc) const;
+	//gets the list of terrains
+	const std::vector<TERRAIN>& get_terrain_list() const;
 
 	//clobbers over the terrain at location 'loc', with the given terrain
 	void set_terrain(const location& loc, TERRAIN ter);
@@ -165,7 +166,7 @@ public:
 private:
 	int num_starting_positions() const;
 
-	std::vector<TERRAIN> terrainPrecedence_;
+	std::vector<TERRAIN> terrainList_;
 	std::map<TERRAIN,terrain_type> letterToTerrain_;
 	std::map<std::string,terrain_type> terrain_;
 
