@@ -1,3 +1,5 @@
+#include "global.hpp"
+
 #include "language.hpp"
 #include "log.hpp"
 #include "multiplayer.hpp"
@@ -9,9 +11,13 @@
 #include "scoped_resource.hpp"
 #include "show_dialog.hpp"
 #include "util.hpp"
+#include "wassert.hpp"
 
 #include <sstream>
 
+#define LOG_NW lg::info(lg::network)
+#define ERR_NW lg::err(lg::network)
+
 namespace {
 
 class gamelist_manager : public gui::dialog_action
@@ -30,6 +36,8 @@ public:
 		return cfg.child("gamelist");
 	}
 
+	~gamelist_manager() {} // Workaround for IRIX Mips CC
+
 	enum { UPDATED_GAMELIST = -3 };
 
 private:
@@ -64,11 +72,13 @@ void receive_gamelist(display& disp, config& data)
 class wait_for_start : public lobby::dialog
 {
 public:
-	wait_for_start(display& disp, config& cfg, const game_data& data, int team_num,
-			const std::string& team_name, const std::string& team_leader)
+	wait_for_start(display& disp, const config& game_config, config& cfg,
+			const game_data& data, int team_num, const std::string& team_name,
+			const std::string& team_leader)
 		: status(START_GAME), got_side(false), team(team_num),
-		  name(team_name), leader(team_leader), disp_(disp), sides_(cfg),
-		  units_data_(data), cancel_button_(NULL), menu_(NULL)
+		  name(team_name), leader(team_leader), disp_(disp),
+		  sides_(cfg), units_data_(data), game_config_(game_config),
+		  cancel_button_(NULL), menu_(NULL)
 	{
 		SDL_Rect empty_rect = {0,0,0,0};
 		area_ = empty_rect;
@@ -81,13 +91,14 @@ public:
 		}
 
 		std::vector<std::string> details;
+		const config* const era_cfg = game_config_.find_child("era","id",sides_["era"]);
 
 		const config::child_list& sides = sides_.get_children("side");
 		for(config::child_list::const_iterator s = sides.begin(); s != sides.end(); ++s) {
 			const config& sd = **s;
 
 			std::string description = sd["description"];
-			std::string side_name = sd["name"];
+			std::string side_name = (*(era_cfg->find_child("multiplayer_side", "id", sd["id"])))["name"];
 			std::string leader_type = sd["type"];
 
 			if(first && (s - sides.begin() == size_t(team-1))) {
@@ -107,45 +118,48 @@ public:
 				// Dumps the "image" part of the faction name, if any,
 				// to replace it by a picture of the actual leader
 				if(side_name[0] == font::IMAGE) {
-					int p = side_name.find_first_of(",");
+					int p = side_name.find_first_of(COLUMN_SEPARATOR);
 					if(p != std::string::npos && p < side_name.size()) {
-						side_name = "&" + leader_image + "," + side_name.substr(p+1);
+						side_name = IMAGE_PREFIX + leader_image + COLUMN_SEPARATOR + side_name.substr(p+1);
 					}
 				}
 			}
 
 			std::stringstream str;
-			str << description << "," << side_name << ",";
+			str << description << COLUMN_SEPARATOR << side_name << COLUMN_SEPARATOR;
 			if(!leader_name.empty())
 				str << "(" << leader_name << ")";
-			str << "," << sd["gold"] << " " << _("Gold") << "," << sd["team_name"];
+			str << COLUMN_SEPARATOR << sd["gold"] << ' ' << sgettext("unit^Gold")
+			    << COLUMN_SEPARATOR << sd["team_name"];
 			details.push_back(str.str());
 		}
 
 		SDL_Rect rect = area_;
-		rect.h /= 2;
+		rect.h -= 2 * gui::ButtonVPadding;
+		if (cancel_button_)
+			rect.h -= cancel_button_->height();
 		SDL_FillRect(disp_.video().getSurface(),&rect,SDL_MapRGB(disp_.video().getSurface()->format,0,0,0));
 
-		menu_.assign(new gui::menu(disp_,details,false,area_.h/2));
-		menu_->set_loc(area_.x,area_.y);
-		menu_->set_width(area_.w);
+		menu_.assign(new gui::menu(disp_, details, false, rect.h));
+		menu_->set_location(rect);
 	}
 	
 	void set_area(const SDL_Rect& area) {
-
 		area_ = area;
-		generate_menu(true);
-
-		const std::string text = _("Waiting for game to start...");
-		SDL_Rect rect = font::draw_text(NULL,disp_.screen_area(),font::SIZE_NORMAL,font::NORMAL_COLOUR,text,0,0);
-		rect.x = area.x + area.w/2 - rect.w/2;
-		rect.y = area.y + (area.h*3)/4 - rect.h/2;
-		font::draw_text(&disp_,rect,font::SIZE_NORMAL,font::NORMAL_COLOUR,text,rect.x,rect.y);
 
 		cancel_button_.assign(new gui::button(disp_,_("Cancel")));
-		cancel_button_->set_location(area.x+area.w - cancel_button_->width() - gui::ButtonHPadding,
-			                         area.y+area.h - cancel_button_->height() - gui::ButtonVPadding);
-		cancel_button_->draw();
+		int y = area.y + area.h - cancel_button_->height() - gui::ButtonVPadding;
+		cancel_button_->set_location(area.x + area.w - cancel_button_->width() - gui::ButtonHPadding, y);
+
+		std::string const text = _("Waiting for game to start...");
+		SDL_Rect rect = font::draw_text(NULL, disp_.screen_area(), font::SIZE_NORMAL,
+		                                font::NORMAL_COLOUR, text, 0, 0);
+		rect.x = area_.x + gui::ButtonHPadding;
+		rect.y = y + 4;
+		font::draw_text(&disp_, rect, font::SIZE_NORMAL, font::NORMAL_COLOUR, text, rect.x, rect.y);
+
+		generate_menu(true);
+		events::raise_draw_event();
 	}
 
 	void clear_widgets() {
@@ -154,20 +168,17 @@ public:
 	}
 
 	lobby::RESULT process() {
-		int mousex, mousey;
-		const bool button = SDL_GetMouseState(&mousex,&mousey)&SDL_BUTTON_LMASK;
-		if(cancel_button_->process(mousex,mousey,button)) {
+		if (cancel_button_->pressed())
 			return lobby::QUIT;
-		}
 
 		if(menu_ != NULL) {
-			menu_->process(mousex,mousey,button,false,false,false,false);
+			menu_->process();
 		}
 
 		config reply;
 		const network::connection res = network::receive_data(reply);
 		if(res) {
-			std::cerr << "received data while waiting: " << reply.write() << "\n";
+			LOG_NW << "received data while waiting: " << reply.write() << "\n";
 			const config::child_list& assigns = reply.get_children("reassign_side");
 			for(config::child_list::const_iterator a = assigns.begin(); a != assigns.end(); ++a) {
 				if(lexical_cast_default<int>((**a)["from"]) == team) {
@@ -181,9 +192,9 @@ public:
 				return lobby::QUIT;
 			} else if(reply["side_secured"].empty() == false) {
 				got_side = true;
-				std::cerr << "received side secured message\n";
+				LOG_NW << "received side secured message\n";
 			} else if(reply.child("start_game")) {
-				std::cerr << "received start_game message\n";
+				LOG_NW << "received start_game message\n";
 				//break out of dialog
 				status = START_GAME;
 				return lobby::CREATE;
@@ -191,12 +202,14 @@ public:
 				status = GAME_CANCELLED;
 				return lobby::QUIT;
 			} else if(reply.child("scenario_diff")) {
-				std::cerr << "received diff for scenario....applying...\n";
+				LOG_NW << "received diff for scenario....applying...\n";
 				sides_.apply_diff(*reply.child("scenario_diff"));
 				generate_menu(false);
 			} else if(reply.child("side")) {
 				sides_ = reply;
-				std::cerr << "got some sides. Current number of sides = " << sides_.get_children("side").size() << "," << reply.get_children("side").size() << "\n";
+				LOG_NW << "got some sides. Current number of sides = "
+				       << sides_.get_children("side").size() << ","
+				       << reply.get_children("side").size() << "\n";
 			} else {
 				data_.push_back(reply);
 			}
@@ -216,7 +229,7 @@ private:
 	display& disp_;
 	config& sides_;
 	const game_data& units_data_;
-
+	const config& game_config_;
 	util::scoped_ptr<gui::button> cancel_button_;
 	util::scoped_ptr<gui::menu> menu_;
 	std::deque<config> data_;
@@ -239,13 +252,13 @@ private:
 void play_multiplayer_client(display& disp, game_data& units_data, config& cfg,
                              game_state& state, std::string& host)
 {
-	log_scope("playing multiplayer client");
+	log_scope2(network, "playing multiplayer client");
 
 	const network::manager net_manager;
 
 	if(host.empty()) {
 		host = preferences::network_host();
-		const int res = gui::show_dialog(disp,NULL,"","",
+		const int res = gui::show_dialog(disp,NULL,_("Connect to Host"),"",
 		                                 gui::OK_CANCEL,NULL,NULL,
 		                                 _("Choose host to connect to") + std::string(": "),&host);
 		if(res != 0 || host.empty()) {
@@ -314,7 +327,7 @@ void play_multiplayer_client(display& disp, game_data& units_data, config& cfg,
 				throw network::error(_("Connection timed out"));
 			}
 
-			std::cerr << "login response: '" << data.write() << "'\n";
+			LOG_NW << "login response: '" << data.write() << "'\n";
 
 			error = data.child("error");
 		} while(error != NULL);
@@ -329,7 +342,7 @@ void play_multiplayer_client(display& disp, game_data& units_data, config& cfg,
 			receive_gamelist(disp,data);
 		}
 
-		std::cerr << "when receiving gamelist got '" << data.write() << "'\n";
+		LOG_NW << "when receiving gamelist got '" << data.write() << "'\n";
 
 		bool observer = false;
 
@@ -370,6 +383,8 @@ void play_multiplayer_client(display& disp, game_data& units_data, config& cfg,
 						status = 1;
 						break;
 					}
+					default:
+						break;
 				}
 			}
     
@@ -441,14 +456,14 @@ void play_multiplayer_client(display& disp, game_data& units_data, config& cfg,
 			const config* const era_cfg = cfg.find_child("era","id",era);
 
 			if(era_cfg == NULL) {
-				std::cerr << "era '" << era << "' not found\n";
+				ERR_NW << "era '" << era << "' not found\n";
 				return;
 			}
 
 			const config::child_list& possible_sides = era_cfg->get_children("multiplayer_side");
 
 			if(possible_sides.empty()) {
-				std::cerr << "no multiplayer sides found\n";
+				ERR_NW << "no multiplayer sides found\n";
 				return;
 			}
 
@@ -478,22 +493,24 @@ void play_multiplayer_client(display& disp, game_data& units_data, config& cfg,
 				leader = leader_selector.get_selected_leader();
 			}
 
-			assert(choice < possible_sides.size());
+			wassert(choice < possible_sides.size());
 
 			const config& chosen_side = *possible_sides[choice];
+			response["id"] = chosen_side["id"];
 			team_name = response["name"] = chosen_side["name"];
 			if(leader.empty()) {
 				team_leader = response["type"] = chosen_side["type"];
 			} else {
 				team_leader = response["type"] = leader;
 			}
+			response["random_faction"] = chosen_side["random_faction"];
 			response["recruit"] = chosen_side["recruit"];
 			response["music"] = chosen_side["music"];
 
 			network::send_data(response);
 		}
     
-		wait_for_start waiter(disp,sides,units_data,team_num,team_name,team_leader);
+		wait_for_start waiter(disp,cfg,sides,units_data,team_num,team_name,team_leader);
 		std::vector<std::string> messages;
 		config game_data;
 		lobby::RESULT dialog_res = lobby::CONTINUE;
@@ -540,7 +557,7 @@ void play_multiplayer_client(display& disp, game_data& units_data, config& cfg,
 		config replay_data_store;
 		if(replay_data != NULL) {
 			replay_data_store = *replay_data;
-			std::cerr << "setting replay\n";
+			LOG_NW << "setting replay\n";
 			recorder = replay(replay_data_store);
 			if(!recorder.empty()) {
 				const int res = gui::show_dialog(disp,NULL,
@@ -552,7 +569,7 @@ void play_multiplayer_client(display& disp, game_data& units_data, config& cfg,
 					sides = state.starting_pos;
 					recorder.set_skip(0);
 				} else {
-					std::cerr << "skipping...\n";
+					LOG_NW << "skipping...\n";
 					recorder.set_skip(-1);
 				}
 			}
@@ -560,15 +577,11 @@ void play_multiplayer_client(display& disp, game_data& units_data, config& cfg,
 			sides.clear_children("replay");
 		}
     
-		std::cerr << "starting game\n";
+		LOG_NW << "starting game\n";
 
 		state.starting_pos = sides;
 		state.snapshot = sides;
-
-                for(std::map<std::string, player_info>::iterator i=state.players.begin();
-                    i!=state.players.end(); ++i) {
-                  i->second.can_recruit.clear();
-                }
+		state.players.clear();
     
 		recorder.set_save_info(state);
     
@@ -650,7 +663,7 @@ void leader_list_manager::update_leader_list(int side_index)
 			const std::string name =  utypes.find(*itor)->second.language_name();
 			const std::string image = utypes.find(*itor)->second.image();
 
-			leader_strings.push_back("&" + image + "," + name);
+			leader_strings.push_back(IMAGE_PREFIX + image + COLUMN_SEPARATOR + name);
 		} else {
 			leader_strings.push_back("?");
 		}
@@ -658,7 +671,8 @@ void leader_list_manager::update_leader_list(int side_index)
 
 	leaders_.push_back("random");
 	// FIXME: Maybe this should not code into the code.
-	leader_strings.push_back(_("&random-enemy.png,Random"));
+	leader_strings.push_back(IMAGE_PREFIX + std::string("random-enemy.png") +
+	                         COLUMN_SEPARATOR + _("Random"));
 
 	if(combo_ != NULL) {
 		combo_->set_items(leader_strings);
@@ -674,8 +688,10 @@ void leader_list_manager::set_leader(const std::string& leader)
 	int leader_index = 0;
 	for(std::vector<std::string>::const_iterator itor = leaders_.begin();
 			itor != leaders_.end(); ++itor) {
-		if (leader == *itor) 
+		if (leader == *itor) {
 			combo_->set_selected(leader_index);
+			return;
+		}
 		++leader_index;
 	}
 }
@@ -708,28 +724,22 @@ leader_preview_pane::leader_preview_pane(display& disp, const game_data* data,
 	set_location(leader_pane_position);
 }
 
-void leader_preview_pane::process()
+void leader_preview_pane::process_event()
 {
-	int mousex, mousey;
-	const int mouse_flags = SDL_GetMouseState(&mousex,&mousey);
-	const bool left_button = mouse_flags&SDL_BUTTON_LMASK;
-
-	if(leader_combo_.process(mousex, mousey, left_button)) {
+	if (leader_combo_.changed()) {
 		set_dirty();
 	}
 }
 
-void leader_preview_pane::draw()
+void leader_preview_pane::draw_contents()
 {
-	if(!dirty())
-		return;
-
 	bg_restore();
 
 	surface const screen = disp().video().getSurface();
 
-	const SDL_Rect area = { location().x+leader_pane_border, location().y+leader_pane_border,
-	                        location().w-leader_pane_border*2, location().h-leader_pane_border*2 };
+	SDL_Rect const &loc = location();
+	const SDL_Rect area = { loc.x + leader_pane_border, loc.y + leader_pane_border,
+	                        loc.w - leader_pane_border * 2, loc.h - leader_pane_border * 2 };
 	SDL_Rect clip_area = area;
 	const clip_rect_setter clipper(screen,clip_area);
 
@@ -741,7 +751,7 @@ void leader_preview_pane::draw()
 		std::ostringstream recruit_string;
 
 		if(faction[0] == font::IMAGE) {
-			int p = faction.find_first_of(",");
+			int p = faction.find_first_of(COLUMN_SEPARATOR);
 			if(p != std::string::npos && p < faction.size())
 				faction = faction.substr(p+1);
 		}
@@ -784,9 +794,6 @@ void leader_preview_pane::draw()
 
 		leader_combo_.set_location(leader_rect.x + leader_rect.w + 10, leader_rect.y + (leader_rect.h - leader_combo_.height()) / 2);
 	}
-
-
-	set_dirty(false);
 }
 
 bool leader_preview_pane::show_above() const
@@ -804,11 +811,9 @@ void leader_preview_pane::set_selection(int selection)
 	selection_ = selection;
 	leaders_.update_leader_list(selection_);
 	set_dirty();
-	leader_combo_.set_dirty();
 }
 
 std::string leader_preview_pane::get_selected_leader()
 {
 	return leaders_.get_leader();
 }
-
