@@ -1,4 +1,4 @@
-/* $Id: multiplayer_connect.cpp,v 1.76 2004/10/03 19:29:54 gruikya Exp $ */
+/* $Id: multiplayer_connect.cpp,v 1.101 2005/02/04 23:43:26 j_daniel Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -11,6 +11,8 @@
    See the COPYING file for more details.
 */
 
+#include "global.hpp"
+
 #include "events.hpp"
 #include "font.hpp"
 #include "language.hpp"
@@ -26,29 +28,35 @@
 #include "replay.hpp"
 #include "show_dialog.hpp"
 #include "util.hpp"
+#include "wassert.hpp"
 #include "widgets/textbox.hpp"
 #include "widgets/button.hpp"
 #include "widgets/combo.hpp"
 #include "widgets/menu.hpp"
 #include "widgets/slider.hpp"
 
-#include <iostream>
 #include <sstream>
 #include <string>
 #include <vector>
 
+#define ERR_CF lg::err(lg::config)
+#define LOG_CF lg::info(lg::config)
+#define ERR_NW lg::err(lg::network)
+#define LOG_NW lg::info(lg::network)
+#define LOG_G lg::info(lg::general)
+
 mp_connect::mp_connect(display& disp, std::string game_name,
 		       const config &cfg, game_data& data, game_state& state,
-		       bool join) : 
+		       bool join, const std::string& default_controller) : 
 	    disp_(&disp), cfg_(&cfg), data_(&data), state_(&state),
 	    show_replay_(false), save_(false), join_(join),
-	    player_types_(), player_races_(), player_teams_(),
-	    player_colors_(), combos_type_(), combos_race_(),
-	    combos_leader_(), combos_team_(), combos_color_(), sliders_gold_(),
-	    ai_(gui::button(disp, _(" Computer vs Computer "))),
-	    launch_(gui::button(disp, _("I'm Ready"))),
-	    cancel_(gui::button(disp, _("Cancel"))),
-		message_full_(true)
+	    scroll_pane_(disp),
+	    ai_(disp, _(" Computer vs Computer ")),
+	    launch_(disp, _("I'm Ready")),
+	    cancel_(disp, _("Cancel")),
+	    waiting_label_(disp, ""),
+	    message_full_(true), default_controller_(default_controller),
+	    team_prefix_(_("Team") + std::string(" "))
 {
 	// Send Initial information
 	config response;
@@ -85,13 +93,9 @@ int mp_connect::load_map(const std::string& era, config& scenario_data, int num_
 
 		log_scope("loading save");
 
+		state_->players.clear();
 		load_game(*data_, game, *state_);
 
-		for(std::map<std::string, player_info>::iterator i = state_->players.begin(); i != state_->players.end(); ++i) {
-			i->second.available_units.clear();
-			i->second.can_recruit.clear();
-		}
-
 		if(state_->campaign_type != "multiplayer") {
 			gui::show_dialog(*disp_, NULL, "", 
 					 _("This is not a multiplayer save"),
@@ -112,11 +116,11 @@ int mp_connect::load_map(const std::string& era, config& scenario_data, int num_
 		level_ptr = &scenario_data;
 
 		//make all sides untaken
-		for(config::child_itors i = level_ptr->child_range("side"); i.first != i.second; ++i.first) {
-			(**i.first)["taken"] = "";
+		for(config::child_itors j = level_ptr->child_range("side"); j.first != j.second; ++j.first) {
+			(**j.first)["taken"] = "";
 
 			//tell clients not to change their race
-			(**i.first)["allow_changes"] = "no";
+			(**j.first)["allow_changes"] = "no";
 		}
 
 		recorder = replay(state_->replay_data);
@@ -147,7 +151,7 @@ int mp_connect::load_map(const std::string& era, config& scenario_data, int num_
 		(*level_ptr)["turns"] = turns.str();
 	}
 
-	assert(level_ptr != NULL);
+	wassert(level_ptr != NULL);
 
 	if(scenario_data["era"].empty()) {
 		era_ = era;
@@ -160,10 +164,7 @@ int mp_connect::load_map(const std::string& era, config& scenario_data, int num_
 
 	level_ = level_ptr;
 	state_->label = level_->values["name"];
-
-	for(std::map<std::string, player_info>::iterator i = state_->players.begin(); i != state_->players.end(); ++i) {
-		i->second.gold = -10000;
-	}
+	state_->players.clear();
 
 	state_->scenario = scenario_data["id"];
 
@@ -174,7 +175,7 @@ int mp_connect::load_map(const std::string& era, config& scenario_data, int num_
 	const config* const era_cfg = cfg_->find_child("era","id",era_);
 
 	if(era_cfg == NULL) {
-		std::cerr << "ERROR: era '" << era_ << "' not found\n";
+		ERR_CF << "era '" << era_ << "' not found\n";
 		return -1;
 	}
 
@@ -184,7 +185,7 @@ int mp_connect::load_map(const std::string& era, config& scenario_data, int num_
 		gui::show_dialog(*disp_, NULL, "", 
 				 _("No multiplayer sides."),
 				 gui::OK_ONLY);
-		std::cerr << "no multiplayer sides found\n";
+		ERR_CF << "no multiplayer sides found\n";
 		return -1;
 	}
 
@@ -205,7 +206,7 @@ int mp_connect::load_map(const std::string& era, config& scenario_data, int num_
 				(**sd)["description"] = preferences::login();
 				first = false;
 			} else {
-				(**sd)["controller"] = "ai";
+				(**sd)["controller"] = default_controller_;
 				(**sd)["description"] = "";
 			}
 		}
@@ -222,6 +223,9 @@ int mp_connect::load_map(const std::string& era, config& scenario_data, int num_
 		if(side["share_view"].empty())
 			side["share_view"] = share_view ? "yes" : "no";
 
+		if(side["id"].empty())
+			side["id"] = (*possible_sides.front())["id"];
+
 		if(side["name"].empty())
 			side["name"] = (*possible_sides.front())["name"];
 
@@ -286,20 +290,31 @@ void mp_connect::lists_init()
 
 	for(std::vector<config*>::const_iterator race = possible_sides.begin(); race != possible_sides.end(); ++race) {
 		player_races_.push_back((**race)["name"]);
+		possible_faction_ids_.push_back((**race)["id"]);
 	}
 
 	//Teams
 	config::child_iterator sd;
 	for(sd = sides.first; sd != sides.second; ++sd) {
-		const int team_num = sd - sides.first;
+		const int side_num = (sd - sides.first) + 1;
+		(**sd)["colour"] = lexical_cast_default<std::string>(side_num);
+      
 		std::string& team_name = (**sd)["team_name"];
-		if(team_name.empty()) {
-			team_name = lexical_cast<std::string>(team_num+1);
+		if (team_name.empty())
+			team_name = lexical_cast<std::string>(side_num);
+      
+		const std::vector<std::string>::iterator result = 
+			std::find(player_teams_.begin(), player_teams_.end(),
+			          team_prefix_ + team_name);
+		
+		if (result == player_teams_.end()) {
+			player_teams_.push_back(team_prefix_ + team_name);
+			team_names_.push_back(team_name);
+			team_indices_.push_back(player_teams_.size() - 1);
+		} else {
+			team_indices_.push_back(result - player_teams_.begin());
 		}
 
-		player_teams_.push_back(_("Team") + std::string(" ") + team_name);
-		(**sd)["colour"] = lexical_cast_default<std::string>(team_num+1);
-
 		player_leaders_.push_back(leader_list_manager(possible_sides, data_));
 	}
 
@@ -340,6 +355,10 @@ void mp_connect::remove_player(const std::string& name)
 	const std::vector<std::string>::iterator itor = std::find(player_types_.begin(),player_types_.end(),name);
 	if(itor != player_types_.end())
 		player_types_.erase(itor);
+
+	for(size_t n = 0; n != combos_type_.size(); ++n) {
+		combos_type_[n].set_items(player_types_);
+	}
 }
 
 void mp_connect::set_area(const SDL_Rect& rect)
@@ -365,7 +384,8 @@ void mp_connect::set_area(const SDL_Rect& rect)
 	right_button->set_location(right - right_button->width() - gui::ButtonHPadding,bottom-right_button->height()-gui::ButtonVPadding);
 	left_button->set_location(right - right_button->width() - left_button->width() - gui::ButtonHPadding*2,bottom-left_button->height()-gui::ButtonVPadding);
 	
-	ai_.set_location(left+30,bottom-60);
+	ai_.set_location(left+30, bottom-left_button->height()-gui::ButtonVPadding);
+	waiting_label_.set_location(ai_.location().x + ai_.location().w + 10, bottom-left_button->height()-gui::ButtonVPadding);
 
 	//Title and labels
 	gui::draw_dialog_title(left,top,disp_,_("Game Lobby"));
@@ -401,42 +421,47 @@ void mp_connect::set_area(const SDL_Rect& rect)
 		return;
 	}
 
-	combos_type_.clear();
-	combos_race_.clear();
-	combos_leader_.clear();
-	combos_team_.clear();
-	combos_color_.clear();
-	sliders_gold_.clear();
+	//Show buttons
+	ai_.hide(false);
+	launch_.hide(false);
+	cancel_.hide(false);
+	waiting_label_.hide(false);
+	scroll_pane_.hide(false);
 
+	SDL_Rect scroll_pane_rect;
+	scroll_pane_rect.x = rect.x;
+	scroll_pane_rect.y = rect.y + 50;
+	scroll_pane_rect.w = rect.w;
+	scroll_pane_rect.h = launch_.location().y - scroll_pane_rect.y - gui::ButtonVPadding;
+
+	scroll_pane_.set_location(scroll_pane_rect);
 	config::child_iterator sd;
 
 	for(sd = sides.first; sd != sides.second; ++sd) {
 		const int side_num = sd - sides.first;
+		LOG_CF << "Side num: " << side_num << std::endl;
 
 		//Player number
-		font::draw_text(disp_,rect, font::SIZE_XLARGE, font::GOOD_COLOUR,
-		                (*sd)->values["side"], left+10, top+53+(60*side_num));
+		player_numbers_.push_back(gui::label(*disp_, (*sd)->values["side"],
+					font::SIZE_XLARGE, font::GOOD_COLOUR));
 
 		//Player type
 		combos_type_.push_back(gui::combo(*disp_, player_types_));
-		combos_type_.back().set_location(left+30,top+55+(60*side_num));
 
 		//Player race
 		combos_race_.push_back(gui::combo(*disp_, player_races_));
-		combos_race_.back().set_location(left+145,top+55+(60*side_num));
 
 		//Player leader
-		combos_leader_.push_back(gui::combo(*disp_, std::vector<std::string>()));
-		combos_leader_.back().set_location(left+145,top+85+(60*side_num));
+		std::vector<std::string> dummy_leaders;
+		dummy_leaders.push_back("-");
+		combos_leader_.push_back(gui::combo(*disp_, dummy_leaders));
 
 		//Player team
 		combos_team_.push_back(gui::combo(*disp_, player_teams_));
-		combos_team_.back().set_location(left+260,top+55+(60*side_num));
-		combos_team_.back().set_selected(side_num);
+		combos_team_.back().set_selected(team_indices_[ side_num ]);
 
 		//Player color
 		combos_color_.push_back(gui::combo(*disp_, player_colors_));
-		combos_color_.back().set_location(left+375,top+55+(60*side_num));
 		combos_color_.back().set_selected(side_num);
 
 		SDL_Rect r;
@@ -446,30 +471,66 @@ void mp_connect::set_area(const SDL_Rect& rect)
 		r.y = top+55+(60*side_num);
 		r.w = launch_.width()-5;
 		r.h = launch_.height();
-		sliders_gold_.push_back(gui::slider(*disp_, r));
+
+		sliders_gold_.push_back(gui::slider(*disp_));
 		sliders_gold_.back().set_min(20);
 		sliders_gold_.back().set_max(1000);
 		sliders_gold_.back().set_increment(25);
 		sliders_gold_.back().set_value(lexical_cast_default<int>((**sd)["gold"],100));
-		r.w = 30;
-		r.x = left+603;
-		gold_bg_.push_back(surface_restorer(&disp_->video(),r));
-		font::draw_text(disp_, disp_->screen_area(), font::SIZE_SMALL, font::GOOD_COLOUR,
-		                "100", r.x, r.y);
+		sliders_gold_.back().set_location(r);
+
+		labels_gold_.push_back(gui::label(*disp_, "100", font::SIZE_NORMAL, font::GOOD_COLOUR));
+
+		combos_race_.back().enable(!save_);
+		combos_leader_.back().enable(!save_);
+		combos_team_.back().enable(!save_);
+		combos_color_.back().enable(!save_);
 	}
 
+		
 	//Doing this after creating the combos, because growing vectors may
 	//move their elements in memory, and we need a stable pointer
 	for(sd = sides.first; sd != sides.second; ++sd) {
 		const int side_num = sd - sides.first;
+		const int spos = 60 * side_num;
+
+		scroll_pane_.add_widget(&player_numbers_[side_num], 10, 3 + spos);
+		scroll_pane_.add_widget(&combos_type_[side_num], 30, 5 + spos);
+		scroll_pane_.add_widget(&combos_race_[side_num], 145, 5 + spos);
+		scroll_pane_.add_widget(&combos_leader_[side_num], 145, 35 + spos);
+		scroll_pane_.add_widget(&combos_team_[side_num], 260, 5 + spos);
+		scroll_pane_.add_widget(&combos_color_[side_num], 375, 5 + spos);
+		scroll_pane_.add_widget(&sliders_gold_[side_num], 490, 5 + spos);
+		scroll_pane_.add_widget(&labels_gold_[side_num], 500 + sliders_gold_[side_num].width(), 5 + spos);
+
 		player_leaders_[side_num].set_combo(&combos_leader_[side_num]);
 	}
 
-	std::cerr << "done set_area()\n";
+	LOG_CF << "done set_area()\n";
 
 	update_whole_screen();
 }
 
+void mp_connect::clear_area()
+{
+	scroll_pane_.clear();
+
+	combos_type_.clear();
+	combos_race_.clear();
+	combos_leader_.clear();
+	combos_team_.clear();
+	combos_color_.clear();
+	sliders_gold_.clear();
+	labels_gold_.clear();
+	possible_faction_ids_.clear();
+
+	ai_.hide();
+	launch_.hide();
+	cancel_.hide();
+	waiting_label_.hide();
+	scroll_pane_.hide();
+}
+
 void mp_connect::gui_update()
 {
 	//Update the GUI based on current config settings.
@@ -482,7 +543,6 @@ void mp_connect::gui_update()
 	}
 
 	const config::child_itors sides = level_->child_range("side");
-	SDL_Rect rect;
 
 	for(size_t n = 0; n != combos_type_.size(); ++n) {
 		config& side = **(sides.first+n);
@@ -520,51 +580,31 @@ void mp_connect::gui_update()
 
 		//Player Faction
 		for (size_t m = 0; m != player_races_.size(); ++m) {
-			if (side["name"] == player_races_[m]) {
+			if (side["id"] == possible_faction_ids_[m]) {
 				if (combos_race_[n].selected() != m) {
 					combos_race_[n].set_selected(m);
-					player_leaders_[n].update_leader_list(m);
+
+					if (!save_)
+						player_leaders_[n].update_leader_list(m);
 				}
 			}
 		}
 
 		// Player leader
-		player_leaders_[n].set_leader(side["type"]);
-
-		//Player Team
-		const std::string& team_name = side["team_name"];
-		if(team_name != "" && isdigit(team_name[0]))
-			combos_team_[n].set_selected(team_name[0] - '1');
-
-		//Player Color
+		if (!save_) 
+			player_leaders_[n].set_leader(side["type"]);
 
 		//Player Gold
-		std::string str = side["gold"];
-		sliders_gold_[n].set_value(atoi(str.c_str()));
-		rect.x = rect_.x + 603;
-		rect.y = rect_.y + 55 + (60 * n);
-		rect.w = 30;
-		rect.h = launch_.height();
-		gold_bg_[n].restore();
-		font::draw_text(disp_, disp_->screen_area(), font::SIZE_SMALL,
-				font::GOOD_COLOUR,
-		                side["gold"],
-				rect.x, rect.y);
-		update_rect(rect);
+		labels_gold_[n].set_text(side["gold"]);
 	}
 
 	const bool full = is_full();
 	if(full != message_full_) {
 		message_full_ = full;
 		if(full) {
-			message_bg_.restore();
-			message_bg_ = surface_restorer();
+			waiting_label_.set_text("");
 		} else {
-			SDL_Rect rect = font::draw_text(NULL,rect_,font::SIZE_SMALL,font::NORMAL_COLOUR,_("Waiting for network players to join"),0,0);
-			rect.x = ai_.location().x + ai_.location().w + 10;
-			rect.y = ai_.location().y;
-			message_bg_ = surface_restorer(&disp_->video(),rect);
-			font::draw_text(disp_,rect,font::SIZE_SMALL,font::NORMAL_COLOUR,_("Waiting for network players to join"),rect.x,rect.y);
+			waiting_label_.set_text(_("Waiting for network players to join"));
 		}
 	}
 }
@@ -578,7 +618,7 @@ lobby::RESULT mp_connect::process()
 	const config* const era_cfg = cfg_->find_child("era","id",era_);
 
 	if(era_cfg == NULL) {
-		std::cerr << "ERROR: cannot find era '" << era_ << "'\n";
+		ERR_CF << "cannot find era '" << era_ << "'\n";
 		return lobby::QUIT;
 	}
 
@@ -586,10 +626,6 @@ lobby::RESULT mp_connect::process()
 
 	const config::child_itors sides = level_->child_range("side");
 
-	int mousex, mousey;
-	const int mouse_flags = SDL_GetMouseState(&mousex,&mousey);
-	const bool left_button = mouse_flags&SDL_BUTTON_LMASK;
-
 	bool start_game = false;
 
 	bool level_changed = false;
@@ -601,23 +637,21 @@ lobby::RESULT mp_connect::process()
 		//Don't let user change this if a player is sitting
 		combos_type_[n].enable(combos_type_[n].selected() < 4);
 
-		int old_select = combos_type_[n].selected();
-		if(combos_type_[n].process(mousex, mousey, left_button)) {
-			if(combos_type_[n].selected() == 0) {
-				side["controller"] = "network";
-				side["description"] = "";
-			} else if(combos_type_[n].selected() == 1){
+		if (combos_type_[n].changed()) {
+			switch (combos_type_[n].selected()) {
+			case 1:
 				side["controller"] = "human";
 				side["description"] = "";
-			} else if(combos_type_[n].selected() == 2){
+				break;
+			case 2:
 				side["controller"] = "ai";
 				side["description"] = _("Computer Player");
-			} else if(combos_type_[n].selected() == 3) {
+				break;
+			case 3:
 				side["controller"] = "null";
 				side["description"] = "";
-			} else if(combos_type_[n].selected() == 4){
-				combos_type_[n].set_selected(old_select);
-			} else if(combos_type_[n].selected() == 5){
+				break;
+			case 5: {
 				side["controller"] = "human";
 				side["description"] = preferences::login();
 				for(size_t m = 0; m != combos_type_.size(); ++m) {
@@ -630,61 +664,52 @@ lobby::RESULT mp_connect::process()
 						}
 					}
 				}
-			}else{
+				break;
+			}
+			case 4:
+				combos_type_[n].set_selected(0);
+			case 0:
 				side["controller"] = "network";
 				side["description"] = "";
+				break;
+			default:
+				// Do nothing
+				;
 			}
 
 			level_changed = true;
 		}
 
-		if(combos_color_[n].process(mousex, mousey, left_button)) {
-			side["colour"] = lexical_cast_default<std::string>(combos_color_[n].selected()+1);
-		}
-
-			
-		//Player race
-		combos_race_[n].enable(!save_);
-		combos_leader_[n].enable(!save_);
-		combos_team_[n].enable(!save_);
-		combos_color_[n].enable(!save_);
-		
-		old_select = combos_race_[n].selected();
-		if(combos_race_[n].process(mousex, mousey, left_button)) {
+		if (!save_ && combos_race_[n].changed()) {
 			const string_map& values =  possible_sides[combos_race_[n].selected()]->values;
 			side["random_faction"] = "";
 			for(string_map::const_iterator i = values.begin(); i != values.end(); ++i) {
-				std::cerr << "value: " << i->first << " , " << i->second<< std::endl;
+				LOG_CF << "value: " << i->first << " , " << i->second << std::endl;
 				side[i->first] = i->second;
 			}
 			level_changed = true;
 
-			if (combos_race_[n].selected() != old_select) {
-				player_leaders_[n].update_leader_list(combos_race_[n].selected());
-			}
+			player_leaders_[n].update_leader_list(combos_race_[n].selected());
+		}
+		
+		//Player colour
+		if (combos_color_[n].changed()) {
+			side["colour"] = lexical_cast_default<std::string>(combos_color_[n].selected()+1);
+			level_changed = true;
 		}
 
 		//Player leader
-		if(combos_leader_[n].process(mousex, mousey, left_button)) {
-			std::stringstream str;
-			str << (combos_team_[n].selected()+1);
+		if (!save_ && combos_leader_[n].changed()) {
 			side["type"] = player_leaders_[n].get_leader();
 			level_changed = true;
 		}
 
 		//Player team
-		if(combos_team_[n].process(mousex, mousey, left_button)) {
-			std::stringstream str;
-			str << (combos_team_[n].selected()+1);
-			side["team_name"] = str.str();
+		if (combos_team_[n].changed()) {
+			side["team_name"] = team_names_[combos_team_[n].selected()];
 			level_changed = true;
 		}
 
-		if(combos_color_[n].process(mousex, mousey, left_button)) {
-			level_changed = true;
-		}
-
-		sliders_gold_[n].process();
 		if(!save_){
 			const int cur_playergold = sliders_gold_[n].value();
 			std::stringstream playergold;
@@ -693,21 +718,14 @@ lobby::RESULT mp_connect::process()
 			{
 				side["gold"] = playergold.str();
 
-				SDL_Rect rect;
-				rect.x = rect_.x + 603;
-				rect.y = rect_.y + 55 + (60 * n);
-				rect.w = 30;
-				rect.h = launch_.height();
-				gold_bg_[n].restore();
-				font::draw_text(disp_, rect_, font::SIZE_SMALL,font::GOOD_COLOUR,(*sides.first[n])["gold"],
-						        rect.x, rect.y);
-				update_rect(rect);
+				//Player Gold
+				labels_gold_[n].set_text(side["gold"]);
 				level_changed = true;
 			}
 		}
 	}
 
-	if(cancel_.process(mousex,mousey,left_button)) {
+	if (cancel_.pressed()) {
 		if(network::nconnections() > 0) {
 			config cfg;
 			cfg.add_child("leave_game");
@@ -717,9 +735,21 @@ lobby::RESULT mp_connect::process()
 		return lobby::QUIT;
 	}
 
-	if(ai_.process(mousex,mousey,left_button)) {
+	if (ai_.pressed()) {
 		for(size_t m = 0; m != combos_team_.size(); ++m) {
 			config& si = **(sides.first+m);
+			//disconnect everyone except the host
+			std::map<config*,network::connection>::iterator pos = positions_.find(&si);
+			if (pos->first->values["description"] == si["description"]) {
+				network::disconnect(pos->second);
+				pos->second = 0;
+			}
+			pos->first->values.erase("taken");
+			remove_player(pos->first->values["description"]);
+			pos->first->values["description"] = "";
+			pos->first->values["name"] = player_races_.front();
+			pos->first->values["random_faction"] = "yes";
+			pos->first->values["type"] = "";
 			si["controller"] = "ai";
 			si["description"] = _("Computer Player");
 			combos_type_[m].set_selected(2);
@@ -729,15 +759,15 @@ lobby::RESULT mp_connect::process()
 
 	launch_.enable(is_full());
 
-	if(launch_.process(mousex,mousey,left_button)) {
+	if (launch_.pressed()) {
 		const config::child_list& real_sides = era_cfg->get_children("multiplayer_side");
 
 		for(config::child_iterator side = sides.first; side != sides.second; ++side) {
 			int ntry = 0;
-			//std::cerr << "Checking random faction for " << (**side)["name"] << ". Random faction is " << (**side)["random_faction"] << "\n";
 			while((**side)["random_faction"] == "yes" && ntry < 1000) {
 				const int choice = rand()%real_sides.size();
 
+				(**side)["id"] = (*real_sides[choice])["id"];
 				(**side)["name"] = (*real_sides[choice])["name"];
 				(**side)["random_faction"] = (*real_sides[choice])["random_faction"];
 
@@ -755,7 +785,6 @@ lobby::RESULT mp_connect::process()
 			}
 
 			if ((**side)["type"] == "random" || (**side)["type"].empty()) {
-				//std::cerr << "Choosing a random type for " << (**side)["name"] << "\n";
 				// Choose a random leader type.  
 				std::vector<std::string> types = 
 					config::split((**side)["leader"]);
@@ -763,9 +792,8 @@ lobby::RESULT mp_connect::process()
 					const int lchoice = rand() % types.size();
 					(**side)["type"] = types[lchoice];
 				} else {
-					lg::err(lg::config) << "Unable to find a type for side " << (**side)["name"] << "!\n";
+					ERR_CF << "Unable to find a type for side " << (**side)["name"] << "!\n";
 				}
-				//std::cerr << "Leader chosen: " << (**side)["type"] << "\n";
 				level_changed = true;
 			}
 		}
@@ -783,7 +811,7 @@ lobby::RESULT mp_connect::process()
 	}
 
 	if(start_game) {
-		std::cerr << "multiplayer_connect returning create...\n";
+		LOG_G << "multiplayer_connect returning create...\n";
 		return lobby::CREATE;
 	}
 
@@ -807,18 +835,7 @@ bool mp_connect::get_network_data(config& cfg)
 
 void mp_connect::start_game()
 {
-	combos_type_.clear();
-	combos_race_.clear();
-	combos_leader_.clear();
-	combos_team_.clear();
-	combos_color_.clear();
-	sliders_gold_.clear();
-
-	ai_.hide();
-	launch_.hide();
-	cancel_.hide();
-	gold_bg_.clear();
-
+	clear_area();
 
 	//Tell everyone to start
 	config cfg;
@@ -832,7 +849,7 @@ void mp_connect::start_game()
 		if(false) {
 			recorder.set_skip(0);
 		} else {
-			std::cerr << "skipping...\n";
+			LOG_G << "skipping...\n";
 			recorder.set_skip(-1);
 		}
 	}
@@ -847,9 +864,7 @@ void mp_connect::start_game()
 	//it was just there to tell clients about the replay data
 	level_->clear_children("replay");
 	std::vector<config*> story;
-	for(std::map<std::string, player_info>::iterator i = state_->players.begin(); i != state_->players.end(); ++i) {
-		i->second.can_recruit.clear();
-	}
+	state_->players.clear();
 
 	play_level(*data_, *cfg_, level_, disp_->video(), *state_, story);
 	recorder.clear();
@@ -879,7 +894,7 @@ void mp_connect::update_network()
 			//We are waiting on someone
 			network::connection sock = network::accept_connection();
 			if(sock) {
-				std::cerr << "Received connection\n";
+				LOG_NW << "Received connection\n";
 				network::send_data(*level_,sock);
 			}
 		}
@@ -893,7 +908,7 @@ void mp_connect::update_network()
 	try {
 		sock = network::receive_data(cfg);
 	} catch(network::error& e) {
-		std::cerr << "caught networking error. we are " << (network::is_server() ? "" : "NOT") << " a server\n";
+		ERR_NW << "caught networking error. we are " << (network::is_server() ? "" : "NOT") << " a server\n";
 		sock = 0;
 
 		//if the problem isn't related to any specific connection,
@@ -913,9 +928,11 @@ void mp_connect::update_network()
 				changes = true;
 				i->second = 0;
 				i->first->values.erase("taken");
-
 				remove_player(i->first->values["description"]);
 				i->first->values["description"] = "";
+				i->first->values["name"] = player_races_.front();
+				i->first->values["random_faction"] = "yes";
+				i->first->values["type"] = "";
 			}
 		}
 
@@ -943,9 +960,14 @@ void mp_connect::update_network()
 			if(pos != positions_.end()) {
 				pos->second = 0;
 				pos->first->values.erase("taken");
+				remove_player(pos->first->values["description"]);
 				pos->first->values["description"] = "";
+				pos->first->values["name"] = player_races_.front();
+				pos->first->values["random_faction"] = "yes";
+				pos->first->values["type"] = "";
 				network::send_data(*level_);
 			}
+			return;
 		}
 
 		int side_taken = atoi(cfg["side"].c_str())-1;
@@ -976,7 +998,7 @@ void mp_connect::update_network()
 					network::send_data(reassign,sock);
 				}
 
-				std::cerr << "client has taken a valid position\n";
+				LOG_CF << "client has taken a valid position\n";
 
 				//broadcast to everyone the new game status
 				pos->first->values["controller"] = "network";
@@ -986,6 +1008,25 @@ void mp_connect::update_network()
 					pos->first->values["description"] = cfg["description"];
 				}
 
+				if(cfg["id"].empty() == false) {
+					const config* const era_cfg = cfg_->find_child("era","id",era_);
+					const config::child_list& possible_sides = era_cfg->get_children("multiplayer_side");
+					const std::vector<std::string>::const_iterator itor = 
+						std::find(possible_faction_ids_.begin(), possible_faction_ids_.end(), cfg["id"] );
+					if(itor == possible_faction_ids_.end()) {
+						ERR_CF << "client sent unknown faction id: " << cfg["id"] << "\n";
+					} else {
+						const int index = itor - possible_faction_ids_.begin();
+						const string_map& values = possible_sides[index]->values;
+						pos->first->values["random_faction"] = "";
+						for(string_map::const_iterator i = values.begin(); i != values.end(); ++i) {
+							LOG_CF << "value: " << i->first << " , " << i->second << std::endl;
+							pos->first->values[i->first] = i->second;
+						}
+						pos->first->values["id"] = cfg["id"];
+					}
+				}
+
 				if(cfg["name"].empty() == false) {
 					pos->first->values["name"] = cfg["name"];
 				}
@@ -1014,7 +1055,7 @@ void mp_connect::update_network()
 				pos->second = sock;
 				network::send_data(*level_);
 
-				std::cerr << "sent player data\n";
+				LOG_NW << "sent player data\n";
 
 				//send a reply telling the client they have secured
 				//the side they asked for
@@ -1022,16 +1063,14 @@ void mp_connect::update_network()
 				side << (side_taken+1);
 				config reply;
 				reply.values["side_secured"] = side.str();
-				std::cerr << "going to send data...\n";
+				LOG_NW << "going to send data...\n";
 				network::send_data(reply,sock);
 
 				// Add to combo list
 				add_player(cfg["description"]);
 			} else {
-				std::cerr << "tried to take illegal side: " << side_taken << "\n";
+				ERR_CF << "tried to take illegal side: " << side_taken << '\n';
 			}
-		} else {
-			std::cerr << "tried to take unknown side: " << side_taken << "\n";
 		}
 	}
 }
