@@ -1,11 +1,14 @@
+#include "global.hpp"
+
 #include "log.hpp"
 #include "network.hpp"
+#include "network_worker.hpp"
 #include "util.hpp"
+#include "wassert.hpp"
 
 #include "SDL_net.h"
 
 #include <algorithm>
-#include <cassert>
 #include <cerrno>
 #include <queue>
 #include <iostream>
@@ -14,6 +17,10 @@
 
 #include <signal.h>
 
+#define LOG_NW lg::info(lg::network)
+#define WRN_NW lg::warn(lg::network)
+// only warnings and not errors to avoid DoS by log flooding
+
 namespace {
 
 //We store the details of a connection in a map that must be looked up by its handle.
@@ -79,6 +86,18 @@ void set_remote_handle(network::connection handle, int remote_handle)
 	get_connection_details(handle).remote_handle = remote_handle;
 }
 
+void check_error()
+{
+	const TCPsocket sock = network_worker_pool::detect_error();
+	if(sock) {
+		for(connection_map::const_iterator i = connections.begin(); i != connections.end(); ++i) {
+			if(i->second.sock == sock) {
+				throw network::error("Error sending data",i->first);
+			}
+		}
+	}
+}
+
 SDLNet_SocketSet socket_set = 0;
 typedef std::vector<network::connection> sockets_list;
 sockets_list sockets;
@@ -102,14 +121,13 @@ typedef std::map<network::connection,partial_buffer> partial_map;
 partial_map received_data;
 partial_map::const_iterator current_connection = received_data.end();
 
-typedef std::multimap<network::connection,partial_buffer> send_queue_map;
-send_queue_map send_queue;
-
 TCPsocket server_socket;
 
 std::deque<network::connection> disconnection_queue;
 std::set<network::connection> bad_sockets;
 
+network_worker_pool::manager* worker_pool_man = NULL;
+
 }
 
 namespace network {
@@ -124,8 +142,6 @@ connection_stats get_connection_stats(connection connection_num)
 	return connection_stats(details.sent,details.received,details.connected_at);
 }
 
-const connection null_connection = 0;
-
 error::error(const std::string& msg, connection sock) : message(msg), socket(sock)
 {
 	if(socket) {
@@ -138,7 +154,7 @@ void error::disconnect()
 	network::disconnect(socket);
 }
 
-manager::manager() : free_(true)
+manager::manager(size_t nthreads) : free_(true)
 {
 	//if the network is already being managed
 	if(socket_set) {
@@ -148,22 +164,26 @@ manager::manager() : free_(true)
 
 	//on Unix-based systems, set sigpipe to be ignored
 #if !(defined(_WIN32) || defined(__APPLE__))
-	std::cerr << "ignoring SIGPIPE\n";
+	WRN_NW << "ignoring SIGPIPE\n";
 	signal(SIGPIPE,SIG_IGN);
 #endif
 
 	if(SDLNet_Init() == -1) {
-		std::cerr << "could not initialize SDLNet; throwing error...\n";
+		lg::err(lg::network) << "could not initialize SDLNet; throwing error...\n";
 		throw error(SDL_GetError());
 	}
 
 	socket_set = SDLNet_AllocSocketSet(64);
+
+	worker_pool_man = new network_worker_pool::manager(nthreads);
 }
 
 manager::~manager()
 {
 	if(free_) {
 		disconnect();
+		delete worker_pool_man;
+		worker_pool_man = NULL;
 		SDLNet_FreeSocketSet(socket_set);
 		socket_set = 0;
 		SDLNet_Quit();
@@ -183,7 +203,7 @@ server_manager::server_manager(int port, CREATE_SERVER create_server) : free_(fa
 			}
 		}
 
-		std::cerr << "server socket initialized: " << server_socket << "\n";
+		LOG_NW << "server socket initialized: " << server_socket << "\n";
 		free_ = true;
 	}
 }
@@ -198,7 +218,7 @@ server_manager::~server_manager()
 
 bool server_manager::is_running() const
 {
-	return static_cast<bool>(server_socket);
+	return server_socket != NULL;
 }
 
 size_t nconnections()
@@ -232,7 +252,7 @@ connection connect(const std::string& host, int port)
 		return create_connection(sock,"",port);
 	}
 
-	std::cerr << "sending handshake...\n";
+	LOG_NW << "sending handshake...\n";
 	//send data telling the remote host that this is a new connection
 	char buf[4];
 	SDLNet_Write32(0,buf);
@@ -241,7 +261,7 @@ connection connect(const std::string& host, int port)
 		SDLNet_TCP_Close(sock);
 		throw network::error("Could not send initial handshake");
 	}
-	std::cerr << "sent handshake...\n";
+	LOG_NW << "sent handshake...\n";
 
 	//allocate this connection a connection handle
 	const network::connection connect = create_connection(sock,host,port);
@@ -275,7 +295,7 @@ connection accept_connection()
 
 	const TCPsocket sock = SDLNet_TCP_Accept(server_socket);
 	if(sock) {
-		std::cerr << "received connection. Pending handshake...\n";
+		LOG_NW << "received connection. Pending handshake...\n";
 		pending_sockets.push_back(sock);
 		if(pending_socket_set == 0) {
 			pending_socket_set = SDLNet_AllocSocketSet(64);
@@ -295,7 +315,7 @@ connection accept_connection()
 		return 0;
 	}
 
-	std::cerr << "pending socket activity...\n";
+	LOG_NW << "pending socket activity...\n";
 
 	for(std::vector<TCPsocket>::iterator i = pending_sockets.begin(); i != pending_sockets.end(); ++i) {
 		if(!SDLNet_SocketReady(*i)) {
@@ -310,18 +330,18 @@ connection accept_connection()
 		SDLNet_TCP_DelSocket(pending_socket_set,sock);
 		pending_sockets.erase(i);
 
-		std::cerr << "receiving data from pending socket...\n";
+		LOG_NW << "receiving data from pending socket...\n";
 
 		const int len = SDLNet_TCP_Recv(sock,buf,4);
 		if(len != 4) {
-			std::cerr << "pending socket disconnected\n";
+			WRN_NW << "pending socket disconnected\n";
 			SDLNet_TCP_Close(sock);
 			return 0;
 		}
 
 		const int handle = SDLNet_Read32(buf);
 
-		std::cerr << "received handshake from client: '" << handle << "'\n";
+		LOG_NW << "received handshake from client: '" << handle << "'\n";
 
 		const int res = SDLNet_TCP_AddSocket(socket_set,sock);
 		if(res == -1) {
@@ -352,13 +372,18 @@ void disconnect(connection s)
 {
 	if(s == 0) {
 		while(sockets.empty() == false) {
-			assert(sockets.back() != 0);
+			wassert(sockets.back() != 0);
 			disconnect(sockets.back());
 		}
 
 		return;
 	}
 
+	const connection_map::iterator info = connections.find(s);
+	if(info != connections.end()) {
+		network_worker_pool::close_socket(info->second.sock);
+	}
+
 	schemas.erase(s);
 	bad_sockets.erase(s);
 	received_data.erase(s);
@@ -381,7 +406,7 @@ void disconnect(connection s)
 		remove_connection(s);
 	} else {
 		if(sockets.size() == 1) {
-			std::cerr << "valid socket: " << (int)*sockets.begin() << "\n";
+			LOG_NW << "valid socket: " << (int)*sockets.begin() << "\n";
 		}
 	}
 }
@@ -393,6 +418,12 @@ void queue_disconnect(network::connection sock)
 
 connection receive_data(config& cfg, connection connection_num, int timeout)
 {
+	if(!socket_set) {
+		return 0;
+	}
+
+	check_error();
+
 	if(disconnection_queue.empty() == false) {
 		const network::connection sock = disconnection_queue.front();
 		disconnection_queue.pop_front();
@@ -448,10 +479,10 @@ connection receive_data(config& cfg, connection connection_num, int timeout)
 
 				len = SDLNet_Read32(num_buf);
 
-				std::cerr << "received packet length: " << len << "\n";
+				LOG_NW << "received packet length: " << len << "\n";
 
 				if((len < 1) || (len > 10000000)) {
-					std::cerr << "bad length in network packet. Throwing error\n";
+					WRN_NW << "bad length in network packet. Throwing error\n";
 					throw error("network error: bad length data",*i);
 				}
 
@@ -461,7 +492,7 @@ connection receive_data(config& cfg, connection connection_num, int timeout)
 				//make sure that this connection still has data
 				const int res = SDLNet_CheckSockets(socket_set,0);
 				if(res <= 0 || !SDLNet_SocketReady(sock)) {
-					std::cerr << "packet has no data after length. Throwing error\n";
+					WRN_NW << "packet has no data after length. Throwing error\n";
 					throw error("network error: received wrong number of bytes: 0",*i);
 				}
 			}
@@ -472,31 +503,31 @@ connection receive_data(config& cfg, connection connection_num, int timeout)
 			const size_t expected = buf.buf.size() - buf.upto;
 			const int nbytes = SDLNet_TCP_Recv(sock,&buf.buf[buf.upto],expected);
 			if(nbytes <= 0) {
-				std::cerr << "SDLNet_TCP_Recv returned " << nbytes << " error in socket\n";
+				WRN_NW << "SDLNet_TCP_Recv returned " << nbytes << " error in socket\n";
 				throw error("remote host disconnected",*i);
 			}
 
 			details.received += nbytes;
 
 			buf.upto += nbytes;
-			std::cerr << "received " << nbytes << "=" << buf.upto << "/" << buf.buf.size() << "\n";
+			LOG_NW << "received " << nbytes << "=" << buf.upto << "/" << buf.buf.size() << "\n";
 
 			if(buf.upto == buf.buf.size()) {
 				current_connection = received_data.end();
 				const std::string buffer(buf.buf.begin(),buf.buf.end());
 				received_data.erase(part_received); //invalidates buf. don't use again
 				if(buffer == "") {
-					std::cerr << "buffer from remote host is empty\n";
+					WRN_NW << "buffer from remote host is empty\n";
 					throw error("remote host closed connection",*i);
 				}
 
 				if(buffer[buffer.size()-1] != 0) {
-					std::cerr << "buf not nul-delimited. Network error\n";
+					WRN_NW << "buf not nul-delimited. Network error\n";
 					throw error("sanity check on incoming data failed",*i);
 				}
 
 				const schema_map::iterator schema = schemas.find(*i);
-				assert(schema != schemas.end());
+				wassert(schema != schemas.end());
 
 				cfg.read_compressed(buffer,schema->second.incoming);
 
@@ -526,6 +557,7 @@ void set_default_send_size(size_t max_size)
 
 void send_data(const config& cfg, connection connection_num, size_t max_size, SEND_TYPE mode)
 {
+	LOG_NW << "in send_data()...\n";
 	if(cfg.empty()) {
 		return;
 	}
@@ -542,13 +574,12 @@ void send_data(const config& cfg, connection connection_num, size_t max_size, SE
 		max_size = 8;
 	}
 
-	log_scope("sending data");
+	log_scope2(network, "sending data");
 	if(!connection_num) {
-		std::cerr << "sockets: " << sockets.size() << "\n";
+		LOG_NW << "sockets: " << sockets.size() << "\n";
 		for(sockets_list::const_iterator i = sockets.begin();
 		    i != sockets.end(); ++i) {
-			std::cerr << "server socket: " << server_socket << "\n";
-			std::cerr << "current socket: " << *i << "\n";
+			LOG_NW << "server socket: " << server_socket << "\ncurrent socket: " << *i << "\n";
 			send_data(cfg,*i);
 		}
 
@@ -556,28 +587,22 @@ void send_data(const config& cfg, connection connection_num, size_t max_size, SE
 	}
 
 	const schema_map::iterator schema = schemas.find(connection_num);
-	assert(schema != schemas.end());
+	wassert(schema != schemas.end());
 
-	std::string value(4,'x');
-	value += cfg.write_compressed(schema->second.outgoing);
+	const std::string& value = cfg.write_compressed(schema->second.outgoing);
 
 //	std::cerr << "--- SEND DATA to " << ((int)connection_num) << ": '"
 //	          << cfg.write() << "'\n--- END SEND DATA\n";
 
-	char buf[4];
-	SDLNet_Write32(value.size()+1-4,buf);
-	std::copy(buf,buf+4,value.begin());
+	std::vector<char> buf(4 + value.size() + 1);
+	SDLNet_Write32(value.size()+1,&buf[0]);
+	std::copy(value.begin(),value.end(),buf.begin()+4);
+	buf.back() = 0;
 
-	//place the data in the send queue
-	const send_queue_map::iterator itor = send_queue.insert(std::pair<network::connection,partial_buffer>(connection_num,partial_buffer()));
+	const connection_map::iterator info = connections.find(connection_num);
+	wassert(info != connections.end());
 
-	itor->second.buf.resize(value.size()+1);
-	std::copy(value.begin(),value.end(),itor->second.buf.begin());
-	itor->second.buf.back() = 0;
-
-	if(mode == SEND_DATA) {
-		process_send_queue(connection_num,max_size);
-	}
+	network_worker_pool::queue_data(info->second.sock,buf);
 }
 
 void queue_data(const config& cfg, connection connection_num)
@@ -587,62 +612,7 @@ void queue_data(const config& cfg, connection connection_num)
 
 void process_send_queue(connection connection_num, size_t max_size)
 {
-	if(connection_num == 0) {
-		for(sockets_list::iterator i = sockets.begin(); i != sockets.end(); ++i) {
-			process_send_queue(*i,max_size);
-		}
-
-		return;
-	}
-
-	if(max_size == 0) {
-		max_size = default_max_send_size;
-	}
-	
-	if(max_size != 0 && max_size < 8) {
-		max_size = 8;
-	}
-
-	connection_details& details = get_connection_details(connection_num);
-	const TCPsocket sock = details.sock;
-
-	std::pair<send_queue_map::iterator,send_queue_map::iterator> itor = send_queue.equal_range(connection_num);
-	if(itor.first != itor.second) {
-		std::vector<char>& buf = itor.first->second.buf;
-		size_t& upto = itor.first->second.upto;
-
-		size_t bytes_to_send = buf.size() - upto;
-		if(max_size != 0 && bytes_to_send > max_size) {
-			bytes_to_send = max_size;
-		}
-
-		std::cerr << "sending " << bytes_to_send << " from send queue\n";
-
-		const int res = SDLNet_TCP_Send(sock,&buf[upto],bytes_to_send);
-		if(res < 0 || res != int(bytes_to_send) && errno != EAGAIN) {
-			std::cerr << "sending data failed: " << res << "/" << bytes_to_send << "\n";
-			throw error("Sending queued data failed",connection_num);
-		}
-
-		std::cerr << "sent.\n";
-
-		upto += res;
-		details.sent += res;
-
-		//if we've now sent the entire item, erase it from the send queue
-		if(upto == buf.size()) {
-			std::cerr << "erasing item from the send queue\n";
-			send_queue.erase(itor.first);
-		} else if(upto > buf.size()) {
-			std::cerr << "ERROR: buffer overrun sending data\n";
-		}
-
-		//if we haven't sent 'max_size' bytes yet, try to go onto the next item in
-		//the queue by recursing
-		if(bytes_to_send < max_size || max_size == 0) {
-			process_send_queue(connection_num,max_size-bytes_to_send);
-		}
-	}
+	check_error();
 }
 
 void send_data_all_except(const config& cfg, connection connection_num, size_t max_size)
@@ -682,6 +652,4 @@ std::pair<int,int> current_transfer_stats()
 		return std::pair<int,int>(current_connection->second.upto,current_connection->second.buf.size());
 }
 
-bool sends_queued() { return send_queue.empty() == false; }
-
 } //end namespace network
