@@ -14,11 +14,14 @@ namespace network {
 //a network manager must be created before networking can be used.
 //it must be destroyed only after all networking activity stops.
 struct manager {
-	manager();
+	explicit manager(size_t nthreads=1);
 	~manager();
 
 private:
 	bool free_;
+	
+	manager(const manager&);
+	void operator=(const manager&);
 };
 
 //a server manager causes listening on a given port to occur
@@ -43,7 +46,7 @@ private:
 
 typedef int connection;
 
-extern const connection null_connection;
+connection const null_connection = 0;
 
 //the number of peers we are connected to
 size_t nconnections();
@@ -134,8 +137,6 @@ struct error
 	void disconnect();
 };
 
-bool sends_queued();
-
 }
 
 #endif
