@@ -1,4 +1,4 @@
-/* $Id: playlevel.hpp,v 1.18 2004/09/11 21:22:57 gruikya Exp $ */
+/* $Id: playlevel.hpp,v 1.19 2005/01/22 10:47:16 silene Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -44,6 +44,11 @@ struct end_level_exception {
 	bool gold_bonus;
 };
 
+struct end_turn_exception {
+	end_turn_exception(int r = 0): redo(r) {}
+	int redo;
+};
+
 LEVEL_RESULT play_level(game_data& gameinfo, const config& terrain_config,
 		config* level, CVideo& video,
 		game_state& state_of_game,
