@@ -1,4 +1,4 @@
-/* $Id: preferences.hpp,v 1.41 2004/10/02 15:53:58 Sirp Exp $ */
+/* $Id: preferences.hpp,v 1.47 2005/02/06 10:40:12 isaaccp Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -20,6 +20,15 @@
 #include <utility>
 #include <set>
 
+// only there temporary
+#ifdef USE_TINY_GUI
+const int min_allowed_width = 320;
+const int min_allowed_height = 240;
+#else
+const int min_allowed_width = 800;
+const int min_allowed_height = 600;
+#endif
+
 namespace preferences {
 
 	struct manager
@@ -43,8 +52,8 @@ namespace preferences {
 	bool turbo();
 	void set_turbo(bool ison);
 
-	const std::string& locale();
-	void set_locale(const std::string& s);
+	const std::string& language();
+	void set_language(const std::string& s);
 
 	int music_volume();
 	void set_music_volume(int vol);
@@ -89,6 +98,30 @@ namespace preferences {
 
 	bool show_combat();
 
+	bool allow_observers();
+	void set_allow_observers(bool value);
+
+	bool fog();
+	void set_fog(bool value);
+
+	bool shroud();
+	void set_shroud(bool value);
+
+	int turns();
+	void set_turns(int value);
+
+	int village_gold();
+	void set_village_gold(int value);
+
+	int xp_modifier();
+	void set_xp_modifier(int value);
+
+	int era();
+	void set_era(int value);
+
+	int map();
+	void set_map(int value);
+
 	bool show_ai_moves();
 	void set_show_ai_moves(bool value);
 
@@ -113,8 +146,8 @@ namespace preferences {
 	bool show_haloes();
 	void set_show_haloes(bool value);
 
-	bool show_unit_genders();
-	void set_show_unit_genders(bool value);
+	bool show_fps();
+	void set_show_fps(bool value);
 
 	std::set<std::string> &encountered_units();
 	std::set<std::string> &encountered_terrains();
