@@ -1,4 +1,4 @@
-/* $Id: replay.cpp,v 1.81 2004/10/03 19:23:10 Sirp Exp $ */
+/* $Id: replay.cpp,v 1.90 2005/01/28 20:17:46 silene Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -10,6 +10,9 @@
 
    See the COPYING file for more details.
 */
+
+#include "global.hpp"
+
 #include "actions.hpp"
 #include "ai_interface.hpp"
 #include "dialogs.hpp"
@@ -26,6 +29,7 @@
 #include "statistics.hpp"
 #include "unit_display.hpp"
 #include "util.hpp"
+#include "wassert.hpp"
 
 #include <cstdio>
 #include <cstdlib>
@@ -101,9 +105,9 @@ namespace {
 	config create_verification(const unit_map& units)
 	{
 		config res;
-		char buf[50];
-		sprintf(buf,"%d",units.size());
-		res["num_units"] = buf;
+		std::stringstream buf;
+		buf << units.size();
+		res["num_units"] = buf.str();
 
 		for(unit_map::const_iterator i = units.begin(); i != units.end(); ++i) {
 			config u;
@@ -223,8 +227,8 @@ bool replay::skipping() const
 	return at_end() == false && skip_ != 0;
 }
 
-void replay::save_game(game_data& data, const std::string& label, const config& snapshot,
-					   const config& starting_pos, bool include_replay)
+void replay::save_game(const std::string& label, const config& snapshot,
+                       const config& starting_pos, bool include_replay)
 {
 	log_scope("replay::save_game");
 	saveInfo_.snapshot = snapshot;
@@ -496,13 +500,13 @@ int replay::get_random()
 
 const config* replay::get_random_results() const
 {
-	assert(random_ != NULL);
+	wassert(random_ != NULL);
 	return random_->child("results");
 }
 
 void replay::set_random_results(const config& cfg)
 {
-	assert(random_ != NULL);
+	wassert(random_ != NULL);
 	random_->clear_children("results");
 	random_->add_child("results",cfg);
 }
@@ -893,7 +897,7 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 			tgt = units.find(dst);
 
 			if(u != units.end() && tgt != units.end()) {
-				attack(disp,map,teams,src,dst,weapon_num,units,state,gameinfo,false);
+				attack(disp, map, teams, src, dst, weapon_num, units, state, gameinfo);
 			}
 
 			u = units.find(src);
