@@ -1,4 +1,4 @@
-/* $Id: sdl_utils.cpp,v 1.57 2004/09/03 21:42:00 silene Exp $ */
+/* $Id: sdl_utils.cpp,v 1.71 2005/01/09 00:02:52 silene Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -11,9 +11,12 @@
    See the COPYING file for more details.
 */
 
+#include "global.hpp"
+
 #include <algorithm>
 #include <cmath>
 #include <iostream>
+#include <map>
 
 #include "config.hpp"
 #include "game.hpp"
@@ -22,6 +25,29 @@
 #include "show_dialog.hpp"
 #include "util.hpp"
 #include "video.hpp"
+#include "wassert.hpp"
+
+SDLKey sdl_keysym_from_name(std::string const &keyname)
+{
+	static bool initialized = false;
+	typedef std::map<std::string const, SDLKey> keysym_map_t;
+	static keysym_map_t keysym_map;
+
+	if (!initialized) {
+		for(SDLKey i = SDLK_FIRST; i < SDLK_LAST; i = SDLKey(int(i) + 1)) {
+			std::string name = SDL_GetKeyName(i);
+			if (!name.empty())
+				keysym_map[name] = i;
+		}
+		initialized = true;
+	}
+
+	keysym_map_t::const_iterator it = keysym_map.find(keyname);
+	if (it != keysym_map.end())
+		return it->second;
+	else
+		return SDLK_UNKNOWN;
+}
 
 bool point_in_rect(int x, int y, const SDL_Rect& rect)
 {
@@ -34,6 +60,16 @@ bool rects_overlap(const SDL_Rect& rect1, const SDL_Rect& rect2)
 	       point_in_rect(rect1.x,rect1.y+rect1.h,rect2) || point_in_rect(rect1.x+rect1.w,rect1.y+rect1.h,rect2);
 }
 
+SDL_Rect intersect_rects(SDL_Rect const &rect1, SDL_Rect const &rect2)
+{
+	SDL_Rect res;
+	res.x = maximum<int>(rect1.x, rect2.x);
+	res.y = maximum<int>(rect1.y, rect2.y);
+	res.w = maximum<int>(minimum<int>(rect1.x + rect1.w, rect2.x + rect2.w) - res.x, 0);
+	res.h = maximum<int>(minimum<int>(rect1.y + rect1.h, rect2.y + rect2.h) - res.y, 0);
+	return res;
+}
+
 bool operator<(const surface& a, const surface& b)
 {
 	return a.get() < b.get();
@@ -57,7 +93,7 @@ namespace {
 
 }
 
-surface make_neutral_surface(surface surf)
+surface make_neutral_surface(surface const &surf)
 {
 	if(surf == NULL) {
 		std::cerr << "null neutral surface...\n";
@@ -73,7 +109,7 @@ surface make_neutral_surface(surface surf)
 }
 
 
-surface create_optimized_surface(surface surf)
+surface create_optimized_surface(surface const &surf)
 {
 	if(surf == NULL)
 		return NULL;
@@ -81,16 +117,16 @@ surface create_optimized_surface(surface surf)
 	surface const result = display_format_alpha(surf);
 	if(result == surf) {
 		std::cerr << "resulting surface is the same as the source!!!\n";
+	} else if(result == NULL) {
+		return surf;
 	}
 
-	if(result != NULL) {
-		SDL_SetAlpha(result,SDL_SRCALPHA|SDL_RLEACCEL,SDL_ALPHA_OPAQUE);
-	}
+	SDL_SetAlpha(result,SDL_SRCALPHA|SDL_RLEACCEL,SDL_ALPHA_OPAQUE);
 
 	return result;
 }
 
-surface scale_surface(surface surf, int w, int h)
+surface scale_surface(surface const &surf, int w, int h)
 {
 	if(surf == NULL)
 		return NULL;
@@ -134,7 +170,7 @@ surface scale_surface(surface surf, int w, int h)
 	return create_optimized_surface(dst);
 }
 
-surface scale_surface_blended(surface surf, int w, int h)
+surface scale_surface_blended(surface const &surf, int w, int h)
 {
 	if(surf== NULL)
 		return NULL;
@@ -174,12 +210,12 @@ surface scale_surface_blended(surface surf, int w, int h)
 				//we now have a rectangle, (xsrc,ysrc,xratio,yratio) which we
 				//want to derive the pixel from
 				for(double xloc = xsrc; xloc < xsrc+xratio; xloc += 1.0) {
-					const double xsize = minimum<double>(floor(xloc+1.0)-xloc,xsrc+xratio-xloc);
+					const double xsize = minimum<double>(std::floor(xloc+1.0)-xloc,xsrc+xratio-xloc);
 					for(double yloc = ysrc; yloc < ysrc+yratio; yloc += 1.0) {
 						const int xsrcint = maximum<int>(0,minimum<int>(src->w-1,static_cast<int>(xsrc)));
 						const int ysrcint = maximum<int>(0,minimum<int>(src->h-1,static_cast<int>(ysrc)));
 
-						const double ysize = minimum<double>(floor(yloc+1.0)-yloc,ysrc+yratio-yloc);		
+						const double ysize = minimum<double>(std::floor(yloc+1.0)-yloc,ysrc+yratio-yloc);		
 
 						Uint8 r,g,b,a;
 
@@ -210,7 +246,7 @@ surface scale_surface_blended(surface surf, int w, int h)
 	return create_optimized_surface(dst);
 }
 
-surface adjust_surface_colour(surface surf, int r, int g, int b)
+surface adjust_surface_colour(surface const &surf, int r, int g, int b)
 {
 	if(r == 0 && g == 0 && b == 0 || surf == NULL)
 		return create_optimized_surface(surf);
@@ -244,7 +280,7 @@ surface adjust_surface_colour(surface surf, int r, int g, int b)
 	return create_optimized_surface(nsurf);
 }
 
-surface greyscale_image(surface surf)
+surface greyscale_image(surface const &surf)
 {
 	if(surf == NULL)
 		return NULL;
@@ -284,7 +320,7 @@ surface greyscale_image(surface surf)
 	return create_optimized_surface(nsurf);
 }
 
-surface brighten_image(surface surf, double amount)
+surface brighten_image(surface const &surf, double amount)
 {
 	if(surf == NULL) {
 		return NULL;
@@ -319,7 +355,7 @@ surface brighten_image(surface surf, double amount)
 	return create_optimized_surface(nsurf);
 }
 
-surface adjust_surface_alpha(surface surf, double amount)
+surface adjust_surface_alpha(surface const &surf, double amount)
 {
 	if(surf== NULL) {
 		return NULL;
@@ -352,7 +388,7 @@ surface adjust_surface_alpha(surface surf, double amount)
 	return create_optimized_surface(nsurf);
 }
 
-surface adjust_surface_alpha_add(surface surf, int amount)
+surface adjust_surface_alpha_add(surface const &surf, int amount)
 {
 	if(surf== NULL) {
 		return NULL;
@@ -386,7 +422,7 @@ surface adjust_surface_alpha_add(surface surf, int amount)
 }
 
 // Applies a mask on a surface
-surface mask_surface(surface surf, surface mask)
+surface mask_surface(surface const &surf, surface const &mask)
 {
 	if(surf == NULL) {
 		return NULL;
@@ -429,8 +465,92 @@ surface mask_surface(surface surf, surface mask)
 	//return create_optimized_surface(nsurf);
 }
 
+// Cross-fades a surface
+surface blur_surface(surface const &surf, int depth)
+{
+	if(surf == NULL) {
+		return NULL;
+	}
+
+	surface nsurf = make_neutral_surface(surf);
+	surface res = create_compatible_surface(nsurf, surf->w, surf->h);
+	
+	if(nsurf == NULL || res == NULL) {
+		std::cerr << "could not make neutral surface...\n";
+		return NULL;
+	}
+
+	for(int count = 0; count < depth; ++count) {
+		surface_lock lock(nsurf);
+		surface_lock rlock(res);
+
+		{
+			Uint32* p = lock.pixels();
+			Uint32* left = p - 1;
+			Uint32* right = p + 1;
+			Uint32* top = p - res->w;
+			Uint32* bottom = p + res->w;
+			Uint32* rp = rlock.pixels();
+
+			for(int y = 0; y < res->h; ++y) {
+				for(int x = 0; x < res->w; ++x) {
+					Uint8 red, green, blue, alpha;
+					Uint16 rred=0, rgreen=0, rblue=0, ralpha=0;
+
+					if (x != 0) {
+						SDL_GetRGBA(*left, nsurf->format, &red, &green, &blue, &alpha);
+						rred += red;
+						rgreen += green;
+						rblue += blue;
+						ralpha += alpha;
+					}
+
+					if (x != res->w - 1) {
+						SDL_GetRGBA(*right, nsurf->format, &red, &green, &blue, &alpha);
+						rred += red;
+						rgreen += green;
+						rblue += blue;
+						ralpha += alpha;
+					}
+
+					if (y != 0) {
+						SDL_GetRGBA(*top, nsurf->format, &red, &green, &blue, &alpha);
+						rred += red;
+						rgreen += green;
+						rblue += blue;
+						ralpha += alpha;
+					}
+
+					if (y != res->h - 1) {
+						SDL_GetRGBA(*bottom, nsurf->format, &red, &green, &blue, &alpha);
+						rred += red;
+						rgreen += green;
+						rblue += blue;
+						ralpha += alpha;
+					}
+
+					rred /= 4;
+					rgreen /= 4;
+					rblue /= 4;
+					ralpha /= 4;
+
+					*rp = SDL_MapRGBA(res->format, (Uint8)rred, (Uint8)rgreen, (Uint8)rblue, (Uint8)ralpha);
+
+					++left; ++right; ++top; ++bottom; ++rp;
+				}
+			}
+		}
+
+		if (count < depth - 1) {
+			nsurf = res;
+		}
+	}
+
+	return create_optimized_surface(res);
+}
+
 // Cuts a rectangle from a surface.
-surface cut_surface(surface surf, const SDL_Rect& r)
+surface cut_surface(surface const &surf, SDL_Rect const &r)
 {
 	surface res = create_compatible_surface(surf, r.w, r.h);
 
@@ -453,14 +573,14 @@ surface cut_surface(surface surf, const SDL_Rect& r)
 		Uint8* line_dest = dest + y * rpitch;
 		size_t size = r.w + r.x <= surf->w ? r.w : surf->w - r.x; 
 		
-		assert(rpitch >= r.w * rbpp);
+		wassert(rpitch >= r.w * rbpp);
 		memcpy(line_dest, line_src, size * rbpp);
 	}
 
 	return res;
 }
 
-surface blend_surface(surface surf, double amount, Uint32 colour)
+surface blend_surface(surface const &surf, double amount, Uint32 colour)
 {
 	if(surf== NULL) {
 		return NULL;
@@ -504,7 +624,7 @@ surface blend_surface(surface surf, double amount, Uint32 colour)
 	return create_optimized_surface(nsurf);
 }
 
-surface flip_surface(surface surf)
+surface flip_surface(surface const &surf)
 {
 	if(surf == NULL) {
 		return NULL;
@@ -533,7 +653,7 @@ surface flip_surface(surface surf)
 	return create_optimized_surface(nsurf);
 }
 
-surface flop_surface(surface surf)
+surface flop_surface(surface const &surf)
 {
 	if(surf == NULL) {
 		return NULL;
@@ -562,7 +682,8 @@ surface flop_surface(surface surf)
 	return create_optimized_surface(nsurf);
 }
 
-surface create_compatible_surface(surface surf, int width, int height)
+
+surface create_compatible_surface(surface const &surf, int width, int height)
 {
 	if(surf == NULL)
 		return NULL;
@@ -577,7 +698,7 @@ surface create_compatible_surface(surface surf, int width, int height)
 		                        surf->format->Rmask,surf->format->Gmask,surf->format->Bmask,surf->format->Amask);
 }
 
-void fill_rect_alpha(SDL_Rect& rect, Uint32 colour, Uint8 alpha, surface target)
+void fill_rect_alpha(SDL_Rect &rect, Uint32 colour, Uint8 alpha, surface const &target)
 {
 	if(alpha == SDL_ALPHA_OPAQUE) {
 		SDL_FillRect(target,&rect,colour);
@@ -597,7 +718,7 @@ void fill_rect_alpha(SDL_Rect& rect, Uint32 colour, Uint8 alpha, surface target)
 	SDL_BlitSurface(tmp,NULL,target,&rect);
 }
 
-surface get_surface_portion(surface src, SDL_Rect& area)
+surface get_surface_portion(surface const &src, SDL_Rect &area)
 {
 	if(area.x >= src->w || area.y >= src->h) {
 		std::cerr << "illegal surface portion...\n";
@@ -643,7 +764,7 @@ private:
 
 }
 
-SDL_Rect get_non_transperant_portion(surface surf)
+SDL_Rect get_non_transperant_portion(surface const &surf)
 {
 	SDL_Rect res = {0,0,0,0};
 	const surface nsurf(make_neutral_surface(surf));
@@ -761,12 +882,27 @@ surface_restorer::~surface_restorer()
 	restore();
 }
 
-void surface_restorer::restore()
+void surface_restorer::restore(SDL_Rect const &dst) const
 {
-	if(surface_ != NULL) {
-		SDL_BlitSurface(surface_,NULL,target_->getSurface(),&rect_);
-		update_rect(rect_);
-	}
+	if (surface_.null())
+		return;
+	SDL_Rect dst2 = intersect_rects(dst, rect_);
+	if (dst2.w == 0 || dst2.h == 0)
+		return;
+	SDL_Rect src = dst2;
+	src.x -= rect_.x;
+	src.y -= rect_.y;
+	SDL_BlitSurface(surface_, &src, target_->getSurface(), &dst2);
+	update_rect(dst2);
+}
+
+void surface_restorer::restore() const
+{
+	if (surface_.null())
+		return;
+	SDL_Rect dst = rect_;
+	SDL_BlitSurface(surface_, NULL, target_->getSurface(), &dst);
+	update_rect(rect_);
 }
 
 void surface_restorer::update()
