@@ -1,6 +1,9 @@
+#include "../global.hpp"
+
 #include "game.hpp"
+#include "../util.hpp"
+#include "../wassert.hpp"
 
-#include <cassert>
 #include <iostream>
 
 int game::id_num = 1;
@@ -10,7 +13,7 @@ game::game(const player_map& info) : player_info_(&info), id_(id_num++), started
 
 bool game::is_owner(network::connection player) const
 {
-	assert(!players_.empty());
+	wassert(!players_.empty());
 	return player == players_.front();
 }
 
@@ -21,7 +24,7 @@ bool game::is_member(network::connection player) const
 
 bool game::is_needed(network::connection player) const
 {
-	assert(!players_.empty());
+	wassert(!players_.empty());
 	return player == players_.front();
 }
 
@@ -113,7 +116,7 @@ void game::start_game()
 
 bool game::take_side(network::connection player, const config& cfg)
 {
-	assert(is_member(player));
+	wassert(is_member(player));
 
 	const std::string& side = cfg["side"];
 	
@@ -190,6 +193,16 @@ const std::string& game::transfer_side_control(const config& cfg)
 		return already;
 	}
 
+	if(cfg["orphan_side"] != "yes") {
+		const size_t nsides = level_.get_children("side").size();
+		const size_t active_side = end_turn_/nsides + 1;
+
+		if(lexical_cast_default<size_t>(side,0) == active_side) {
+			static const std::string not_during_turn = "You cannot change a side's controller during its turn";
+			return not_during_turn;
+		}
+	}
+
 	config response;
 	config& change = response.add_child("change_controller");
 
@@ -207,6 +220,11 @@ const std::string& game::transfer_side_control(const config& cfg)
 
 	sides_taken_.insert(side);
 	
+	//send everyone a message saying that the observer who is taking the side has quit
+	config observer_quit;
+	observer_quit.add_child("observer_quit").values["name"] = player;
+	send_data(observer_quit);
+
 	static const std::string success = "";
 	return success;
 }
@@ -236,6 +254,7 @@ bool game::describe_slots()
 
 	if(buf != (*description())["slots"]) {
 		description()->values["slots"] = buf;
+		description()->values["observer"] = level_["observer"];
 		return true;
 	} else {
 		return false;
@@ -276,13 +295,13 @@ void game::ban_player(network::connection sock)
 
 bool game::process_commands(const config& cfg)
 {
-	std::cerr << "processing commands: '" << cfg.write() << "'\n";
+	//std::cerr << "processing commands: '" << cfg.write() << "'\n";
 	bool res = false;
 	const config::child_list& cmd = cfg.get_children("command");
 	for(config::child_list::const_iterator i = cmd.begin(); i != cmd.end(); ++i) {
 		if((**i).child("end_turn") != NULL) {
 			res = res || end_turn();
-			std::cerr << "res: " << (res ? "yes" : "no") << "\n";
+			//std::cerr << "res: " << (res ? "yes" : "no") << "\n";
 		}
 	}
 
@@ -316,6 +335,10 @@ void game::add_player(network::connection player)
 {
 	//if the game has already started, we add the player as an observer
 	if(started_) {
+		if(!allow_observers_) {
+			return;
+		}
+
 		const player_map::const_iterator info = player_info_->find(player);
 		if(info != player_info_->end()) {
 			config observer_join;
