@@ -3,6 +3,9 @@
 
 #include <iosfwd>
 
+#include <map>
+#include <string>
+
 class metrics
 {
 public:
@@ -11,6 +14,8 @@ public:
 	void service_request();
 	void no_requests();
 
+	void game_terminated(const std::string& reason);
+
 	friend std::ostream& operator<<(std::ostream& out, metrics& met);
 
 private:
@@ -19,6 +24,7 @@ private:
 	int nrequests_;
 	int nrequests_waited_;
 	const time_t started_at_;
+	std::map<std::string,int> terminations_;
 };
 
 std::ostream& operator<<(std::ostream& out, metrics& met);
