@@ -1,4 +1,4 @@
-/* $Id: show_dialog.cpp,v 1.96 2004/09/28 20:09:08 silene Exp $ */
+/* $Id: show_dialog.cpp,v 1.108 2005/01/23 15:35:38 j_daniel Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -11,6 +11,8 @@
    See the COPYING file for more details.
 */
 
+#include "global.hpp"
+
 #include "config.hpp"
 #include "cursor.hpp"
 #include "events.hpp"
@@ -29,6 +31,7 @@
 #include "widgets/button.hpp"
 #include "widgets/menu.hpp"
 #include "widgets/textbox.hpp"
+#include "wassert.hpp"
 
 #include "SDL_ttf.h"
 
@@ -37,6 +40,7 @@
 
 #define ERR_DP lg::err(lg::display)
 #define LOG_DP lg::info(lg::display)
+#define ERR_G  lg::err(lg::general)
 
 namespace {
 bool is_in_dialog = false;
@@ -57,6 +61,25 @@ dialog_manager::dialog_manager() : cursor::setter(cursor::NORMAL), reset_to(is_i
 dialog_manager::~dialog_manager()
 {
 	is_in_dialog = reset_to;
+	int mousex, mousey;
+	int mouse_flags = SDL_GetMouseState(&mousex, &mousey);
+	SDL_Event pb_event;
+	pb_event.type = SDL_MOUSEMOTION;
+	pb_event.motion.state = 0;
+	pb_event.motion.x = mousex;
+	pb_event.motion.y = mousey;
+	pb_event.motion.xrel = 0;
+	pb_event.motion.yrel = 0;
+	SDL_PushEvent(&pb_event);
+	if (!(mouse_flags & SDL_BUTTON_RMASK) || is_in_dialog)
+		return;
+	// based on krom's idea; remove if you don't like the "responsiveness"
+	pb_event.type = SDL_MOUSEBUTTONDOWN;
+	pb_event.button.button = SDL_BUTTON_RIGHT;
+	pb_event.button.state = SDL_PRESSED;
+	pb_event.button.x = mousex;
+	pb_event.button.y = mousey;
+	SDL_PushEvent(&pb_event);
 }
 
 void draw_dialog_frame(int x, int y, int w, int h, display& disp, const std::string* dialog_style, surface_restorer* restorer)
@@ -184,7 +207,8 @@ SDL_Rect draw_dialog_title(int x, int y, display* disp, const std::string& text)
 		rect = disp->screen_area();
 	}
 
-	return font::draw_text(disp,rect,font::SIZE_LARGE,font::TITLE_COLOUR,text,x,y+5,NULL,false,font::USE_MARKUP,TTF_STYLE_BOLD);
+	return font::draw_text(disp, rect, font::SIZE_LARGE, font::TITLE_COLOUR,
+	                       text, x, y + 5, false, TTF_STYLE_BOLD);
 }
 
 void draw_dialog(int x, int y, int w, int h, display& disp, const std::string& title,
@@ -358,6 +382,12 @@ private:
 
 }
 
+void show_error_message(display &disp, std::string const &message)
+{
+	ERR_G << message << std::endl;
+	show_dialog(disp, NULL, _("Error"), message, OK_ONLY);
+}
+
 int show_dialog(display& disp, surface image,
 				const std::string& caption, const std::string& msg,
 				DIALOG_TYPE type,
@@ -365,6 +395,7 @@ int show_dialog(display& disp, surface image,
 				const std::vector<preview_pane*>* preview_panes,
 				const std::string& text_widget_label,
 				std::string* text_widget_text,
+				int text_widget_max_chars,
 				dialog_action* action, std::vector<check_item>* options, int xloc, int yloc,
 				const std::string* dialog_style, std::vector<dialog_button>* action_buttons,
 				const std::string& help_topic)
@@ -401,7 +432,7 @@ int show_dialog(display& disp, surface image,
 	static const std::string default_text_string = "";
 	const unsigned int text_box_width = 350;
 	textbox text_widget(disp,text_box_width,
-	                    use_textbox ? *text_widget_text : default_text_string, editable_textbox);
+	                    use_textbox ? *text_widget_text : default_text_string, editable_textbox, text_widget_max_chars);
 
 	int text_widget_width = 0;
 	int text_widget_height = 0;
@@ -419,7 +450,13 @@ int show_dialog(display& disp, surface image,
 
 	menu menu_(disp,menu_items,type == MESSAGE);
 
+	menu_.set_numeric_keypress_selection(use_textbox == false);
+
+#ifdef USE_TINY_GUI
+	const int max_line_length = 30;
+#else
 	const int max_line_length = 54;
+#endif
 
 	std::string message = msg;
 	text_to_lines(message,max_line_length);
@@ -427,19 +464,20 @@ int show_dialog(display& disp, surface image,
 	SDL_Rect text_size = { 0, 0, 0, 0 };
 	if(!message.empty()) {
 		text_size = font::draw_text(NULL, clipRect, message_font_size,
-						            font::NORMAL_COLOUR, message, 0, 0, NULL);
+		                            font::NORMAL_COLOUR, message, 0, 0);
 	}
 
 	SDL_Rect caption_size = { 0, 0, 0, 0 };
 	if (!caption.empty() && image != NULL) {
 		caption_size = font::draw_text(NULL, clipRect, caption_font_size,
-		                               font::NORMAL_COLOUR,caption,0,0,NULL);
+		                               font::NORMAL_COLOUR, caption, 0, 0);
 	}
 
 	const char** button_list = NULL;
 	std::vector<button> buttons;
 	switch(type) {
 		case MESSAGE:
+		default:
 			break;
 
 		case OK_ONLY: {
@@ -662,7 +700,7 @@ int show_dialog(display& disp, surface image,
 	const int menu_xpos = xloc+image_width+left_padding+image_h_padding;
 	const int menu_ypos = yloc+top_padding+text_and_image_height+menu_hpadding+ (use_textbox ? text_widget.location().h + top_padding : 0);
 	if(menu_.height() > 0) {
-		menu_.set_loc(menu_xpos,menu_ypos);
+		menu_.set_location(menu_xpos,menu_ypos);
 	}
 
 	if(image != NULL) {
@@ -674,7 +712,7 @@ int show_dialog(display& disp, surface image,
 		font::draw_text(&disp, clipRect, caption_font_size,
 		                font::NORMAL_COLOUR, caption,
 		                xloc+image_width+left_padding+image_h_padding,
-		                yloc+top_padding, NULL);
+		                yloc+top_padding);
 	}
 
 	font::draw_text(&disp, clipRect, message_font_size,
@@ -720,22 +758,16 @@ int show_dialog(display& disp, surface image,
 		int mousex, mousey;
 		const int mouse_flags = SDL_GetMouseState(&mousex,&mousey);
 
-		const bool new_right_button = mouse_flags&SDL_BUTTON_RMASK;
-		const bool new_left_button = mouse_flags&SDL_BUTTON_LMASK;
+		const bool new_right_button = (mouse_flags&SDL_BUTTON_RMASK) != 0;
+		const bool new_left_button = (mouse_flags&SDL_BUTTON_LMASK) != 0;
 		const bool new_key_down = key[SDLK_SPACE] || key[SDLK_RETURN] ||
 		                          key[SDLK_ESCAPE];
 
-		const bool new_up_arrow = key[SDLK_UP];
-		const bool new_down_arrow = key[SDLK_DOWN];
+		const bool new_up_arrow = key[SDLK_UP] != 0;
+		const bool new_down_arrow = key[SDLK_DOWN] != 0;
 
-		const bool new_page_up = key[SDLK_PAGEUP];
-		const bool new_page_down = key[SDLK_PAGEDOWN];
-
-		int select_item = -1;
-		for(int item = 0; item != 10; ++item) {
-			if(key['1' + item])
-				select_item = item;
-		}
+		const bool new_page_up = key[SDLK_PAGEUP] != 0;
+		const bool new_page_down = key[SDLK_PAGEDOWN] != 0;
 
 		if((!key_down && key[SDLK_RETURN] || menu_.double_clicked()) &&
 		   (type == YES_NO || type == OK_CANCEL || type == OK_ONLY || type == CLOSE_ONLY)) {
@@ -786,12 +818,7 @@ int show_dialog(display& disp, surface image,
 		first_time = false;
 
 		if(menu_.height() > 0) {
-			const int res = menu_.process(mousex,mousey,new_left_button,
-			                              !up_arrow && new_up_arrow,
-			                              !down_arrow && new_down_arrow,
-			                              !page_up && new_page_up,
-			                              !page_down && new_page_down,
-			                              select_item);
+			const int res = menu_.process();
 			if(res != -1)
 			{
 				return res;	
@@ -843,14 +870,13 @@ int show_dialog(display& disp, surface image,
 		}
 
 		for(unsigned int n = 0; n != check_buttons.size(); ++n) {
-			const bool pressed = check_buttons[n].process(mousex,mousey,left_button);
-			check_buttons[n].draw();
+			const bool pressed = check_buttons[n].pressed();
 
 			if(options != NULL && n < options->size()) {
 				(*options)[n].checked = check_buttons[n].checked();
 			} else if(pressed) {
 				const size_t options_size = options == NULL ? 0 : options->size();
-				assert(action_buttons != NULL && action_buttons->size() > n - options_size);
+				wassert(action_buttons != NULL && action_buttons->size() > n - options_size);
 
 				const dialog_button_action::RESULT res = (*action_buttons)[n - options_size].handler->button_pressed(menu_.selection());
 				if(res == dialog_button_action::DELETE_ITEM) {
@@ -865,15 +891,15 @@ int show_dialog(display& disp, surface image,
 			}
 		}
 
+		disp.video().flip();
+		SDL_Delay(10);
+
 		if(action != NULL) {
 			const int act = action->do_action();
 			if(act != dialog_action::CONTINUE_DIALOG) {
 				return act;
 			}
 		}
-
-		disp.video().flip();
-		SDL_Delay(10);
 	}
 
 	return -1;
@@ -935,7 +961,7 @@ network::connection network_data_dialog(display& disp, const std::string& msg, c
 		}
 
 		dialog_action_receive_network receiver(connection_num,cfg,stats);
-		const int res = show_dialog(disp,NULL,"",str.str(),CANCEL_ONLY,NULL,NULL,"",NULL,&receiver);
+		const int res = show_dialog(disp,NULL,"",str.str(),CANCEL_ONLY,NULL,NULL,"",NULL,-1,&receiver);
 		if(res != int(dialog_action_receive_network::CONNECTION_CONTINUING)) {
 			return receiver.result();
 		}
