@@ -1,4 +1,4 @@
-/* $Id: show_dialog.hpp,v 1.40 2004/09/18 18:52:20 gruikya Exp $ */
+/* $Id: show_dialog.hpp,v 1.42 2004/12/19 22:06:33 silene Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -138,6 +138,7 @@ int show_dialog(display& screen, surface image,
 				const std::vector<preview_pane*>* preview_panes=NULL,
 				const std::string& text_widget_label="",
 				std::string* text_widget_text=NULL,
+				const int text_widget_max_chars = 256,
 				dialog_action* action=NULL,
 				std::vector<check_item>* options=NULL, int xloc=-1, int yloc=-1,
 				const std::string* dialog_style=NULL,
@@ -145,6 +146,8 @@ int show_dialog(display& screen, surface image,
 				const std::string& help_topic=""
 			 );
 
+void show_error_message(display &screen, std::string const &message);
+
 network::connection network_data_dialog(display& disp, const std::string& msg, config& cfg, network::connection connection_num=0);
 
 void check_quit(display& screen);
