@@ -1,4 +1,4 @@
-/* $Id: sound.cpp,v 1.23 2004/09/15 18:52:51 cedricd Exp $ */
+/* $Id: sound.cpp,v 1.27 2005/01/08 13:57:23 ydirson Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -11,15 +11,13 @@
    See the COPYING file for more details.
 */
 
-//disable the very annoying VC++ warning 4786
-#ifdef WIN32
-#pragma warning(disable:4786)
-#endif
+#include "global.hpp"
 
 #include "filesystem.hpp"
 #include "game_config.hpp"
 #include "log.hpp"
 #include "sound.hpp"
+#include "wesconfig.h"
 
 #include "SDL_mixer.h"
 
@@ -136,7 +134,20 @@ void play_sound(const std::string& file)
 
 	std::map<std::string,Mix_Chunk*>::const_iterator itor = sound_cache.find(file);
 	if(itor == sound_cache.end()) {
+#ifdef USE_ZIPIOS
+		std::string s = read_file("sounds/" + file);
+		if (s.empty()) {
+			return;
+		}
 
+		SDL_RWops* ops = SDL_RWFromMem((void*)s.c_str(), s.size());
+		Mix_Chunk* const sfx = Mix_LoadWAV_RW(ops,0);
+		if(sfx == NULL) {
+			std::cerr << "Could not load sound file '" << file << "': "
+			          << SDL_GetError() << "\n";
+			return;
+		}
+#else
 		const std::string& filename = get_binary_file_location("sounds",file);
 
 		if(filename.empty()) {
@@ -149,6 +160,7 @@ void play_sound(const std::string& file)
 			          << SDL_GetError() << "\n";
 			return;
 		}
+#endif
 
 		itor = sound_cache.insert(std::pair<std::string,Mix_Chunk*>(file,sfx)).first;
 	}
