@@ -1,4 +1,4 @@
-/* $Id: team.cpp,v 1.72 2004/09/21 00:50:47 Sirp Exp $ */
+/* $Id: team.cpp,v 1.78 2005/02/03 19:48:00 silene Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -11,18 +11,24 @@
    See the COPYING file for more details.
 */
 
+#include "global.hpp"
+
 #include "game_config.hpp"
 #include "game_events.hpp"
 #include "gamestatus.hpp"
+#include "log.hpp"
 #include "network.hpp"
 #include "replay.hpp"
 #include "team.hpp"
 #include "util.hpp"
+#include "wassert.hpp"
 
 #include <algorithm>
 #include <cstdlib>
 #include <sstream>
 
+#define LOG_NG lg::info(lg::engine)
+
 namespace {
 	std::vector<team>* teams = NULL;
 }
@@ -94,15 +100,20 @@ team::team_info::team_info(const config& cfg)
 		}
 	}
 
-	if(cfg["controller"] == "human")
+	std::string control = cfg["controller"];
+	if (control == "human")
 		controller = HUMAN;
-	else if(cfg["controller"] == "network")
+	else if (control == "network")
 		controller = NETWORK;
-	else if(cfg["controller"] == "null")
+	else if (control == "null")
 		controller = EMPTY;
 	else
 		controller = AI;
 
+	std::string persist = cfg["persistent"];
+	if (persist.empty()) persistent = controller == HUMAN;
+	else persistent = persist == "1";
+
 	ai_algorithm = cfg["ai_algorithm"];
 
 	if(ai_algorithm.empty()) {
@@ -178,8 +189,8 @@ team::team_info::team_info(const config& cfg)
 	share_view = (cfg["share_view"] == "yes");
 	share_maps = !share_view && (cfg["share_maps"] != "no");
 
-	std::cerr << "team_info::team_info(...): team_name: " << team_name;
-	std::cerr << ", share_maps: " << share_maps << ", share_view: " << share_view << ".\n";
+	LOG_NG << "team_info::team_info(...): team_name: " << team_name
+	       << ", share_maps: " << share_maps << ", share_view: " << share_view << ".\n";
 	
 	music = cfg["music"];
 }
@@ -216,9 +227,11 @@ void team::team_info::write(config& cfg) const
 	case HUMAN: cfg["controller"] = "human"; break;
 	case NETWORK: cfg["controller"] = "network"; break;
 	case EMPTY: cfg["controller"] = "null"; break;
-	default: assert(false);
+	default: wassert(false);
 	}
 
+	cfg["persistent"] = persistent ? "1" : "0";
+
 	sprintf(buf,"%d",villages_per_scout);
 	cfg["villages_per_scout"] = buf;
 
@@ -267,8 +280,8 @@ team::team(const config& cfg, int gold) : gold_(gold), auto_shroud_updates_(true
 	shroud_.set_enabled(cfg["shroud"] == "yes");
 	shroud_.read(cfg["shroud_data"]);	
 	
-	std::cerr << "team::team(...): team_name: " << info_.team_name;
-	std::cerr << ", shroud: " << uses_shroud() << ", fog: " << uses_fog() << ".\n";
+	LOG_NG << "team::team(...): team_name: " << info_.team_name
+	       << ", shroud: " << uses_shroud() << ", fog: " << uses_fog() << ".\n";
 	
 	//gold is the maximum of 'gold' and what is given in the config file
 	if(info_.gold.empty() == false)
@@ -378,7 +391,8 @@ void team::set_time_of_day(int turn, const time_of_day& tod)
 	aggression_ = lexical_cast_default<double>(aiparams_["aggression"],0.5);
 	caution_ = lexical_cast_default<double>(aiparams_["caution"],0.25);
 
-	std::cerr << "for turn " << turn << ", time of day '" << tod.id << "' set ai_params to: ---\n" << aiparams_.write() << "\n---\n";
+	LOG_NG << "for turn " << turn << ", time of day '" << tod.id << "' set ai_params to: ---\n"
+	       << aiparams_.write() << "\n---\n";
 }
 
 void team::spend_gold(int amount)
@@ -473,6 +487,11 @@ bool team::is_empty() const
 	return info_.controller == team_info::EMPTY;
 }
 
+bool team::is_persistent() const
+{
+	return info_.persistent;
+}
+
 void team::make_human()
 {
 	info_.controller = team_info::HUMAN;
@@ -676,7 +695,6 @@ void validate_side(int side)
 	}
 
 	if(side < 1 || side > int(teams->size())) {
-		std::cerr << "invalid side " << side << " throwing game_error\n";
 		throw gamestatus::game_error("invalid side found in unit definition");
 	}
 }
