@@ -1,4 +1,4 @@
-/* $Id: unit.cpp,v 1.99 2004/10/02 15:53:58 Sirp Exp $ */
+/* $Id: unit.cpp,v 1.120 2005/01/22 12:09:36 silene Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -11,6 +11,8 @@
    See the COPYING file for more details.
 */
 
+#include "global.hpp"
+
 #include "font.hpp"
 #include "game_config.hpp"
 #include "gamestatus.hpp"
@@ -21,6 +23,7 @@
 #include "replay.hpp"
 #include "unit.hpp"
 #include "util.hpp"
+#include "wassert.hpp"
 
 #include <algorithm>
 #include <cstdlib>
@@ -53,6 +56,7 @@ bool compare_unit_values::operator()(const unit& a, const unit& b) const
 //constructor for reading a unit
 unit::unit(const game_data& data, const config& cfg) : state_(STATE_NORMAL),
                                            moves_(0), user_end_turn_(false), facingLeft_(true),
+					   resting_(false),
                                            recruit_(false),
                                            guardian_(false), upkeep_(UPKEEP_FREE)
 {
@@ -61,10 +65,6 @@ unit::unit(const game_data& data, const config& cfg) : state_(STATE_NORMAL),
 
 unit_race::GENDER unit::generate_gender(const unit_type& type, bool gen)
 {
-	if(preferences::show_unit_genders() == false) {
-		gen = false;
-	}
-
 	const std::vector<unit_race::GENDER>& genders = type.genders();
 	if(genders.empty() == false) {
 		return gen ? genders[get_random()%genders.size()] : genders.front();
@@ -88,7 +88,8 @@ unit::unit(const unit_type* t, int side, bool use_traits, bool dummy_unit, unit_
 			   backupMaxMovement_(type_->movement()),
 			   recruit_(false), attacks_(type_->attacks()),
 			   backupAttacks_(type_->attacks()),
-               guardian_(false), upkeep_(UPKEEP_FULL_PRICE)
+               guardian_(false), upkeep_(UPKEEP_FULL_PRICE),
+               unrenamable_(false)
 {
 	//dummy units used by the 'move_unit_fake' command don't need to have a side.
 	if(dummy_unit == false) validate_side(side_);
@@ -124,7 +125,8 @@ unit::unit(const unit_type* t, const unit& u) :
 			   attacks_(type_->attacks()), backupAttacks_(type_->attacks()),
 			   modifications_(u.modifications_),
 			   traitsDescription_(u.traitsDescription_),
-			   guardian_(false), upkeep_(u.upkeep_)
+			   guardian_(false), upkeep_(u.upkeep_),
+			   unrenamable_(u.unrenamable_)
 {
 	validate_side(side_);
 
@@ -145,39 +147,33 @@ void unit::generate_traits()
 		return;
 
 	//calculate the unit's traits
-	const std::vector<config*> traits = type().possible_traits();
+	std::vector<config*> candidate_traits = type().possible_traits();
+	std::vector<config*> traits;
 
 	const size_t num_traits = type().num_traits();
-	if(traits.size() >= num_traits) {
-		std::set<int> chosen_traits;
-		for(size_t i = 0; i != num_traits; ++i) {
-			int num = get_random()%(traits.size()-i);
-			while(chosen_traits.count(num)) {
-				++num;
-			}
+	for(size_t n = 0; n != num_traits && candidate_traits.empty() == false; ++n) {
+		const int num = get_random()%candidate_traits.size();
+		traits.push_back(candidate_traits[num]);
+		candidate_traits.erase(candidate_traits.begin()+num);
+	}
 
-			chosen_traits.insert(num);
+	std::vector<std::string> description;
+	
+	for(std::vector<config*>::const_iterator j = traits.begin(); j != traits.end(); ++j) {
+		add_modification("trait",**j);
+		description.push_back((**j)["name"]);
+	}
 
-			add_modification("trait",*traits[num]);
+	traitsDescription_ = "";
+
+	//we want to make sure the description always has a consistent ordering
+	std::sort(description.begin(),description.end());
+	for(std::vector<std::string>::const_iterator i = description.begin(); i != description.end(); ++i) {
+		if(i != description.begin()) {
+			traitsDescription_ += ", ";
 		}
 
-		//build the traits description, making sure the traits are always
-		//in the same order.
-		for(std::set<int>::const_iterator itor = chosen_traits.begin();
-		    itor != chosen_traits.end(); ++itor) {
-			const std::string& trait_name = (*traits[*itor])["name"];
-			const std::string& lang_trait = string_table["trait_"+trait_name];
-			if(lang_trait.empty() == false)
-				traitsDescription_ += lang_trait;
-			else
-				traitsDescription_ += trait_name;
-
-			traitsDescription_ += ",";
-		}
-
-		//get rid of the trailing comma
-		if(!traitsDescription_.empty())
-			traitsDescription_.resize(traitsDescription_.size()-1);
+		traitsDescription_ += gettext(i->c_str());
 	}
 }
 
@@ -215,7 +211,9 @@ const std::string& unit::unit_description() const
 
 void unit::rename(const std::string& new_description)
 {
-	description_ = new_description;
+	if (!unrenamable_) {
+		description_ = new_description;
+	}
 }
 
 int unit::side() const
@@ -233,6 +231,11 @@ void unit::set_side(int new_side)
 	side_ = new_side;
 }
 
+bool unit::unrenamable() const
+{
+	return unrenamable_;
+}
+
 double unit::alpha() const
 {
 	return type().alpha();
@@ -301,7 +304,6 @@ void unit::new_turn()
 		set_flag("ambush");
 	if(type().has_ability("nightstalk"))
 		set_flag("nightstalk");
-
 	if(stone())
 		set_attacked();
 }
@@ -402,10 +404,11 @@ void unit::heal()
 
 void unit::heal(int amount)
 {
-	if(hitpoints_ < max_hitpoints()) {
+	int max_hp = max_hitpoints();
+	if (hitpoints_ < max_hp) {
 		hitpoints_ += amount;
-		if(hitpoints_ > max_hitpoints())
-			hitpoints_ = max_hitpoints();
+		if (hitpoints_ > max_hp)
+			hitpoints_ = max_hp;
 	}
 }
 
@@ -414,6 +417,10 @@ void unit::heal_all()
 	hitpoints_ = max_hitpoints();
 }
 
+static bool is_terrain(std::string const &terrain, gamemap::TERRAIN type) {
+	return std::count(terrain.begin(), terrain.end(), static_cast<gamemap::TERRAIN>(type)) != 0;
+}
+
 bool unit::invisible(const std::string& terrain, int lawful_bonus, 
 		const gamemap::location& loc, 
 		const unit_map& units,const std::vector<team>& teams) const
@@ -421,12 +428,15 @@ bool unit::invisible(const std::string& terrain, int lawful_bonus,
 	bool is_inv = false;
 
 	static const std::string forest_invisible("ambush");
-	if(std::count(terrain.begin(),terrain.end(),static_cast<gamemap::TERRAIN>(gamemap::FOREST)) && has_flag(forest_invisible)) {
+	if (has_flag(forest_invisible) && is_terrain(terrain, gamemap::FOREST)) {
 		is_inv = true;
 	}
-	static const std::string night_invisible("nightstalk");
-	if((lawful_bonus < 0) && has_flag(night_invisible)) {
-		is_inv = true;
+	else
+	{
+		static const std::string night_invisible("nightstalk");
+		if ((lawful_bonus < 0) && has_flag(night_invisible)) {
+			is_inv = true;
+		}
 	}
 
 	if(is_inv){
@@ -459,6 +469,11 @@ bool unit::incapacitated() const
 	return stone();
 }
 
+bool unit::emits_zoc() const
+{
+	return type().has_zoc() && stone() == false;
+}
+
 bool unit::matches_filter(const config& cfg) const
 {
 	const std::string& description = cfg["description"];
@@ -470,6 +485,7 @@ bool unit::matches_filter(const config& cfg) const
 	const std::string& role = cfg["role"];
 	const std::string& race = cfg["race"];
 	const std::string& gender = cfg["gender"];
+	const std::string& canrecruit = cfg["canrecruit"];
 
 	if(description.empty() == false && description != this->underlying_description()) {
 		return false;
@@ -549,6 +565,9 @@ bool unit::matches_filter(const config& cfg) const
 		return false;
 	}
 
+	if (canrecruit.empty() == false && (canrecruit == "1") != can_recruit())
+		return false;
+
 	//if there are [not] tags below this tag, it means that the filter
 	//should not match if what is in the [not] tag does match
 	const config::child_list& negatives = cfg.get_children("not");
@@ -599,9 +618,9 @@ void unit::read(const game_data& data, const config& cfg)
 	else
 		throw gamestatus::load_game_failed("Unit not found: '" + cfg["type"] + "'");
 
-	assert(type_ != NULL);
+	wassert(type_ != NULL);
 
-	const std::string& gender = preferences::show_unit_genders() ? cfg["gender"] : "";
+	const std::string& gender = cfg["gender"];
 	if(gender == "male") {
 		gender_ = unit_race::MALE;
 	} else if(gender == "female") {
@@ -631,7 +650,7 @@ void unit::read(const game_data& data, const config& cfg)
 
 	description_ = cfg["user_description"];
 	underlying_description_ = cfg["description"];
-	if(description_ == "") {
+	if(description_.empty()) {
 		description_ = underlying_description_;
 	}
 
@@ -712,6 +731,7 @@ void unit::read(const game_data& data, const config& cfg)
 		experience_ = atoi(experience.c_str());
 
 	resting_ = (cfg["resting"] == "yes");
+	unrenamable_ = (cfg["unrenamable"] == "yes");
 }
 
 void unit::write(config& cfg) const
@@ -779,6 +799,7 @@ void unit::write(config& cfg) const
 	cfg["moves"] = buf;
 
 	cfg["resting"] = resting_ ? "yes" : "no"; 
+	cfg["unrenamable"] = unrenamable_ ? "yes" : "no";
 }
 
 void unit::assign_role(const std::string& role)
@@ -791,38 +812,8 @@ const std::vector<attack_type>& unit::attacks() const
 	return attacks_;
 }
 
-int unit::longest_range() const
-{
-	int res = 0;
-	for(std::vector<attack_type>::const_iterator i = attacks_.begin(); i != attacks_.end(); ++i) {
-		if(i->hexes() >= res) {
-			res = i->hexes();
-		}
-	}
-
-	return res;
-}
-
-std::vector<attack_type> unit::attacks_at_range(int range) const
-{
-	std::vector<attack_type> res;
-	for(std::vector<attack_type>::const_iterator i = attacks_.begin(); i != attacks_.end(); ++i) {
-		if(i->hexes() >= range)
-			res.push_back(*i);
-	}
-
-	return res;
-}
-
 int unit::movement_cost(const gamemap& map, gamemap::TERRAIN terrain) const
 {
-//don't allow level 0 units to take villages - removed until AI
-//is smart enough to deal with this.
-//	if(type_->level() == 0 && map.is_village(terrain))
-//		return 100;
-
-
-
 	const int res = type_->movement_type().movement_cost(map,terrain);
 
 	static const std::string slowed_string("slowed");
@@ -833,8 +824,7 @@ int unit::movement_cost(const gamemap& map, gamemap::TERRAIN terrain) const
 	return res;
 }
 
-int unit::defense_modifier(const gamemap& map,
-                              gamemap::TERRAIN terrain) const
+int unit::defense_modifier(const gamemap& map, gamemap::TERRAIN terrain) const
 {
 	return type_->movement_type().defense_modifier(map,terrain);
 }
@@ -850,12 +840,15 @@ const std::string& unit::image() const
 		case STATE_NORMAL: return type_->image();
 		case STATE_DEFENDING_LONG:
 		case STATE_DEFENDING_SHORT: {
+			if(get_animation()) {
+				const std::string& res = anim_.get_current_frame().image;
+				if(res != "") {
+					return res;
+				}
+			}
+
 			const attack_type::RANGE range = (state_ == STATE_DEFENDING_LONG) ? attack_type::LONG_RANGE : attack_type::SHORT_RANGE;
-			const unit_animation* const anim = type_->defend_animation(getsHit_,range);
-			if (anim != NULL)
-				return anim->get_current_frame().image;
-			else
-				return type_->image_defensive(range);
+			return type_->image_defensive(range);
 		}
 		case STATE_ATTACKING: {
 			if(attackType_ == NULL)
@@ -876,14 +869,42 @@ const std::string& unit::image() const
 	}
 }
 
-void unit::set_defending(bool newval, bool hits, int ms, attack_type::RANGE range)
+const unit_animation* unit::get_animation() const
 {
-	state_ = newval ? (range == attack_type::LONG_RANGE ? STATE_DEFENDING_LONG :
-	                   STATE_DEFENDING_SHORT): STATE_NORMAL;
-	// attackingMilliseconds_ = ms;
-	type_->defend_animation(getsHit_,range);
+	switch(state_) {
+	case STATE_DEFENDING_LONG:
+	case STATE_DEFENDING_SHORT: {
+			const attack_type::RANGE range = (state_ == STATE_DEFENDING_LONG) ? attack_type::LONG_RANGE : attack_type::SHORT_RANGE;
+			const unit_animation* const anim = type_->defend_animation(getsHit_,range);
+			return anim;
+	}
+	default:
+		return NULL;
+	}
+}
 
+void unit::set_standing()
+{
+	state_ = STATE_NORMAL;
+}
+
+void unit::set_defending(bool hits, attack_type::RANGE range, int start_frame, int acceleration)
+{
+	state_ = range == attack_type::LONG_RANGE ? STATE_DEFENDING_LONG : STATE_DEFENDING_SHORT;
 	getsHit_ = hits;
+					   
+	const unit_animation* const anim = get_animation();
+	if(anim != NULL) {
+		anim_ = *anim;
+		anim_.start_animation(start_frame,unit_animation::UNIT_FRAME,acceleration);
+	}
+}
+
+void unit::update_defending_frame()
+{
+	if(get_animation()) {
+		anim_.update_current_frames();
+	}
 }
 
 void unit::set_attacking(bool newval, const attack_type* type, int ms)
@@ -1018,7 +1039,7 @@ void unit::add_modification(const std::string& type,
 						description << "; ";
 					}
 					
-					description << string_table[a->name()] << " " << desc;
+					description << gettext(a->name().c_str()) << " " << desc;
 				}
 			}
 		} else if(apply_to == "hitpoints") {
@@ -1202,7 +1223,7 @@ int unit::upkeep() const
 	case UPKEEP_FREE: return 0;
 	case UPKEEP_LOYAL: return minimum<int>(1,type().level());
 	case UPKEEP_FULL_PRICE: return type().level();
-	default: assert(false); return 0;
+	default: wassert(false); return 0;
 	}
 }
 
