@@ -1,4 +1,4 @@
-/* $Id: unit.hpp,v 1.53 2004/08/29 20:33:01 Sirp Exp $ */
+/* $Id: unit.hpp,v 1.59 2005/02/06 10:40:12 isaaccp Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -51,6 +51,9 @@ public:
 	int experience() const;
 	int max_experience() const;
 	bool get_experience(int xp);
+	//the name of this unit is fixed by scenario, it is impossible to
+	//change it.
+	bool unrenamable() const;
 	bool advances() const;
 	int side() const;
 	unit_race::GENDER gender() const;
@@ -86,6 +89,8 @@ public:
 
 	bool incapacitated() const;
 
+	bool emits_zoc() const;
+
 	bool matches_filter(const config& cfg) const;
 
 	void set_flag(const std::string& flag);
@@ -103,8 +108,6 @@ public:
 	void assign_role(const std::string& role);
 
 	const std::vector<attack_type>& attacks() const;
-	int longest_range() const;
-	std::vector<attack_type> attacks_at_range(int range) const;
 
 	int movement_cost(const gamemap& map, gamemap::TERRAIN terrain) const;
 	int defense_modifier(const gamemap& map, gamemap::TERRAIN terrain) const;
@@ -114,8 +117,9 @@ public:
 	//(could be in the middle of an attack etc)
 	const std::string& image() const;
 
-	void set_defending(bool newval, bool hits=false, int ms=0,
-	                   attack_type::RANGE range=attack_type::LONG_RANGE);
+	void set_standing();
+	void set_defending(bool hits, attack_type::RANGE range, int start_frame, int acceleration);
+	void update_defending_frame();
 	void set_attacking(bool newval, const attack_type* type=NULL, int ms=0);
 
 	void set_leading(bool newval);
@@ -214,6 +218,11 @@ private:
 
 	UPKEEP_COST upkeep_;
 
+	bool unrenamable_;
+
+	unit_animation anim_;
+	const unit_animation* get_animation() const;
+
 	void reset_modifications();
 	void apply_modifications();
 	void remove_temporary_modifications();
