@@ -1,3 +1,5 @@
+#include "global.hpp"
+
 #include "actions.hpp"
 #include "events.hpp"
 #include "game_config.hpp"
@@ -9,6 +11,7 @@
 #include "sound.hpp"
 #include "unit_display.hpp"
 #include "util.hpp"
+#include "wassert.hpp"
 
 #define LOG_DP lg::info(lg::display)
 
@@ -62,11 +65,11 @@ void move_unit_between(display& disp, const gamemap& map, const gamemap::locatio
 	LOG_DP << "submerge: " << src_submerge << " -> " << dst_submerge << "\n";
 
 	const gamemap::TERRAIN terrain = map.get_terrain(b);
-	const int nsteps = disp.turbo() ? 3 : 10*u.movement_cost(map,terrain);
+	const int nsteps = disp.turbo() ? 3 : 7*u.movement_cost(map,terrain);
 	const double xstep = double(xdst - xsrc) / nsteps;
 	const double ystep = double(ydst - ysrc) / nsteps;
 
-	const int time_between_frames = disp.turbo() ? 2 : 10;
+	const int time_between_frames = disp.turbo() ? 10 : 20;
 	int ticks = SDL_GetTicks();
 
 	int skips = 0;
@@ -83,7 +86,8 @@ void move_unit_between(display& disp, const gamemap& map, const gamemap::locatio
 	}
 	
 	const unit_animation *teleport_animation_p = u.type().teleport_animation();
-	if (teleport_animation_p && !tiles_adjacent(a, b) && !disp.fogged(a.x, a.y)) { // teleport
+	bool teleport_unit = teleport_animation_p && !tiles_adjacent(a, b);
+	if (teleport_unit && !disp.fogged(a.x, a.y)) { // teleport
 		unit_animation teleport_animation =  *teleport_animation_p;
 		int animation_time;
 		const int begin_at = teleport_animation.get_first_frame_time(unit_animation::UNIT_FRAME);
@@ -144,33 +148,26 @@ void move_unit_between(display& disp, const gamemap& map, const gamemap::locatio
 
 		//invalidate the source tile and all adjacent tiles,
 		//since the unit can partially overlap adjacent tiles
-		/* FIXME: This code is wrong for short-range teleportation since the teleported unit is not
-		 * near its source location; consequently the unit leaves a ghost trace behind it. It does
-		 * not happen for long-range teleportation since the scrolling has the nice side effect of
-		 * cleaning up the display. -- silene */
-		disp.draw_tile(a.x,a.y);
-		for(int tile = 0; tile != 6; ++tile) {
+		disp.draw_tile(a.x, a.y);
+		for(int tile = 0; tile != 6; ++tile)
 			disp.draw_tile(src_adjacent[tile].x, src_adjacent[tile].y);
+
+		if (!teleport_unit) {
+			const int height_adjust = src_height_adjust + (dst_height_adjust - src_height_adjust) * i / nsteps;
+			const double submerge = src_submerge + (dst_submerge - src_submerge) * i / nsteps;
+			const int xpos = xloc;
+			const int ypos = yloc - height_adjust;
+			disp.draw_unit(xpos, ypos, image, false, 1.0, 0, 0.0, submerge);
+
+			if (halo_effect != 0) {
+				int d = disp.hex_size() / 2;
+				halo::set_location(halo_effect, xpos + d, ypos + d);
+			}
 		}
 
-		const int height_adjust = src_height_adjust + (dst_height_adjust - src_height_adjust) * i / nsteps;
-		const double submerge = src_submerge + (dst_submerge - src_submerge) * i / nsteps;
-
-		const int xpos = xloc;
-		const int ypos = yloc - height_adjust;
-
-		// disp.invalidate_animations();
-		disp.draw(false);
-		disp.draw_unit(xpos, ypos, image, false, 1.0, 0, 0.0, submerge);
-
-		if(halo_effect != 0)
-			halo::set_location(halo_effect, xpos+disp.hex_size()/2, ypos+disp.hex_size()/2);
-
 		const int new_ticks = SDL_GetTicks();
 		const int wait_time = time_between_frames - (new_ticks - ticks);
-		if(wait_time > 0) {
-			SDL_Delay(wait_time);
-		}
+		SDL_Delay(maximum<int>(wait_time,1));
 
 		ticks = SDL_GetTicks();
 
@@ -185,7 +182,7 @@ void move_unit_between(display& disp, const gamemap& map, const gamemap::locatio
 	gamemap::location dst_adjacent[6];
 	get_adjacent_tiles(b, dst_adjacent);
 
-	if (teleport_animation_p && !tiles_adjacent(a, b) && !disp.fogged(b.x, b.y)) { // teleport
+	if (teleport_unit && !disp.fogged(b.x, b.y)) { // teleport
 		unit_animation teleport_animation =  *teleport_animation_p;
 		int animation_time;
 		const int end_at = teleport_animation.get_last_frame_time();
@@ -302,9 +299,9 @@ void unit_die(display& disp, const gamemap::location& loc, const unit& u)
 
 namespace {
   
-bool unit_attack_ranged(display& disp, unit_map& units, const gamemap& map,
-                        const gamemap::location& a, const gamemap::location& b, int damage,
-                        const attack_type& attack)
+bool unit_attack_ranged(display& disp, unit_map& units,
+                        const gamemap::location& a, const gamemap::location& b,
+                        int damage, const attack_type& attack)
 {
 	const bool hide = disp.update_locked() || disp.fogged(a.x,a.y) && disp.fogged(b.x,b.y)
 	                  || preferences::show_combat() == false;
@@ -316,7 +313,7 @@ bool unit_attack_ranged(display& disp, unit_map& units, const gamemap& map,
 	unit_map::iterator leader = units.end();
 	if(leader_loc.valid()) {
 		leader = units.find(leader_loc);
-		assert(leader != units.end());
+		wassert(leader != units.end());
 		leader->second.set_leading(true);
 	}
 
@@ -375,9 +372,9 @@ bool unit_attack_ranged(display& disp, unit_map& units, const gamemap& map,
 	attack_anim.update_current_frames();
 	int animation_time = attack_anim.get_animation_time();
 
-	while(animation_time < end_at) {
+	def->second.set_defending(hits, attack_type::LONG_RANGE, animation_time, acceleration);
 
-		def->second.set_defending(true,hits,animation_time - missile_impact,attack_type::LONG_RANGE);
+	while(animation_time < end_at) {
 
 		//this is a while instead of an if, because there might be multiple
 		//sounds playing simultaneously or close together
@@ -560,6 +557,7 @@ bool unit_attack_ranged(display& disp, unit_map& units, const gamemap& map,
 		// ticks = SDL_GetTicks();
 
 		attack_anim.update_current_frames();
+		def->second.update_defending_frame();
 		animation_time = attack_anim.get_animation_time();
 		events::pump();
 		disp.update_display();
@@ -578,14 +576,14 @@ bool unit_attack_ranged(display& disp, unit_map& units, const gamemap& map,
 		damage = 0;
 	}
 
-	def->second.set_defending(false);
-
 	if(leader_loc.valid()){
 		leader->second.set_leading(false);
 	}
 
-	disp.draw_tile(a.x,a.y);
-	disp.draw_tile(b.x,b.y);
+	disp.invalidate(a);
+	disp.invalidate(b);
+
+	def->second.set_standing();
 
 	if(leader_loc.valid()){
 		disp.draw_tile(leader_loc.x,leader_loc.y);
@@ -595,8 +593,6 @@ bool unit_attack_ranged(display& disp, unit_map& units, const gamemap& map,
 		unit_die(disp,def->first,def->second);
 	}
 
-	disp.update_display();
-
 	return dead;
 }
 
@@ -613,8 +609,6 @@ bool unit_attack(display& disp, unit_map& units, const gamemap& map,
 		int xloc = disp.get_location_x(a);
 		int yloc = disp.get_location_y(a);
 
-		SDL_Rect area = disp.map_area();
-
 		//we try to scroll the map if the unit is at the edge.
 		//keep track of the old position, and if the map moves at all,
 		//then recenter it on the unit
@@ -626,16 +620,14 @@ bool unit_attack(display& disp, unit_map& units, const gamemap& map,
 	}
 
 	log_scope("unit_attack");
-	disp.invalidate_all();
-	disp.draw(true,true);
 
 	const unit_map::iterator att = units.find(a);
-	assert(att != units.end());
+	wassert(att != units.end());
 
 	unit& attacker = att->second;
 
 	const unit_map::iterator def = units.find(b);
-	assert(def != units.end());
+	wassert(def != units.end());
 
 	if(b.x > a.x) {
 		att->second.set_facing_left(true);
@@ -646,7 +638,7 @@ bool unit_attack(display& disp, unit_map& units, const gamemap& map,
 	}
 
 	if(attack.range() == attack_type::LONG_RANGE) {
-		return unit_attack_ranged(disp,units,map,a,b,damage,attack);
+		return unit_attack_ranged(disp, units, a, b, damage, attack);
 	}
 
 	unit_animation attack_anim = attack.animation();
@@ -662,14 +654,12 @@ bool unit_attack(display& disp, unit_map& units, const gamemap& map,
 	const int time_resolution = 20;
 	const int acceleration = disp.turbo() ? 5 : 1;
 
-	def->second.set_defending(true,attack_type::SHORT_RANGE);
-
 	const gamemap::location leader_loc = under_leadership(units,a);
 	unit_map::iterator leader = units.end();
 	if(leader_loc.valid()){
 		LOG_DP << "found leader at " << (leader_loc.x+1) << "," << (leader_loc.y+1) << "\n";
 		leader = units.find(leader_loc);
-		assert(leader != units.end());
+		wassert(leader != units.end());
 		leader->second.set_leading(true);
 	}
 
@@ -712,9 +702,10 @@ bool unit_attack(display& disp, unit_map& units, const gamemap& map,
 	attack_anim.start_animation(begin_at, unit_animation::UNIT_FRAME, acceleration);
 
 	int animation_time = attack_anim.get_animation_time();
-	while(animation_time < end_at) {
 
-		def->second.set_defending(true,hits,animation_time,attack_type::SHORT_RANGE);
+	def->second.set_defending(hits, attack_type::SHORT_RANGE, animation_time, acceleration);
+
+	while(animation_time < end_at) {
 
 		//this is a while instead of an if, because there might be multiple
 		//sounds playing simultaneously or close together
@@ -828,6 +819,7 @@ bool unit_attack(display& disp, unit_map& units, const gamemap& map,
 		ticks = SDL_GetTicks();
 
 		attack_anim.update_current_frames();
+		def->second.update_defending_frame();
 		animation_time = attack_anim.get_animation_time();
 		events::pump();
 		disp.update_display();
@@ -847,24 +839,22 @@ bool unit_attack(display& disp, unit_map& units, const gamemap& map,
 		damage = 0;
 	}
 
-	def->second.set_defending(false);
-
 	if(leader_loc.valid()){
 		leader->second.set_leading(false);
 	}
 
-	disp.draw_tile(a.x,a.y);
-	disp.draw_tile(b.x,b.y);
+	disp.invalidate(a);
+	disp.invalidate(b);
 	if(leader_loc.valid()) {
 		disp.draw_tile(leader_loc.x,leader_loc.y);
 	}
 
+	def->second.set_standing();
+
 	if(dead) {
 		unit_display::unit_die(disp,def->first,def->second);
 	}
 
-	disp.update_display();
-
 	return dead;
 }
 
