@@ -1,4 +1,4 @@
-/* $Id: unit_types.cpp,v 1.68 2004/09/29 00:19:05 Sirp Exp $ */
+/* $Id: unit_types.cpp,v 1.76 2005/02/06 10:40:12 isaaccp Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -10,15 +10,18 @@
 
    See the COPYING file for more details.
 */
+
+#include "global.hpp"
+
 #include "game_config.hpp"
 #include "gamestatus.hpp"
 #include "language.hpp"
 #include "log.hpp"
 #include "unit_types.hpp"
 #include "util.hpp"
+#include "wassert.hpp"
 
 #include <algorithm>
-#include <cassert>
 #include <cstdlib>
 #include <iostream>
 
@@ -39,6 +42,9 @@ unit_animation::frame::frame(const config& cfg)
 	halo_y = atoi(cfg["halo_y"].c_str());
 }
 
+unit_animation::unit_animation()
+{}
+
 unit_animation::unit_animation(const config& cfg) 
 {
 	config::const_child_itors range = cfg.child_range("frame");
@@ -153,7 +159,6 @@ attack_type::attack_type(const config& cfg)
 		icon_ = "attacks/" + name_ + ".png";
 
 	range_ = cfg["range"] == "long" ? LONG_RANGE : SHORT_RANGE;
-	hexes_ = maximum<int>(1,atoi(cfg["hexes"].c_str()));
 	damage_ = atol(cfg["damage"].c_str());
 	num_attacks_ = atol(cfg["number"].c_str());
 
@@ -190,11 +195,6 @@ attack_type::RANGE attack_type::range() const
 	return range_;
 }
 
-int attack_type::hexes() const
-{
-	return hexes_;
-}
-
 int attack_type::damage() const
 {
 	return damage_;
@@ -227,7 +227,7 @@ bool attack_type::slow() const
 
 const unit_animation& attack_type::animation() const
 {
-	assert(animation_.empty() == false);
+	wassert(animation_.empty() == false);
 	return animation_[rand()%animation_.size()];
 }
 
@@ -369,15 +369,14 @@ int unit_movement_type::movement_cost(const gamemap& map,gamemap::TERRAIN terrai
 	int res = -1;
 
 	if(movement_costs != NULL) {
-		const std::vector<std::string> names = map.underlying_terrain_name(terrain);
-		if(names.size() != 1) {
-			lg::err(lg::config) << "terrain '" << terrain << "' has " << names.size() << " underlying names - 0 expected\n";
+		if(underlying.size() != 1) {
+			lg::err(lg::config) << "terrain '" << terrain << "' has " << underlying.size() << " underlying names - 0 expected\n";
 			return impassable;
 		}
 
-		const std::string& name = names.front();
+		const std::string& id = map.get_terrain_info(underlying[0]).id();
 
-		const std::string& val = (*movement_costs)[name];
+		const std::string& val = (*movement_costs)[id];
 
 		if(val != "") {
 			res = atoi(val.c_str());
@@ -429,14 +428,13 @@ int unit_movement_type::defense_modifier(const gamemap& map,gamemap::TERRAIN ter
 	const config* const defense = cfg_.child("defense");
 
 	if(defense != NULL) {
-		const std::vector<std::string> names = map.underlying_terrain_name(terrain);
-		if(names.size() != 1) {
-			lg::err(lg::config) << "terrain '" << terrain << "' has " << names.size() << " underlying names - 0 expected\n";
+		if(underlying.size() != 1) {
+			lg::err(lg::config) << "terrain '" << terrain << "' has " << underlying.size() << " underlying names - 0 expected\n";
 			return 100;
 		}
 
-		const std::string& name = names.front();
-		const std::string& val = (*defense)[name];
+		const std::string& id = map.get_terrain_info(underlying[0]).id();
+		const std::string& val = (*defense)[id];
 
 		if(val != "") {
 			res = atoi(val.c_str());
@@ -844,7 +842,7 @@ unit_type::experience_accelerator::~experience_accelerator()
 
 int unit_type::experience_needed() const
 {
-	return (atoi(cfg_["experience"].c_str())*experience_modifier)/100;
+	return (lexical_cast_default<int>(cfg_["experience"],500)*experience_modifier)/100;
 }
 
 std::vector<std::string> unit_type::advances_to() const
@@ -963,6 +961,11 @@ bool unit_type::can_advance() const
 	return can_advance_;
 }
 
+bool unit_type::has_zoc() const
+{
+	return level() > 0;
+}
+
 bool unit_type::has_ability(const std::string& ability) const
 {
 	return std::find(abilities_.begin(),abilities_.end(),ability) != abilities_.end();
