@@ -1,4 +1,4 @@
-/* $Id: unit_types.hpp,v 1.53 2004/09/26 08:20:01 silene Exp $ */
+/* $Id: unit_types.hpp,v 1.56 2005/02/06 10:40:12 isaaccp Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -40,6 +40,7 @@ public:
 		int halo_x, halo_y;
 	};
 
+	unit_animation();
 	explicit unit_animation(const config& cfg);
 
 	enum FRAME_TYPE { UNIT_FRAME, MISSILE_FRAME };
@@ -82,7 +83,6 @@ public:
 	const std::string& special() const;
 	const std::string& icon() const;
 	RANGE range() const;
-	int hexes() const;
 	int damage() const;
 	int num_attacks() const;
 	double attack_weight() const;
@@ -231,6 +231,8 @@ public:
 	bool not_living() const;
 	bool can_advance() const;
 
+	bool has_zoc() const;
+
 	bool has_ability(const std::string& ability) const;
 
 	const std::vector<config*>& possible_traits() const;
