@@ -1,4 +1,4 @@
-/* $Id: util.hpp,v 1.18 2004/08/28 21:16:17 gruikya Exp $ */
+/* $Id: util.hpp,v 1.20 2005/01/19 21:40:37 silene Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -32,20 +32,14 @@ template<typename T>
 inline const T& maximum(const T& a, const T& b) { return a < b ? b : a; }
 
 template<typename T>
-inline bool is_odd(T num) { return (static_cast<unsigned int>(num > 0 ? num : -num)&1) == 1; }
+inline bool is_odd(T num) {
+  int n = static_cast< int >(num);
+  return static_cast< unsigned int >(n >= 0 ? n : -n) & 1;
+}
 
 template<typename T>
 inline bool is_even(T num) { return !is_odd(num); }
 
-//place in our own namespace as to not clash with possible
-//standard library definitions
-namespace util {
-
-template<typename T>
-T round(T n) { return floor(n + 0.5); }
-
-}
-
 struct bad_lexical_cast {};
 
 template<typename To, typename From>
