@@ -4,10 +4,12 @@
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #else
-# define VERSION "0.8.5"
-# define WESNOTH_DEFAULT_SERVER "devsrv.wesnoth.org"
+# define VERSION "0.8.10"
+# define WESNOTH_DEFAULT_SERVER "server.wesnoth.org"
 # define PACKAGE "wesnoth"
-# define LOCALEDIR "messages"
+# ifndef LOCALEDIR
+#  define LOCALEDIR "translations"
+# endif
 #endif
 
 
