@@ -10,10 +10,13 @@
    See the COPYING file for more details.
 */
 
-#include "../font.hpp"
-#include "../events.hpp"
-#include "../language.hpp"
+#include "../global.hpp"
+
 #include "../display.hpp"
+#include "../events.hpp"
+#include "../filesystem.hpp"
+#include "../font.hpp"
+#include "../language.hpp"
 #include "../show_dialog.hpp"
 #include "file_chooser.hpp"
 
@@ -27,10 +30,11 @@ namespace {
 namespace gui {
 
 file_chooser::file_chooser(display &disp, std::string start_file) 
-  : widget(disp), disp_(disp), delete_button_(disp, _("Delete File")),
+	: widget(disp), delete_button_(disp, _("Delete File")),
 	  path_delim_('/'), current_dir_(get_path(start_file)),
 	  chosen_file_(start_file), file_list_(disp, files_in_current_dir_, false),
-	  filename_textbox_(disp, 100, start_file, true), choice_made_(false),
+	  filename_textbox_(disp, 100, start_file, true),
+	  current_path_label_(disp, current_dir_), choice_made_(false),
 	  last_selection_(-1) {
 	// If the start file is not a file or directory, use the root.
 	if(!file_exists(chosen_file_) && !is_directory(chosen_file_)
@@ -39,40 +43,12 @@ file_chooser::file_chooser(display &disp, std::string start_file)
 		chosen_file_ = current_dir_;
 	}
 	// Set sizes to some default values.
-	set_location(1, 1);
-	set_width(400);
-	set_height(500);
+	set_measurements(400, 500);
 	update_file_lists();
-}
-
-void file_chooser::adjust_layout() {
-	const int current_path_y = location().y;
-	current_path_rect_.y = current_path_y;
-	current_path_rect_.w = width();
-	current_path_rect_.x = location().x;
-	current_path_rect_.h = 18;
-	const int file_list_y = current_path_y + current_path_rect_.h + 10;
-	const int filename_textbox_y = location().y + height() - filename_textbox_.height();
-	const int file_list_height = filename_textbox_y  - file_list_y - 10 - 32;
-
-	const int delete_button_x = location().x + width() - delete_button_.width();
-	const int delete_button_y = file_list_y + file_list_height + 5;
-	delete_button_.set_location(delete_button_x, delete_button_y);
-
-	file_list_.set_width(width());
-	filename_textbox_.set_width(width());
-
-	file_list_.set_loc(location().x, file_list_y);
-	filename_textbox_.set_location(location().x, filename_textbox_y);
-
-	file_list_.set_max_height(file_list_height);
-	// When the layout has changed we want to redisplay the files to
-	// make them fit into the newly adjusted widget.
-	set_dirty(true);
+	display_chosen_file();
 }
 
 void file_chooser::display_current_files() {
-	bg_restore();
 	std::vector<std::string> to_show;
 	if (!is_root(current_dir_)) {
 		to_show.push_back("..");
@@ -83,11 +59,11 @@ void file_chooser::display_current_files() {
 	for (it = to_show.begin(); it != to_show.end(); it++) {
 		// Add an image to show that these are directories.
 		std::stringstream ss;
-		ss << font::IMAGE << dir_picture << ',' << *it;
+		ss << font::IMAGE << dir_picture << COLUMN_SEPARATOR << *it;
 		*it = ss.str();
 	}
 	for (it = files_in_current_dir_.begin(); it != files_in_current_dir_.end(); it++) {
-		const std::string display_string = std::string(" ,") + *it;
+		const std::string display_string = COLUMN_SEPARATOR + *it;
 		to_show.push_back(display_string);
 	}
 	const int menu_font_size = font::SIZE_NORMAL; // Known from menu.cpp.
@@ -100,14 +76,10 @@ void file_chooser::display_current_files() {
 		}
 	}
 	file_list_.set_items(to_show);
-	// This will prevent the "box" with filenames from changing size on
-	// every redisplay, it looks better when it's static.
-	file_list_.set_width(width());  
+	current_path_label_.set_text(current_dir_);
 }
 
 void file_chooser::display_chosen_file() {
-	// Clearing is not really necessary, but things end up nicer if we do.
-	filename_textbox_.clear(); 
 	if (is_directory(chosen_file_)) {
 		filename_textbox_.set_text(strip_last_delim(chosen_file_) + path_delim_);
 	}
@@ -116,28 +88,17 @@ void file_chooser::display_chosen_file() {
 	}
 }
 
-void file_chooser::draw() {
-	if (!dirty()) {
-		return;
-	}
-	display_current_files();
-	display_chosen_file();
-	font::draw_text(&disp_, current_path_rect_, font::SIZE_NORMAL, font::NORMAL_COLOUR,
-					current_dir_, current_path_rect_.x, current_path_rect_.y,
-					disp_.video().getSurface());
-	set_dirty(false);
-}
-
-void file_chooser::process() {
+void file_chooser::process_event() {
 	CKey key;
 	int mousex, mousey;
-	const int mouse_flags = SDL_GetMouseState(&mousex,&mousey);
+	SDL_GetMouseState(&mousex,&mousey);
+	
 	// The menu does not implement focus functionality, so we fake
 	// it. We give the file list focus whenever the filename textbox
 	// does not have focus. Inflexible but easy solution.
-	if (!(mousex > location().x && (unsigned)mousex < location().x + width()
-		&& mousey > location().y 
-		&& (unsigned)mousey < location().y + height() - filename_textbox_.height())) {
+	SDL_Rect const &loc = location();
+	if (!(mousex > loc.x && (unsigned)mousex < loc.x + loc.w && mousey > loc.y 
+		&& (unsigned)mousey < loc.y + loc.h - filename_textbox_.height())) {
 		// Hmm, as I understand it this should happen automatically when
 		// the mouse is in the textbox again. However this is not the
 		// case so this is done explicitly here.
@@ -147,15 +108,7 @@ void file_chooser::process() {
 		filename_textbox_.set_focus(false);
 	}
 	if (!filename_textbox_.focus()) {
-		const bool new_left_button = mouse_flags&SDL_BUTTON_LMASK;
-		
-		const bool new_up_arrow = key[SDLK_UP];
-		const bool new_down_arrow = key[SDLK_DOWN];
-		
-		const bool new_page_up = key[SDLK_PAGEUP];
-		const bool new_page_down = key[SDLK_PAGEDOWN];
-		file_list_.process(mousex, mousey, new_left_button, new_up_arrow,
-						   new_down_arrow, new_page_up, new_page_down, -1);
+		file_list_.process();
 		const int new_selection = file_list_.selection();
 		const bool double_click = file_list_.double_clicked();
 		if (double_click && new_selection >= 0) {
@@ -170,12 +123,12 @@ void file_chooser::process() {
 	if (delete_button_.pressed()) {
 		const int ret = remove(get_current_file().c_str());
 		if (ret == -1) {
-			show_dialog(disp_, NULL, "", _("Deletion of the file failed."), OK_ONLY);
+			show_dialog(disp(), NULL, "", _("Deletion of the file failed."), OK_ONLY);
 		}
 		else {
 			update_file_lists();
 			chosen_file_ = current_dir_;
-			set_dirty(true);
+			display_chosen_file();
 		}
 	}
 }
@@ -232,7 +185,7 @@ void file_chooser::entry_chosen(const unsigned entry) {
 			choice_made_ = true;
 		}
 	}
-	set_dirty(true);
+	display_chosen_file();
 }
 
 std::string file_chooser::get_current_file() const {
@@ -255,34 +208,24 @@ std::string file_chooser::get_choice() const {
 	return get_current_file();
 }
 
-void file_chooser::set_dirty(bool dirty) {
-	widget::set_dirty(dirty);
-	if (dirty) {
-		// These will set themselves to false when they are done
-		// drawing.
-		filename_textbox_.set_dirty(true);
-		delete_button_.set_dirty(true);
-	}
-}
+void file_chooser::update_location(SDL_Rect const &rect) {
+	const int file_list_y = rect.y + current_path_label_.height() + 10;
+	const int filename_textbox_y = rect.y + rect.h - filename_textbox_.height();
+	const int file_list_height = filename_textbox_y  - file_list_y - 10 - 32;
 
-void file_chooser::set_location(const SDL_Rect& rect) {
-	widget::set_location(rect);
-	adjust_layout();
-}
+	const int delete_button_x = rect.x + rect.w - delete_button_.width();
+	const int delete_button_y = file_list_y + file_list_height + 5;
+	delete_button_.set_location(delete_button_x, delete_button_y);
 
-void file_chooser::set_location(int x, int y) {
-	widget::set_location(x, y);
-	adjust_layout();
-}
+	current_path_label_.set_width(rect.w);
+	file_list_.set_width(rect.w);
+	filename_textbox_.set_width(rect.w);
 
-void file_chooser::set_width(int w) {
-	widget::set_width(w);
-	adjust_layout();
-}
+	current_path_label_.set_location(rect.x, rect.y);
+	file_list_.set_location(rect.x, file_list_y);
+	filename_textbox_.set_location(rect.x, filename_textbox_y);
 
-void file_chooser::set_height(int h) {
-	widget::set_height(h);
-	adjust_layout();
+	file_list_.set_max_height(file_list_height);
 }
 
 std::string file_chooser::get_path(const std::string file_or_dir) const {
@@ -347,7 +290,8 @@ void file_chooser::update_file_lists() {
 	files_in_current_dir_.clear();
 	dirs_in_current_dir_.clear();
 	get_files_in_dir(current_dir_, &files_in_current_dir_,
-					 &dirs_in_current_dir_, FILE_NAME_ONLY); 
+	                 &dirs_in_current_dir_, FILE_NAME_ONLY);
+	display_current_files();
 }
 
 void file_chooser::handle_event(const SDL_Event& event) {
