@@ -1,4 +1,4 @@
-/* $Id: textbox.cpp,v 1.56 2004/09/27 00:24:42 ydirson Exp $ */
+/* $Id: textbox.cpp,v 1.68 2005/01/23 00:28:34 Sirp Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -10,6 +10,9 @@
 
    See the COPYING file for more details.
 */
+
+#include "../global.hpp"
+
 #include "textbox.hpp"
 #include "../clipboard.hpp"
 #include "../font.hpp"
@@ -28,23 +31,25 @@ namespace gui {
 
 const int font_size = font::SIZE_PLUS;
 
-textbox::textbox(display& d, int width, const std::string& text, bool editable, size_t max_size)
-	   : widget(d), max_size_(max_size), text_(string_to_wstring(text)),
+textbox::textbox(display& d, int width, const std::string& text, bool editable, size_t max_size, double alpha, double alpha_focus)
+	   : scrollarea(d), max_size_(max_size), text_(string_to_wstring(text)),
 	     cursor_(text_.size()), selstart_(-1), selend_(-1),
 	     grabmouse_(false), text_pos_(0), editable_(editable),
 	     show_cursor_(true), show_cursor_at_(0), text_image_(NULL),
-	     scrollbar_(d,this),
-	     uparrow_(d,"",gui::button::TYPE_PRESS,"uparrow-button"),
-             downarrow_(d,"",gui::button::TYPE_PRESS,"downarrow-button"),
-	     scroll_bottom_(false), wrap_(false), line_height_(0), yscroll_(0)
+	     wrap_(false), line_height_(0), yscroll_(0), alpha_(alpha),
+		  alpha_focus_(alpha_focus)
 {
-	static const SDL_Rect area = d.screen_area();
-	const int height = font::draw_text(NULL,area,font_size,font::NORMAL_COLOUR,"ABCD",0,0).h;
-	const SDL_Rect starting_rect = {0,0,width,height};
-	set_location(starting_rect);
+	// static const SDL_Rect area = d.screen_area();
+	// const int height = font::draw_text(NULL,area,font_size,font::NORMAL_COLOUR,"ABCD",0,0).h;
+	set_measurements(width, font::get_max_height(font_size));
 	update_text_cache(true);
 }
 
+void textbox::set_inner_location(SDL_Rect const &rect)
+{
+	bg_register(rect);
+}
+
 const std::string textbox::text() const
 {
 	const std::string &ret = wstring_to_string(text_);
@@ -52,16 +57,45 @@ const std::string textbox::text() const
 }
 
 // set_text does not respect max_size_
-void textbox::set_text(std::string text)
+void textbox::set_text(const std::string& text)
 {
 	text_ = string_to_wstring(text);
 	cursor_ = text_.size();
+	text_pos_ = 0;
 	selstart_ = -1;
 	selend_ = -1;
 	set_dirty(true);
 	update_text_cache(true);
 }
 
+void textbox::append_text(const std::string& text)
+{
+	if(text_image_.get() == NULL) {
+		set_text(text);
+		return;
+	}
+
+	const wide_string& wtext = string_to_wstring(text);
+
+	const surface new_text = add_text_line(wtext);
+	const surface new_surface = create_compatible_surface(text_image_,maximum<size_t>(text_image_->w,new_text->w),text_image_->h+new_text->h);
+
+	SDL_SetAlpha(new_text.get(),0,0);
+	SDL_SetAlpha(text_image_.get(),0,0);
+
+	SDL_BlitSurface(text_image_,NULL,new_surface,NULL);
+
+	SDL_Rect target = {0,text_image_->h,new_text->w,new_text->h};
+	SDL_BlitSurface(new_text,NULL,new_surface,&target);
+	text_image_.assign(new_surface);
+
+	text_.resize(text_.size() + wtext.size());
+	std::copy(wtext.begin(),wtext.end(),text_.end()-wtext.size());
+
+	set_dirty(true);
+	update_text_cache(false);
+}
+
 void textbox::clear()
 {
 	text_.clear();
@@ -83,30 +117,18 @@ void textbox::draw_cursor(int pos, display &disp) const
 	}
 }
 
-void textbox::draw()
+void textbox::draw_contents()
 {
-	if(location().x == 0 || !dirty()) {
-		uparrow_.draw();
-		downarrow_.draw();
-		return;
-	}
-
-	bg_restore();
-
-	const bool has_scrollbar = show_scrollbar();
-	SDL_Rect loc = location();
-	if(has_scrollbar && loc.w > scrollbar_.get_max_width()) {
-		scrollbar_.set_dirty();
-		loc.w -= scrollbar_.get_max_width();
-	}
+	SDL_Rect const &loc = inner_location();
 
+	surface surf = disp().video().getSurface();
 	gui::draw_solid_tinted_rectangle(loc.x,loc.y,loc.w,loc.h,0,0,0,
-	                          focus() ? 0.2 : 0.4, disp().video().getSurface());
+	                          focus() ? alpha_focus_ : alpha_, surf);
 	
 	SDL_Rect src;
 
 	if(text_image_ != NULL) {
-		src.y = 0;
+		src.y = yscroll_;
 		src.w = minimum<size_t>(loc.w,text_image_->w);
 		src.h = minimum<size_t>(loc.h,text_image_->h);
 		src.x = text_pos_;
@@ -114,52 +136,6 @@ void textbox::draw()
 		dest.x = loc.x;
 		dest.y = loc.y;
 
-		scrollbar_.enable(has_scrollbar);
-
-		if(has_scrollbar && text_image_->h > 0 && loc.h > uparrow_.height() + downarrow_.height()) {
-			SDL_Rect scroll_loc = {loc.x + loc.w,loc.y + uparrow_.height(),
-			                       scrollbar_.get_max_width(),loc.h - uparrow_.height() - downarrow_.height()};
-			scrollbar_.set_location(scroll_loc);
-
-			uparrow_.set_location(loc.x + loc.w,loc.y);
-			downarrow_.set_location(loc.x + loc.w,loc.y + loc.h - downarrow_.height());
-
-			const size_t max_height = scrollbar_.location().h;
-			const size_t proportion = (loc.h*100)/text_image_->h;
-			const size_t grip_height = maximum<size_t>((max_height*proportion)/100,scrollbar_.get_minimum_grip_height());
-			scrollbar_.set_grip_height(grip_height);
-
-			const size_t max_y = text_image_->h - loc.h;
-
-			const size_t max_grip_y = scrollbar_.location().h - grip_height;
-
-			if(scroll_bottom_) {
-				scrollbar_.set_grip_position(max_grip_y);
-			}
-
-			if(max_grip_y > 0) {
-				const size_t grip_y = scrollbar_.get_grip_position();
-
-				uparrow_.hide(grip_y == 0);
-				downarrow_.hide(grip_y == max_grip_y);
-
-				src.y = (max_y*(grip_y*100)/max_grip_y)/100;
-				lg::info(lg::display) << "set src.y to " << src.y << "/" << max_grip_y << "\n";
-			}
-
-			uparrow_.set_dirty(true);
-			downarrow_.set_dirty(true);
-			uparrow_.draw();
-			downarrow_.draw();
-
-			scrollbar_.redraw();
-		} else {
-			uparrow_.hide();
-			downarrow_.hide();
-		}
-
-		scroll_bottom_ = false;
-
 		// Fills the selected area
 		if(is_selection()) {
 			const int start = minimum<int>(selstart_,selend_);
@@ -175,43 +151,28 @@ void textbox::draw()
 					break;
 				}
 
-				SDL_Rect rect = {location().x + startx,location().y + starty - src.y,right - startx,line_height_};
+				SDL_Rect rect = { loc.x + startx, loc.y + starty - src.y, right - startx, line_height_ };
 
-				SDL_Rect clip = location();
-				const clip_rect_setter clipper(disp().video().getSurface(),clip);
+				const clip_rect_setter clipper(surf, loc);
 
-				Uint32 colour = SDL_MapRGB(disp().video().getSurface()->format, 160, 0, 0);
-				fill_rect_alpha(rect,colour,140,disp().video().getSurface());
+				Uint32 colour = SDL_MapRGB(surf->format, 160, 0, 0);
+				fill_rect_alpha(rect, colour, 140, surf);
 
 				starty += int(line_height_);
 				startx = 0;
 			}
 		}
 
-		yscroll_ = src.y;
-		SDL_BlitSurface(text_image_,&src,disp().video().getSurface(),&dest);
+		SDL_BlitSurface(text_image_, &src, surf, &dest);
 	}
 
 	draw_cursor((cursor_pos_ == 0 ? 0 : cursor_pos_ - 1), disp());
 
-	set_dirty(false);
 	update_rect(loc);
 }
 
 void textbox::process()
 {
-	if(show_scrollbar()) {
-		if(uparrow_.pressed()) {
-			scrollbar_.set_grip_position(scrollbar_.get_grip_position() - scrollbar_.get_grip_height()/5);
-			set_dirty(true);
-		}
-
-		if(downarrow_.pressed()) {
-			scrollbar_.set_grip_position(scrollbar_.get_grip_position() + scrollbar_.get_grip_height()/5);
-			set_dirty(true);
-		}
-	}
-
 	if(editable_) {
 		if(focus()) {
 			const int ticks = SDL_GetTicks();
@@ -225,7 +186,7 @@ void textbox::process()
 			set_dirty();
 		}
 	}
-	
+
 	draw();
 }
 
@@ -241,8 +202,7 @@ bool textbox::editable() const
 
 void textbox::scroll_to_bottom()
 {
-	scroll_bottom_ = true;
-	set_dirty(true);
+	set_position(get_max_position());
 }
 
 void textbox::set_wrap(bool val)
@@ -256,78 +216,83 @@ void textbox::set_wrap(bool val)
 
 void textbox::scroll(int pos)
 {
+	yscroll_ = pos;
 	set_dirty(true);
 }
 
+surface textbox::add_text_line(const wide_string& text)
+{
+	line_height_ = font::get_max_height(font_size);
+
+	if(char_y_.empty()) {
+		char_y_.push_back(0);
+	} else {
+		char_y_.push_back(char_y_.back() + line_height_);
+	}
+
+	char_x_.push_back(0);
+	
+	// Re-calculate the position of each glyph. We approximate this by asking the
+	// width of each substring, but this is a flawed assumption which won't work with
+	// some more complex scripts (that is, RTL languages). This part of the work should
+	// actually be done by the font-rendering system.
+	std::string visible_string;
+	wide_string wrapped_text;
+	 
+	wide_string::const_iterator backup_itor = text.end();
+	
+	wide_string::const_iterator itor = text.begin();
+	while(itor != text.end()) {
+		//If this is a space, save copies of the current state so we can roll back
+		if(char(*itor) == ' ') {
+			backup_itor = itor;
+		}
+		visible_string.append(wchar_to_string(*itor));
+
+		if(char(*itor) == '\n') {
+			backup_itor = text.end();
+			visible_string = "";
+		}
+
+		int w = font::line_width(visible_string, font_size);
+
+		if(wrap_ && w >= inner_location().w) {
+			if(backup_itor != text.end()) {
+				int backup = itor - backup_itor;
+				itor = backup_itor + 1;
+				if(backup > 0) {
+					char_x_.erase(char_x_.end()-backup, char_x_.end());
+					char_y_.erase(char_y_.end()-backup, char_y_.end());
+					wrapped_text.erase(wrapped_text.end()-backup, wrapped_text.end());
+				}
+			}
+			backup_itor = text.end();
+			wrapped_text.push_back(wchar_t('\n'));
+			char_x_.push_back(0);
+			char_y_.push_back(char_y_.back() + line_height_);
+			visible_string = "";
+		} else {
+			wrapped_text.push_back(*itor);
+			char_x_.push_back(w);
+			char_y_.push_back(char_y_.back() + (char(*itor) == '\n' ? line_height_ : 0));
+			++itor;
+		}
+	}
+
+	const std::string s = wstring_to_string(wrapped_text);
+	const surface res(font::get_rendered_text(s, font_size, font::NORMAL_COLOUR));		
+
+	return res;
+}
+
+
 void textbox::update_text_cache(bool changed)
 {
 	if(changed) {
 		char_x_.clear();
 		char_y_.clear();
-		char_x_.push_back(0);
-		char_y_.push_back(0);
 
-		// Re-calculate the position of each glyph. We approximate this by asking the
-		// width of each substring, but this is a flawed assumption which won't work with
-		// some more complex scripts (that is, RTL languages). This part of the work should
-		// actually be done by the font-rendering system.
-		std::string visible_string;
-		wide_string wrapped_text;
-		 
-		wide_string::const_iterator backup_itor;
-		
-		wide_string::const_iterator itor = text_.begin();
-		while(itor != text_.end()) {
-			//If this is a space, save copies of the current state so we can roll back
-			if(char(*itor) == ' ') {
-				backup_itor = itor;
-			}
-			visible_string.append(wchar_to_string(*itor));
-
-			if(char(*itor) == '\n') {
-				backup_itor = text_.end();
-				visible_string = "";
-			}
-
-			int w = font::line_width(visible_string, font_size);
-
-			if(wrap_ && w >= location().w - scrollbar_.get_max_width()) {
-				if(backup_itor != text_.end()) {
-					int backup = itor - backup_itor;
-					itor = backup_itor + 1;
-					if(backup > 0) {
-						char_x_.erase(char_x_.end()-backup, char_x_.end());
-						char_y_.erase(char_y_.end()-backup, char_y_.end());
-						wrapped_text.erase(wrapped_text.end()-backup, wrapped_text.end());
-					}
-				}
-				backup_itor = text_.end();
-				wrapped_text.push_back(wchar_t('\n'));
-				char_x_.push_back(0);
-				char_y_.push_back(char_y_.back()+1);
-				visible_string = "";
-			} else {
-				wrapped_text.push_back(*itor);
-				char_x_.push_back(w);
-				char_y_.push_back(char_y_.back() + (char(*itor) == '\n' ? 1 : 0));
-				++itor;
-			}
-		}
-
-		text_size_.x = 0;
-		text_size_.y = 0;
-		const std::string s = wstring_to_string(wrapped_text);
-		text_size_.w = font::line_width(s, font_size);
-		text_size_.h = location().h;
-
-		text_image_.assign(font::get_rendered_text(s, font_size, font::NORMAL_COLOUR));		
-
-		//so far we've set char_y_ in terms of the line it's on, now set it in terms of proper y
-		//co-ordinates, by calculating the height of a line, and multiplying each member of char_y_ by that height
-		line_height_ = font::get_max_height(font_size);
-		for(std::vector<int>::iterator i = char_y_.begin(); i != char_y_.end(); ++i) {
-			*i = *i * line_height_;
-		}
+		text_image_.assign(add_text_line(text_));
 	}
 
 	int cursor_x = char_x_[cursor_];
@@ -338,6 +303,11 @@ void textbox::update_text_cache(bool changed)
 		text_pos_ = cursor_x;
 	}
 	cursor_pos_ = cursor_x - text_pos_;
+
+	if (!text_image_.null()) {
+		set_full_size(text_image_->h);
+		set_shown_size(location().h);
+	}
 }
 
 bool textbox::is_selection() 
@@ -373,13 +343,13 @@ void textbox::handle_event(const SDL_Event& event)
 		grabmouse_ = false;
 	}
 
-	if( (grabmouse_ && (event.type == SDL_MOUSEMOTION)) ||  (
-		    event.type == SDL_MOUSEBUTTONDOWN  && (mousebuttons & SDL_BUTTON(1))  && ! 
-			(mousex < location().x || mousex > location().x + location().w - (show_scrollbar() ? scrollbar_.get_max_width() : 0) ||
-		    mousey < location().y || mousey > location().y + location().h))) {
+	SDL_Rect const &loc = inner_location();
 
-		const int x = mousex - location().x + text_pos_;
-		const int y = mousey - location().y;
+	if ((grabmouse_ && (event.type == SDL_MOUSEMOTION)) ||
+	    (event.type == SDL_MOUSEBUTTONDOWN && (mousebuttons & SDL_BUTTON(1)) &&
+	     point_in_rect(mousex, mousey, loc))) {
+		const int x = mousex - loc.x + text_pos_;
+		const int y = mousey - loc.y;
 		int pos = 0;
 		int distance = x;
 
@@ -420,11 +390,8 @@ void textbox::handle_event(const SDL_Event& event)
 	//if we don't have the focus, then see if we gain the focus,
 	//otherwise return
 	if(focus() == false) {
-		if(event.type == SDL_MOUSEMOTION &&
-		   mousex >= location().x && mousey >= location().y &&
-		   mousex < location().x + location().w && mousey < location().y + location().h) {
+		if (event.type == SDL_MOUSEMOTION && point_in_rect(mousex, mousey, loc))
 			events::focus_handler(this);
-		}
 		   
 		return;
 	}
@@ -548,9 +515,4 @@ void textbox::handle_event(const SDL_Event& event)
 	draw();
 }
 
-bool textbox::show_scrollbar() const
-{
-	return text_image_ != NULL && text_image_->h > location().h;
-}
-
 } //end namespace gui
